@@ -128,6 +128,10 @@ void QQuickFolderListModelPrivate::_q_directoryChanged(const QString &directory,
     qCDebug(lcFolderListModel) << "_q_directoryChanged called with directory" << directory;
     Q_Q(QQuickFolderListModel);
     Q_UNUSED(directory);
+    if (!resettingModel) {
+        resettingModel = true;
+        q->beginResetModel();
+    }
 
     data = list;
     q->endResetModel();
@@ -254,7 +258,6 @@ QString QQuickFolderListModelPrivate::resolvePath(const QUrl &path)
     \list
     \li \c fileName (\c string)
     \li \c filePath (\c string)
-    \li \c fileURL (\c url) (since Qt 5.2; deprecated since Qt 5.15)
     \li \c fileUrl (\c url) (since Qt 5.15)
     \li \c fileBaseName (\c string)
     \li \c fileSuffix (\c string)
@@ -336,7 +339,6 @@ QQuickFolderListModel::QQuickFolderListModel(QObject *parent)
     d->roleNames[FileLastReadRole] = "fileAccessed";
     d->roleNames[FileIsDirRole] = "fileIsDir";
     d->roleNames[FileUrlRole] = "fileUrl";
-    d->roleNames[FileURLRole] = "fileURL";
     d->init();
 }
 
@@ -349,38 +351,38 @@ QVariant QQuickFolderListModel::data(const QModelIndex &index, int role) const
     Q_D(const QQuickFolderListModel);
     QVariant rv;
 
-    if (index.row() >= d->data.size())
+    const int row = index.row();
+    if (row < 0 || row >= d->data.size())
         return rv;
 
     switch (role)
     {
         case FileNameRole:
-            rv = d->data.at(index.row()).fileName();
+            rv = d->data.at(row).fileName();
             break;
         case FilePathRole:
-            rv = d->data.at(index.row()).filePath();
+            rv = d->data.at(row).filePath();
             break;
         case FileBaseNameRole:
-            rv = d->data.at(index.row()).baseName();
+            rv = d->data.at(row).baseName();
             break;
         case FileSuffixRole:
-            rv = d->data.at(index.row()).suffix();
+            rv = d->data.at(row).suffix();
             break;
         case FileSizeRole:
-            rv = d->data.at(index.row()).size();
+            rv = d->data.at(row).size();
             break;
         case FileLastModifiedRole:
-            rv = d->data.at(index.row()).lastModified();
+            rv = d->data.at(row).lastModified();
             break;
         case FileLastReadRole:
-            rv = d->data.at(index.row()).lastRead();
+            rv = d->data.at(row).lastRead();
             break;
         case FileIsDirRole:
-            rv = d->data.at(index.row()).isDir();
+            rv = d->data.at(row).isDir();
             break;
         case FileUrlRole:
-        case FileURLRole:
-            rv = QUrl::fromLocalFile(d->data.at(index.row()).filePath());
+            rv = QUrl::fromLocalFile(d->data.at(row).filePath());
             break;
         default:
             break;
@@ -493,9 +495,11 @@ void QQuickFolderListModel::setRootFolder(const QUrl &path)
     QFileInfo info(resolvedPath);
     if (!info.exists() || !info.isDir())
         return;
-
-    d->fileInfoThread.setRootPath(resolvedPath);
-    d->rootDir = path;
+    if (path != d->rootDir) {
+        d->fileInfoThread.setRootPath(resolvedPath);
+        d->rootDir = path;
+        emit rootFolderChanged();
+    }
 }
 
 
@@ -554,6 +558,7 @@ void QQuickFolderListModel::setNameFilters(const QStringList &filters)
         return;
     d->fileInfoThread.setNameFilters(filters);
     d->nameFilters = filters;
+    emit nameFilterChanged();
 }
 
 void QQuickFolderListModel::classBegin()
@@ -592,9 +597,11 @@ QQuickFolderListModel::SortField QQuickFolderListModel::sortField() const
 void QQuickFolderListModel::setSortField(SortField field)
 {
     Q_D(QQuickFolderListModel);
+
     if (field != d->sortField) {
         d->sortField = field;
         d->updateSorting();
+        emit sortFieldChanged();
     }
 }
 
@@ -624,6 +631,7 @@ void QQuickFolderListModel::setSortReversed(bool rev)
     if (rev != d->sortReversed) {
         d->sortReversed = rev;
         d->updateSorting();
+        emit sortReversedChanged();
     }
 }
 
@@ -667,8 +675,11 @@ void QQuickFolderListModel::setShowFiles(bool on)
 {
     Q_D(QQuickFolderListModel);
 
-    d->fileInfoThread.setShowFiles(on);
-    d->showFiles = on;
+    if (on != d->showFiles) {
+        d->fileInfoThread.setShowFiles(on);
+        d->showFiles = on;
+        emit showFilesChanged();
+    }
 }
 
 /*!
@@ -693,8 +704,11 @@ void  QQuickFolderListModel::setShowDirs(bool on)
 {
     Q_D(QQuickFolderListModel);
 
-    d->fileInfoThread.setShowDirs(on);
-    d->showDirs = on;
+    if (on != d->showDirs) {
+        d->fileInfoThread.setShowDirs(on);
+        d->showDirs = on;
+        emit showDirsChanged();
+    }
 }
 
 /*!
@@ -716,8 +730,11 @@ void  QQuickFolderListModel::setShowDirsFirst(bool on)
 {
     Q_D(QQuickFolderListModel);
 
-    d->fileInfoThread.setShowDirsFirst(on);
-    d->showDirsFirst = on;
+    if (on != d->showDirsFirst) {
+       d->fileInfoThread.setShowDirsFirst(on);
+       d->showDirsFirst = on;
+       emit showDirsFirstChanged();
+    }
 }
 
 
@@ -744,6 +761,7 @@ void  QQuickFolderListModel::setShowDotAndDotDot(bool on)
     if (on != d->showDotAndDotDot) {
         d->fileInfoThread.setShowDotAndDotDot(on);
         d->showDotAndDotDot = on;
+        emit showDotAndDotDotChanged();
     }
 }
 
@@ -770,6 +788,7 @@ void QQuickFolderListModel::setShowHidden(bool on)
     if (on != d->showHidden) {
         d->fileInfoThread.setShowHidden(on);
         d->showHidden = on;
+        emit showHiddenChanged();
     }
 }
 
@@ -796,6 +815,7 @@ void QQuickFolderListModel::setShowOnlyReadable(bool on)
     if (on != d->showOnlyReadable) {
         d->fileInfoThread.setShowOnlyReadable(on);
         d->showOnlyReadable = on;
+        emit showOnlyReadableChanged();
     }
 }
 
@@ -821,6 +841,7 @@ void QQuickFolderListModel::setCaseSensitive(bool on)
     if (on != d->caseSensitive) {
         d->fileInfoThread.setCaseSensitive(on);
         d->caseSensitive = on;
+        emit caseSensitiveChanged();
     }
 }
 
@@ -884,6 +905,7 @@ void QQuickFolderListModel::setSortCaseSensitive(bool on)
     if (on != d->sortCaseSensitive) {
         d->sortCaseSensitive = on;
         d->updateSorting();
+        emit sortCaseSensitiveChanged();
     }
 }
 
@@ -896,7 +918,6 @@ void QQuickFolderListModel::setSortCaseSensitive(bool on)
     \list
         \li \c fileName (\c string)
         \li \c filePath (\c string)
-        \li \c fileURL (\c url) (since Qt 5.2; deprecated since Qt 5.15)
         \li \c fileUrl (\c url) (since Qt 5.15)
         \li \c fileBaseName (\c string)
         \li \c fileSuffix (\c string)
@@ -910,7 +931,7 @@ QVariant QQuickFolderListModel::get(int idx, const QString &property) const
 {
     int role = roleFromString(property);
     if (role >= 0 && idx >= 0)
-        return data(index(idx, 0), role);
+        return data(index(idx), role);
     else
         return QVariant();
 }
