@@ -2,6 +2,9 @@
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "quicklintplugin.h"
+#include "qquickliteralbindingcheck_p.h"
+#include <QtQmlCompiler/private/qqmlsasourcelocation_p.h>
+#include <QtQmlCompiler/private/qqmljsutils_p.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -15,6 +18,8 @@ static constexpr QQmlSA::LoggerWarningId quickUnexpectedVarType { "Quick.unexpec
 static constexpr QQmlSA::LoggerWarningId quickPropertyChangesParsed { "Quick.property-changes-parsed" };
 static constexpr QQmlSA::LoggerWarningId quickControlsAttachedPropertyReuse { "Quick.controls-attached-property-reuse" };
 static constexpr QQmlSA::LoggerWarningId quickAttachedPropertyReuse { "Quick.attached-property-reuse" };
+static constexpr QQmlSA::LoggerWarningId quickColor { "Quick.color" };
+static constexpr QQmlSA::LoggerWarningId quickStateNoChildItem { "Quick.state-no-child-item" };
 
 ForbiddenChildrenPropertyValidatorPass::ForbiddenChildrenPropertyValidatorPass(
         QQmlSA::PassManager *manager)
@@ -37,7 +42,7 @@ bool ForbiddenChildrenPropertyValidatorPass::shouldRun(const QQmlSA::Element &el
     if (!element.parentScope())
         return false;
 
-    for (const auto &pair : m_types.asKeyValueRange()) {
+    for (const auto &pair : std::as_const(m_types).asKeyValueRange()) {
         if (element.parentScope().inherits(pair.first))
             return true;
     }
@@ -47,8 +52,17 @@ bool ForbiddenChildrenPropertyValidatorPass::shouldRun(const QQmlSA::Element &el
 
 void ForbiddenChildrenPropertyValidatorPass::run(const QQmlSA::Element &element)
 {
-    for (const auto &elementPair : m_types.asKeyValueRange()) {
+    for (const auto &elementPair : std::as_const(m_types).asKeyValueRange()) {
         const QQmlSA::Element &type = elementPair.first;
+        const QQmlSA::Element parentScope = element.parentScope();
+
+        // If the parent's default property is not what we think it is, then we can't say whether
+        // the element in question is actually a visual child of the (document) parent scope.
+        const QQmlSA::Property defaultProperty
+                = parentScope.property(parentScope.defaultPropertyName());
+        if (defaultProperty != type.property(type.defaultPropertyName()))
+            continue;
+
         if (!element.parentScope().inherits(type))
             continue;
 
@@ -75,8 +89,13 @@ QString AttachedPropertyTypeValidatorPass::addWarning(TypeDescription attachType
 {
     QVarLengthArray<QQmlSA::Element, 4> elements;
 
-    const QQmlSA::Element baseType = resolveType(attachType.module, attachType.name);
     const QQmlSA::Element attachedType = resolveAttached(attachType.module, attachType.name);
+    if (!attachedType) {
+        emitWarning(
+                "Cannot find attached type for %1/%2"_L1.arg(attachType.module, attachType.name),
+                quickAttachedPropertyType);
+        return QString();
+    }
 
     for (const TypeDescription &desc : allowedTypes) {
         const QQmlSA::Element type = resolveType(desc.module, desc.name);
@@ -103,6 +122,10 @@ void AttachedPropertyTypeValidatorPass::checkWarnings(const QQmlSA::Element &ele
         if (scopeUsedIn.inherits(type))
             return;
     }
+    // You can use e.g. Layout.leftMargin: 4 in PropertyChanges;
+    // custom parser can do arbitrary things with their contained bindings
+    if ( QQmlJSScope::scope(scopeUsedIn)->isInCustomParserParent() )
+        return;
 
     if (warning->allowInDelegate) {
         if (scopeUsedIn.isPropertyRequired(u"index"_s)
@@ -111,9 +134,7 @@ void AttachedPropertyTypeValidatorPass::checkWarnings(const QQmlSA::Element &ele
 
         // If the scope is at the root level, we cannot know whether it will be used
         // as a delegate or not.
-        // ### TODO: add a method to check whether a scope is the global scope
-        // so that we do not need to use internalId
-        if (!scopeUsedIn.parentScope() || scopeUsedIn.parentScope().internalId() == u"global"_s)
+        if (scopeUsedIn.isFileRootComponent())
             return;
 
         for (const QQmlSA::Binding &binding :
@@ -434,7 +455,7 @@ VarBindingTypeValidatorPass::VarBindingTypeValidatorPass(
 {
     QMultiHash<QString, QQmlSA::Element> propertyTypes;
 
-    for (const auto pair : expectedPropertyTypes.asKeyValueRange()) {
+    for (const auto &pair : expectedPropertyTypes.asKeyValueRange()) {
         const QQmlSA::Element propType = pair.second.module.isEmpty()
                 ? resolveBuiltinType(pair.second.name)
                 : resolveType(pair.second.module, pair.second.name);
@@ -507,6 +528,210 @@ void VarBindingTypeValidatorPass::onBinding(const QQmlSA::Element &element,
     }
 }
 
+class ColorValidatorPass : public QQmlSA::PropertyPass
+{
+public:
+    ColorValidatorPass(QQmlSA::PassManager *manager);
+
+    void onBinding(const QQmlSA::Element &element, const QString &propertyName,
+                   const QQmlSA::Binding &binding, const QQmlSA::Element &bindingScope,
+                   const QQmlSA::Element &value) override;
+private:
+    QQmlSA::Element m_colorType;
+
+    static inline const QRegularExpression s_hexPattern{ "^#[0-9A-Fa-f]{6}([0-9A-Fa-f]{2})?$"_L1 };
+    // list taken from https://doc.qt.io/qt-6/qcolor.html#fromString
+    QStringList m_colorNames = {
+        u"aliceblue"_s,
+        u"antiquewhite"_s,
+        u"aqua"_s,
+        u"aquamarine"_s,
+        u"azure"_s,
+        u"beige"_s,
+        u"bisque"_s,
+        u"black"_s,
+        u"blanchedalmond"_s,
+        u"blue"_s,
+        u"blueviolet"_s,
+        u"brown"_s,
+        u"burlywood"_s,
+        u"cadetblue"_s,
+        u"chartreuse"_s,
+        u"chocolate"_s,
+        u"coral"_s,
+        u"cornflowerblue"_s,
+        u"cornsilk"_s,
+        u"crimson"_s,
+        u"cyan"_s,
+        u"darkblue"_s,
+        u"darkcyan"_s,
+        u"darkgoldenrod"_s,
+        u"darkgray"_s,
+        u"darkgreen"_s,
+        u"darkgrey"_s,
+        u"darkkhaki"_s,
+        u"darkmagenta"_s,
+        u"darkolivegreen"_s,
+        u"darkorange"_s,
+        u"darkorchid"_s,
+        u"darkred"_s,
+        u"darksalmon"_s,
+        u"darkseagreen"_s,
+        u"darkslateblue"_s,
+        u"darkslategray"_s,
+        u"darkslategrey"_s,
+        u"darkturquoise"_s,
+        u"darkviolet"_s,
+        u"deeppink"_s,
+        u"deepskyblue"_s,
+        u"dimgray"_s,
+        u"dimgrey"_s,
+        u"dodgerblue"_s,
+        u"firebrick"_s,
+        u"floralwhite"_s,
+        u"forestgreen"_s,
+        u"fuchsia"_s,
+        u"gainsboro"_s,
+        u"ghostwhite"_s,
+        u"gold"_s,
+        u"goldenrod"_s,
+        u"gray"_s,
+        u"green"_s,
+        u"greenyellow"_s,
+        u"grey"_s,
+        u"honeydew"_s,
+        u"hotpink"_s,
+        u"indianred"_s,
+        u"indigo"_s,
+        u"ivory"_s,
+        u"khaki"_s,
+        u"lavender"_s,
+        u"lavenderblush"_s,
+        u"lawngreen"_s,
+        u"lemonchiffon"_s,
+        u"lightblue"_s,
+        u"lightcoral"_s,
+        u"lightcyan"_s,
+        u"lightgoldenrodyellow"_s,
+        u"lightgray"_s,
+        u"lightgreen"_s,
+        u"lightgrey"_s,
+        u"lightpink"_s,
+        u"lightsalmon"_s,
+        u"lightseagreen"_s,
+        u"lightskyblue"_s,
+        u"lightslategray"_s,
+        u"lightslategrey"_s,
+        u"lightsteelblue"_s,
+        u"lightyellow"_s,
+        u"lime"_s,
+        u"limegreen"_s,
+        u"linen"_s,
+        u"magenta"_s,
+        u"maroon"_s,
+        u"mediumaquamarine"_s,
+        u"mediumblue"_s,
+        u"mediumorchid"_s,
+        u"mediumpurple"_s,
+        u"mediumseagreen"_s,
+        u"mediumslateblue"_s,
+        u"mediumspringgreen"_s,
+        u"mediumturquoise"_s,
+        u"mediumvioletred"_s,
+        u"midnightblue"_s,
+        u"mintcream"_s,
+        u"mistyrose"_s,
+        u"moccasin"_s,
+        u"navajowhite"_s,
+        u"navy"_s,
+        u"oldlace"_s,
+        u"olive"_s,
+        u"olivedrab"_s,
+        u"orange"_s,
+        u"orangered"_s,
+        u"orchid"_s,
+        u"palegoldenrod"_s,
+        u"palegreen"_s,
+        u"paleturquoise"_s,
+        u"palevioletred"_s,
+        u"papayawhip"_s,
+        u"peachpuff"_s,
+        u"peru"_s,
+        u"pink"_s,
+        u"plum"_s,
+        u"powderblue"_s,
+        u"purple"_s,
+        u"red"_s,
+        u"rosybrown"_s,
+        u"royalblue"_s,
+        u"saddlebrown"_s,
+        u"salmon"_s,
+        u"sandybrown"_s,
+        u"seagreen"_s,
+        u"seashell"_s,
+        u"sienna"_s,
+        u"silver"_s,
+        u"skyblue"_s,
+        u"slateblue"_s,
+        u"slategray"_s,
+        u"slategrey"_s,
+        u"snow"_s,
+        u"springgreen"_s,
+        u"steelblue"_s,
+        u"tan"_s,
+        u"teal"_s,
+        u"thistle"_s,
+        u"tomato"_s,
+        u"turquoise"_s,
+        u"violet"_s,
+        u"wheat"_s,
+        u"white"_s,
+        u"whitesmoke"_s,
+        u"yellow"_s,
+        u"yellowgreen"_s,
+    };
+};
+
+
+ColorValidatorPass::ColorValidatorPass(QQmlSA::PassManager *manager)
+    : PropertyPass(manager), m_colorType(resolveType("QtQuick"_L1, "color"_L1))
+{
+    Q_ASSERT_X(std::is_sorted(m_colorNames.cbegin(), m_colorNames.cend()), "ColorValidatorPass",
+               "m_colorNames should be sorted!");
+}
+
+void ColorValidatorPass::onBinding(const QQmlSA::Element &element, const QString &propertyName,
+                                   const QQmlSA::Binding &binding, const QQmlSA::Element &,
+                                   const QQmlSA::Element &)
+{
+    if (binding.bindingType() != QQmlSA::BindingType::StringLiteral)
+        return;
+    const auto propertyType = element.property(propertyName).type();
+    if (!propertyType || propertyType != m_colorType)
+        return;
+
+    QString colorName = binding.stringValue();
+    // for "named" colors, QColor::fromString does not care about
+    // the case
+    if (!colorName.startsWith(u'#'))
+        colorName = std::move(colorName).toLower();
+    if (s_hexPattern.match(colorName).hasMatch())
+        return;
+
+    if (std::binary_search(m_colorNames.cbegin(), m_colorNames.cend(), colorName))
+        return;
+
+    if (colorName == u"transparent")
+        return;
+
+    auto suggestion = QQmlJSUtils::didYouMean(
+            colorName, m_colorNames,
+            QQmlSA::SourceLocationPrivate::sourceLocation(binding.sourceLocation()));
+
+    emitWarningWithOptionalFix(*this, "Invalid color \"%1\"."_L1.arg(colorName), quickColor,
+                               binding.sourceLocation(), suggestion);
+}
+
 void AttachedPropertyReuse::onRead(const QQmlSA::Element &element, const QString &propertyName,
                                    const QQmlSA::Element &readScope,
                                    QQmlSA::SourceLocation location)
@@ -552,6 +777,7 @@ void AttachedPropertyReuse::onRead(const QQmlSA::Element &element, const QString
             emitWarning("Using attached type %1 already initialized in a parent scope."_L1.arg(
                                 element.name()),
                         category, attachedLocation, suggestion);
+            return;
         }
 
         return;
@@ -610,6 +836,11 @@ void QmlLintQuickPlugin::registerPasses(QQmlSA::PassManager *manager,
     if (hasQuick) {
         manager->registerElementPass(std::make_unique<AnchorsValidatorPass>(manager));
         manager->registerElementPass(std::make_unique<PropertyChangesValidatorPass>(manager));
+        manager->registerElementPass(std::make_unique<StateNoItemChildrenValidator>(manager));
+        manager->registerPropertyPass(std::make_unique<QQuickLiteralBindingCheck>(manager),
+                                      QAnyStringView(), QAnyStringView());
+        manager->registerPropertyPass(std::make_unique<ColorValidatorPass>(manager),
+                                      QAnyStringView(), QAnyStringView());
 
         auto forbiddenChildProperty =
                 std::make_unique<ForbiddenChildrenPropertyValidatorPass>(manager);
@@ -655,6 +886,9 @@ void QmlLintQuickPlugin::registerPasses(QQmlSA::PassManager *manager,
                                   QAnyStringView warning, bool allowInDelegate = false) {
         QString attachedTypeName = attachedPropertyType->addWarning(attachedType, allowedTypes,
                                                                     allowInDelegate, warning);
+        if (attachedTypeName.isEmpty())
+            return;
+
         manager->registerPropertyPass(attachedPropertyType, attachedType.module,
                                       u"$internal$."_s + attachedTypeName, {}, false);
     };
@@ -675,18 +909,18 @@ void QmlLintQuickPlugin::registerPasses(QQmlSA::PassManager *manager,
                              { { "columnWidthProvider", { "", "function" } },
                                { "rowHeightProvider", { "", "function" } } });
         addAttachedWarning({ "QtQuick", "Accessible" }, { { "QtQuick", "Item" } },
-                           "Accessible must be attached to an Item");
+                           "Accessible attached property must be attached to an object deriving from Item or Action");
         addAttachedWarning({ "QtQuick", "LayoutMirroring" },
                            { { "QtQuick", "Item" }, { "QtQuick", "Window" } },
-                           "LayoutDirection attached property only works with Items and Windows");
+                           "LayoutMirroring attached property must be attached to an object deriving from Item or Window");
         addAttachedWarning({ "QtQuick", "EnterKey" }, { { "QtQuick", "Item" } },
-                           "EnterKey attached property only works with Items");
+                           "EnterKey attached property must be attached to an object deriving from Item");
     }
     if (hasQuickLayouts) {
         addAttachedWarning({ "QtQuick.Layouts", "Layout" }, { { "QtQuick", "Item" } },
-                           "Layout must be attached to Item elements");
+                           "Layout attached property must be attached to an object deriving from Item");
         addAttachedWarning({ "QtQuick.Layouts", "StackLayout" }, { { "QtQuick", "Item" } },
-                           "StackLayout must be attached to an Item");
+                           "StackLayout attached property must be attached to an object deriving from Item");
     }
 
 
@@ -697,26 +931,22 @@ void QmlLintQuickPlugin::registerPasses(QQmlSA::PassManager *manager,
 
         addAttachedWarning({ "QtQuick.Templates", "ScrollBar" },
                            { { "QtQuick", "Flickable" }, { "QtQuick.Templates", "ScrollView" } },
-                           "ScrollBar must be attached to a Flickable or ScrollView");
+                           "ScrollBar attached property must be attached to an object deriving from Flickable or ScrollView");
         addAttachedWarning({ "QtQuick.Templates", "ScrollIndicator" },
                            { { "QtQuick", "Flickable" } },
-                           "ScrollIndicator must be attached to a Flickable");
+                           "ScrollIndicator attached property must be attached to an object deriving from Flickable");
         addAttachedWarning({ "QtQuick.Templates", "TextArea" }, { { "QtQuick", "Flickable" } },
-                           "TextArea must be attached to a Flickable");
+                           "TextArea attached property must be attached to an object deriving from Flickable");
         addAttachedWarning({ "QtQuick.Templates", "SplitView" }, { { "QtQuick", "Item" } },
-                           "SplitView attached property only works with Items");
+                           "SplitView attached property must be attached to an object deriving from Item");
         addAttachedWarning({ "QtQuick.Templates", "StackView" }, { { "QtQuick", "Item" } },
-                           "StackView attached property only works with Items");
+                           "StackView attached property must be attached to an object deriving from Item");
         addAttachedWarning({ "QtQuick.Templates", "ToolTip" }, { { "QtQuick", "Item" } },
-                           "ToolTip must be attached to an Item");
+                           "ToolTip attached property must be attached to an object deriving from Item");
         addAttachedWarning({ "QtQuick.Templates", "SwipeDelegate" }, { { "QtQuick", "Item" } },
-                           "Attached properties of SwipeDelegate must be accessed through an Item");
+                           "SwipeDelegate attached property must be attached to an object deriving from Item");
         addAttachedWarning({ "QtQuick.Templates", "SwipeView" }, { { "QtQuick", "Item" } },
-                           "SwipeView must be attached to an Item");
-        addAttachedWarning(
-                { "QtQuick.Templates", "Tumbler" }, { { "QtQuick", "Tumbler" } },
-                "Tumbler: attached properties of Tumbler must be accessed through a delegate item",
-                true);
+                           "SwipeView attached property must be attached to an object deriving from Item");
         addVarBindingWarning("QtQuick.Templates", "Tumbler",
                              { { "contentItem", { "QtQuick", "PathView" } },
                                { "contentItem", { "QtQuick", "ListView" } } });
@@ -794,6 +1024,40 @@ void PropertyChangesValidatorPass::run(const QQmlSA::Element &element)
     }
 }
 
+StateNoItemChildrenValidator::StateNoItemChildrenValidator(QQmlSA::PassManager *manager)
+    : QQmlSA::ElementPass(manager)
+    , m_state(resolveType("QtQuick", "State"))
+    , m_anchorChanges(resolveType("QtQuick", "AnchorChanges"))
+    , m_parentChanges(resolveType("QtQuick", "ParentChange"))
+    , m_propertyChanges(resolveType("QtQuick", "PropertyChanges"))
+    , m_stateChangeScript(resolveType("QtQuick", "StateChangeScript"))
+{}
+
+bool StateNoItemChildrenValidator::shouldRun(const QQmlSA::Element &element)
+{
+    return element.inherits(m_state);
+}
+
+void StateNoItemChildrenValidator::run(const QQmlSA::Element &element)
+{
+    const auto &childScopes = QQmlJSScope::scope(element)->childScopes();
+    for (const auto &child : childScopes) {
+        if (child->scopeType() != QQmlSA::ScopeType::QMLScope)
+            continue;
+
+        if (child->inherits(QQmlJSScope::scope(m_anchorChanges))
+            || child->inherits(QQmlJSScope::scope(m_parentChanges))
+            || child->inherits(QQmlJSScope::scope(m_propertyChanges))
+            || child->inherits(QQmlJSScope::scope(m_stateChangeScript))) {
+            continue;
+        }
+        QString msg = "A State cannot have a child item of type %1"_L1.arg(child->baseTypeName());
+        auto loc = QQmlSA::SourceLocationPrivate::createQQmlSASourceLocation(
+                child->sourceLocation());
+        emitWarning(msg, quickStateNoChildItem, loc);
+    }
+}
+
 QT_END_NAMESPACE
 
 #include "moc_quicklintplugin.cpp"
