@@ -264,19 +264,14 @@ uint SimpleArrayData::truncate(Object *o, uint newLen)
         return newLen;
 
     if (!dd->attrs) {
-        for (uint i = newLen; i < dd->values.size; ++i)
-            dd->setData(dd->internalClass->engine, i, Value::emptyValue());
         dd->values.size = newLen;
         return newLen;
     }
 
     while (dd->values.size > newLen) {
-        const uint lastIndex = dd->values.size - 1;
-        if (!dd->data(lastIndex).isEmpty() && !dd->attrs[lastIndex].isConfigurable())
+        if (!dd->data(dd->values.size - 1).isEmpty() && !dd->attrs[dd->values.size - 1].isConfigurable())
             return dd->values.size;
-
-        dd->setData(dd->internalClass->engine, lastIndex, Value::emptyValue());
-        dd->values.size = lastIndex;
+        --dd->values.size;
     }
     return dd->values.size;
 }
