@@ -2,17 +2,14 @@
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qv4dateobject_p.h"
-#include "qv4objectproto_p.h"
-#include "qv4scopedvalue_p.h"
 #include "qv4runtime_p.h"
-#include "qv4string_p.h"
-#include "qv4jscall_p.h"
 #include "qv4symbol_p.h"
 
 #include <QtCore/QDebug>
 #include <QtCore/QDateTime>
 #include <QtCore/private/qlocaltime_p.h>
 #include <QtCore/QStringList>
+#include <QtCore/QTimeZone>
 
 #include <wtf/MathExtras.h>
 
@@ -305,7 +302,7 @@ static inline double currentTime()
 
 static inline double TimeClip(double t)
 {
-    if (! qt_is_finite(t) || fabs(t) > 8.64e15)
+    if (!qt_is_finite(t) || fabs(t) > Date::MaxDateVal)
         return qt_qnan();
 
     // +0 looks weird, but is correct. See ES6 20.3.1.15. We must not return -0.
@@ -318,14 +315,14 @@ static inline double ParseString(const QString &s, double localTZA)
       First, try the format defined in ECMA 262's "Date Time String Format";
       only if that fails, fall back to QDateTime for parsing
 
-      The defined string format is YYYY-MM-DDTHH:mm:ss.sssZ; the time (T and all
-      after it) may be omitted; in each part, the second and later components
-      are optional; and there's an extended syntax for negative and large
-      positive years: +/-YYYYYY; the leading sign, even when +, isn't optional.
-      If month or day is omitted, it is 01; if minute or second is omitted, it's
-      00; if milliseconds are omitted, they're 000.
+      The defined string format is yyyy-MM-ddTHH:mm:ss.zzzt; the time (T and all
+      after it) may be omitted. In each part, the second and later components
+      are optional. There's an extended syntax for negative and large positive
+      years: ±yyyyyy; the leading sign, even when +, isn't optional.  If month
+      (MM) or day (dd) is omitted, it is 01; if minute (mm) or second (ss) is
+      omitted, it's 00; if milliseconds (zzz) are omitted, they're 000.
 
-      When the time zone offset is absent, date-only forms are interpreted as
+      When the time zone offset (t) is absent, date-only forms are interpreted as
       indicating a UTC time and date-time forms are interpreted in local time.
     */
 
@@ -534,7 +531,7 @@ static inline double ParseString(const QString &s, double localTZA)
 
         for (const QString &format : formats) {
             dt = format.indexOf(QLatin1String("hh:mm")) < 0
-                    ? QDate::fromString(s, format).startOfDay(Qt::UTC)
+                    ? QDate::fromString(s, format).startOfDay(QTimeZone::UTC)
                     : QDateTime::fromString(s, format); // as local time
             if (dt.isValid())
                 break;
@@ -551,18 +548,18 @@ static inline double ParseString(const QString &s, double localTZA)
   Converts the ECMA Date value \a t (in UTC form) to QDateTime
   according to \a spec.
 */
-static inline QDateTime ToDateTime(double t, Qt::TimeSpec spec)
+static inline QDateTime ToDateTime(double t, QTimeZone zone)
 {
     if (std::isnan(t))
-        return QDateTime();
-    return QDateTime::fromMSecsSinceEpoch(t, spec);
+        return QDateTime().toTimeZone(zone);
+    return QDateTime::fromMSecsSinceEpoch(t, zone);
 }
 
 static inline QString ToString(double t, double localTZA)
 {
     if (std::isnan(t))
         return QStringLiteral("Invalid Date");
-    QString str = ToDateTime(t, Qt::LocalTime).toString() + QLatin1String(" GMT");
+    QString str = ToDateTime(t, QTimeZone::LocalTime).toString() + QLatin1String(" GMT");
     double tzoffset = localTZA + DaylightSavingTA(t, localTZA);
     if (tzoffset) {
         int hours = static_cast<int>(::fabs(tzoffset) / 1000 / 60 / 60);
@@ -582,32 +579,32 @@ static inline QString ToUTCString(double t)
 {
     if (std::isnan(t))
         return QStringLiteral("Invalid Date");
-    return ToDateTime(t, Qt::UTC).toString();
+    return ToDateTime(t, QTimeZone::UTC).toString();
 }
 
 static inline QString ToDateString(double t)
 {
-    return ToDateTime(t, Qt::LocalTime).date().toString();
+    return ToDateTime(t, QTimeZone::LocalTime).date().toString();
 }
 
 static inline QString ToTimeString(double t)
 {
-    return ToDateTime(t, Qt::LocalTime).time().toString();
+    return ToDateTime(t, QTimeZone::LocalTime).time().toString();
 }
 
 static inline QString ToLocaleString(double t)
 {
-    return QLocale().toString(ToDateTime(t, Qt::LocalTime), QLocale::ShortFormat);
+    return QLocale().toString(ToDateTime(t, QTimeZone::LocalTime), QLocale::ShortFormat);
 }
 
 static inline QString ToLocaleDateString(double t)
 {
-    return QLocale().toString(ToDateTime(t, Qt::LocalTime).date(), QLocale::ShortFormat);
+    return QLocale().toString(ToDateTime(t, QTimeZone::LocalTime).date(), QLocale::ShortFormat);
 }
 
 static inline QString ToLocaleTimeString(double t)
 {
-    return QLocale().toString(ToDateTime(t, Qt::LocalTime).time(), QLocale::ShortFormat);
+    return QLocale().toString(ToDateTime(t, QTimeZone::LocalTime).time(), QLocale::ShortFormat);
 }
 
 static double getLocalTZA()
@@ -617,17 +614,43 @@ static double getLocalTZA()
 
 DEFINE_OBJECT_VTABLE(DateObject);
 
-void Heap::DateObject::init(const QDateTime &when)
+quint64 Date::encode(double value)
 {
-    Object::init();
-    date = when.isValid() ? TimeClip(when.toMSecsSinceEpoch()) : qt_qnan();
+    if (std::isnan(value) || fabs(value) > MaxDateVal)
+        return InvalidDateVal;
+
+    // Do the addition in qint64. This way we won't overflow if value is negative
+    // and we will round value in the right direction.
+    // We know we can do this because we know we have more than one bit left in quint64.
+    const quint64 encoded = quint64(qint64(value) + qint64(MaxDateVal));
+
+    return encoded + Extra;
 }
 
-void Heap::DateObject::init(QTime time)
+quint64 Date::encode(const QDateTime &dateTime)
+{
+    return encode(dateTime.isValid() ? dateTime.toMSecsSinceEpoch() : qt_qnan());
+}
+
+void Date::init(double value)
+{
+    storage = encode(value);
+}
+
+void Date::init(const QDateTime &when)
+{
+    storage = encode(when) | HasQDate | HasQTime;
+}
+
+void Date::init(QDate date)
+{
+    storage = encode(date.startOfDay(QTimeZone::UTC)) | HasQDate;
+}
+
+void Date::init(QTime time, ExecutionEngine *engine)
 {
-    Object::init();
     if (!time.isValid()) {
-        date = qt_qnan();
+        storage = encode(qt_qnan()) | HasQTime;
         return;
     }
 
@@ -647,19 +670,141 @@ void Heap::DateObject::init(QTime time)
      */
     static const double d = MakeDay(1971, 3, 1);
     double t = MakeTime(time.hour(), time.minute(), time.second(), time.msec());
-    date = TimeClip(UTC(MakeDate(d, t), internalClass->engine->localTZA));
+    storage = encode(UTC(MakeDate(d, t), engine->localTZA)) | HasQTime;
+}
+
+QDate Date::toQDate() const
+{
+    return toQDateTime().date();
+}
+
+QTime Date::toQTime() const
+{
+    return toQDateTime().time();
+}
+
+QDateTime Date::toQDateTime() const
+{
+    return ToDateTime(operator double(), QTimeZone::LocalTime);
+}
+
+QVariant Date::toVariant() const
+{
+    // Note that we shouldn't and don't read-back here, compared to
+    // most other methods, as this is only used when we perform a
+    // write-back, that is we are sending our version of the data back
+    // to the originating element.
+    switch (storage & (HasQDate | HasQTime)) {
+    case HasQDate:
+        return toQDate();
+    case HasQTime:
+        return toQTime();
+    case (HasQDate | HasQTime):
+        return toQDateTime();
+    default:
+        return QVariant();
+    }
 }
 
 QDateTime DateObject::toQDateTime() const
 {
-    return ToDateTime(date(), Qt::LocalTime);
+    if (d()->isAttachedToProperty())
+        d()->readReference();
+    return d()->toQDateTime();
+}
+
+QString DateObject::toString() const
+{
+    if (d()->isAttachedToProperty())
+        d()->readReference();
+    return ToString(d()->date(), engine()->localTZA);
+}
+
+QString DateObject::dateTimeToString(const QDateTime &dateTime, ExecutionEngine *engine)
+{
+    if (!dateTime.isValid())
+        return QStringLiteral("Invalid Date");
+    return ToString(TimeClip(dateTime.toMSecsSinceEpoch()), engine->localTZA);
+}
+
+double DateObject::dateTimeToNumber(const QDateTime &dateTime)
+{
+    if (!dateTime.isValid())
+        return qQNaN();
+    return TimeClip(dateTime.toMSecsSinceEpoch());
+}
+
+QDateTime DateObject::stringToDateTime(const QString &string, ExecutionEngine *engine)
+{
+    return ToDateTime(ParseString(string, engine->localTZA), QTimeZone::LocalTime);
+}
+
+QDateTime DateObject::timestampToDateTime(double timestamp, QTimeZone zone)
+{
+    return ToDateTime(timestamp, zone);
+}
+
+double DateObject::componentsToTimestamp(
+        double year, double month, double day, double hours,
+        double mins, double secs, double ms, ExecutionEngine *v4)
+{
+    if (year >= 0 && year <= 99)
+        year += 1900;
+    const double t = MakeDate(MakeDay(year, month, day), MakeTime(hours, mins, secs, ms));
+    return UTC(t, v4->localTZA);
+}
+
+QDate DateObject::dateTimeToDate(const QDateTime &dateTime)
+{
+    // If the Date object was parse()d from a string with no time part
+    // or zone specifier it's really the UTC start of the relevant day,
+    // but it's here represented as a local time, which may fall in the
+    // preceding day. See QTBUG-92466 for the gory details.
+    const auto utc = dateTime.toUTC();
+    if (utc.date() != dateTime.date() && utc.addSecs(-1).date() == dateTime.date())
+        return utc.date();
+
+    // This may, of course, be The Wrong Thing if the date was
+    // constructed as a full local date-time that happens to coincide
+    // with the start of a UTC day; however, that would be an odd value
+    // to give to something that, apparently, someone thinks belongs in
+    // a QDate.
+    return dateTime.date();
 }
 
 DEFINE_OBJECT_VTABLE(DateCtor);
 
-void Heap::DateCtor::init(QV4::ExecutionContext *scope)
+Heap::DateObject *Heap::DateObject::detached() const
 {
-    Heap::FunctionObject::init(scope, QStringLiteral("Date"));
+    return internalClass->engine->memoryManager->allocate<QV4::DateObject>(m_date);
+}
+
+bool Heap::DateObject::setVariant(const QVariant &variant)
+{
+    const QMetaType variantReferenceType = variant.metaType();
+    switch (variantReferenceType.id()) {
+    case QMetaType::Double:
+        m_date.init(*static_cast<const double *>(variant.constData()));
+        break;
+    case QMetaType::QDate:
+        m_date.init(*static_cast<const QDate *>(variant.constData()));
+        break;
+    case QMetaType::QTime:
+        m_date.init(*static_cast<const QTime *>(variant.constData()), internalClass->engine);
+        break;
+    case QMetaType::QDateTime:
+        m_date.init(*static_cast<const QDateTime *>(variant.constData()));
+        break;
+    default:
+        return false;
+    }
+
+    return true;
+}
+
+void Heap::DateCtor::init(QV4::ExecutionEngine *engine)
+{
+    Heap::FunctionObject::init(engine, QStringLiteral("Date"));
 }
 
 ReturnedValue DateCtor::virtualCallAsConstructor(const FunctionObject *that, const Value *argv, int argc, const Value *newTarget)
@@ -681,25 +826,22 @@ ReturnedValue DateCtor::virtualCallAsConstructor(const FunctionObject *that, con
             if (String *s = arg->stringValue())
                 t = ParseString(s->toQString(), v4->localTZA);
             else
-                t = TimeClip(arg->toNumber());
+                t = arg->toNumber();
         }
     }
 
     else { // d.argc > 1
-        double year  = argv[0].toNumber();
-        double month = argv[1].toNumber();
-        double day  = argc >= 3 ? argv[2].toNumber() : 1;
-        double hours = argc >= 4 ? argv[3].toNumber() : 0;
-        double mins = argc >= 5 ? argv[4].toNumber() : 0;
-        double secs = argc >= 6 ? argv[5].toNumber() : 0;
-        double ms    = argc >= 7 ? argv[6].toNumber() : 0;
-        if (year >= 0 && year <= 99)
-            year += 1900;
-        t = MakeDate(MakeDay(year, month, day), MakeTime(hours, mins, secs, ms));
-        t = TimeClip(UTC(t, v4->localTZA));
+        const double year  = argv[0].toNumber();
+        const double month = argv[1].toNumber();
+        const double day  = argc >= 3 ? argv[2].toNumber() : 1;
+        const double hours = argc >= 4 ? argv[3].toNumber() : 0;
+        const double mins = argc >= 5 ? argv[4].toNumber() : 0;
+        const double secs = argc >= 6 ? argv[5].toNumber() : 0;
+        const double ms    = argc >= 7 ? argv[6].toNumber() : 0;
+        t = DateObject::componentsToTimestamp(year, month, day, hours, mins, secs, ms, v4);
     }
 
-    ReturnedValue o = Encode(v4->newDateObject(Value::fromDouble(t)));
+    ReturnedValue o = Encode(v4->newDateObject(t));
     if (!newTarget)
         return o;
     Scope scope(v4);
@@ -790,8 +932,12 @@ void DatePrototype::init(ExecutionEngine *engine, Object *ctor)
 
 double DatePrototype::getThisDate(ExecutionEngine *v4, const Value *thisObject)
 {
-    if (const DateObject *that = thisObject->as<DateObject>())
+    if (const DateObject *that = thisObject->as<DateObject>()) {
+        if (that->d()->isAttachedToProperty())
+            that->d()->readReference();
+
         return that->date();
+    }
     v4->throwTypeError();
     return 0;
 }
@@ -1072,7 +1218,7 @@ ReturnedValue DatePrototype::method_setTime(const FunctionObject *b, const Value
     double t = argc ? argv[0].toNumber() : qt_qnan();
     if (v4->hasException)
         return QV4::Encode::undefined();
-    self->setDate(TimeClip(t));
+    self->setDate(t);
     return Encode(self->date());
 }
 
@@ -1089,7 +1235,7 @@ ReturnedValue DatePrototype::method_setMilliseconds(const FunctionObject *b, con
     double ms = argc ? argv[0].toNumber() : qt_qnan();
     if (v4->hasException)
         return QV4::Encode::undefined();
-    self->setDate(TimeClip(UTC(MakeDate(Day(t), MakeTime(HourFromTime(t), MinFromTime(t), SecFromTime(t), ms)), v4->localTZA)));
+    self->setDate(UTC(MakeDate(Day(t), MakeTime(HourFromTime(t), MinFromTime(t), SecFromTime(t), ms)), v4->localTZA));
     return Encode(self->date());
 }
 
@@ -1106,7 +1252,7 @@ ReturnedValue DatePrototype::method_setUTCMilliseconds(const FunctionObject *b,
     double ms = argc ? argv[0].toNumber() : qt_qnan();
     if (v4->hasException)
         return QV4::Encode::undefined();
-    self->setDate(TimeClip(MakeDate(Day(t), MakeTime(HourFromTime(t), MinFromTime(t), SecFromTime(t), ms))));
+    self->setDate(MakeDate(Day(t), MakeTime(HourFromTime(t), MinFromTime(t), SecFromTime(t), ms)));
     return Encode(self->date());
 }
 
@@ -1126,7 +1272,7 @@ ReturnedValue DatePrototype::method_setSeconds(const FunctionObject *b, const Va
     double ms = (argc < 2) ? msFromTime(t) : argv[1].toNumber();
     if (v4->hasException)
         return QV4::Encode::undefined();
-    t = TimeClip(UTC(MakeDate(Day(t), MakeTime(HourFromTime(t), MinFromTime(t), sec, ms)), v4->localTZA));
+    t = UTC(MakeDate(Day(t), MakeTime(HourFromTime(t), MinFromTime(t), sec, ms)), v4->localTZA);
     self->setDate(t);
     return Encode(self->date());
 }
@@ -1141,7 +1287,7 @@ ReturnedValue DatePrototype::method_setUTCSeconds(const FunctionObject *b, const
     double t = self->date();
     double sec = argc ? argv[0].toNumber() : qt_qnan();
     double ms = (argc < 2) ? msFromTime(t) : argv[1].toNumber();
-    t = TimeClip(MakeDate(Day(t), MakeTime(HourFromTime(t), MinFromTime(t), sec, ms)));
+    t = MakeDate(Day(t), MakeTime(HourFromTime(t), MinFromTime(t), sec, ms));
     self->setDate(t);
     return Encode(self->date());
 }
@@ -1165,7 +1311,7 @@ ReturnedValue DatePrototype::method_setMinutes(const FunctionObject *b, const Va
     double ms = (argc < 3) ? msFromTime(t) : argv[2].toNumber();
     if (v4->hasException)
         return QV4::Encode::undefined();
-    t = TimeClip(UTC(MakeDate(Day(t), MakeTime(HourFromTime(t), min, sec, ms)), v4->localTZA));
+    t = UTC(MakeDate(Day(t), MakeTime(HourFromTime(t), min, sec, ms)), v4->localTZA);
     self->setDate(t);
     return Encode(self->date());
 }
@@ -1181,7 +1327,7 @@ ReturnedValue DatePrototype::method_setUTCMinutes(const FunctionObject *b, const
     double min = argc ? argv[0].toNumber() : qt_qnan();
     double sec = (argc < 2) ? SecFromTime(t) : argv[1].toNumber();
     double ms = (argc < 3) ? msFromTime(t) : argv[2].toNumber();
-    t = TimeClip(MakeDate(Day(t), MakeTime(HourFromTime(t), min, sec, ms)));
+    t = MakeDate(Day(t), MakeTime(HourFromTime(t), min, sec, ms));
     self->setDate(t);
     return Encode(self->date());
 }
@@ -1208,7 +1354,7 @@ ReturnedValue DatePrototype::method_setHours(const FunctionObject *b, const Valu
     double ms = (argc < 4) ? msFromTime(t) : argv[3].toNumber();
     if (v4->hasException)
         return QV4::Encode::undefined();
-    t = TimeClip(UTC(MakeDate(Day(t), MakeTime(hour, min, sec, ms)), v4->localTZA));
+    t = UTC(MakeDate(Day(t), MakeTime(hour, min, sec, ms)), v4->localTZA);
     self->setDate(t);
     return Encode(self->date());
 }
@@ -1225,7 +1371,7 @@ ReturnedValue DatePrototype::method_setUTCHours(const FunctionObject *b, const V
     double min = (argc < 2) ? MinFromTime(t) : argv[1].toNumber();
     double sec = (argc < 3) ? SecFromTime(t) : argv[2].toNumber();
     double ms = (argc < 4) ? msFromTime(t) : argv[3].toNumber();
-    t = TimeClip(MakeDate(Day(t), MakeTime(hour, min, sec, ms)));
+    t = MakeDate(Day(t), MakeTime(hour, min, sec, ms));
     self->setDate(t);
     return Encode(self->date());
 }
@@ -1243,7 +1389,7 @@ ReturnedValue DatePrototype::method_setDate(const FunctionObject *b, const Value
     double date = argc ? argv[0].toNumber() : qt_qnan();
     if (v4->hasException)
         return QV4::Encode::undefined();
-    t = TimeClip(UTC(MakeDate(MakeDay(YearFromTime(t), MonthFromTime(t), date), TimeWithinDay(t)), v4->localTZA));
+    t = UTC(MakeDate(MakeDay(YearFromTime(t), MonthFromTime(t), date), TimeWithinDay(t)), v4->localTZA);
     self->setDate(t);
     return Encode(self->date());
 }
@@ -1261,7 +1407,7 @@ ReturnedValue DatePrototype::method_setUTCDate(const FunctionObject *b, const Va
     double date = argc ? argv[0].toNumber() : qt_qnan();
     if (v4->hasException)
         return QV4::Encode::undefined();
-    t = TimeClip(MakeDate(MakeDay(YearFromTime(t), MonthFromTime(t), date), TimeWithinDay(t)));
+    t = MakeDate(MakeDay(YearFromTime(t), MonthFromTime(t), date), TimeWithinDay(t));
     self->setDate(t);
     return Encode(self->date());
 }
@@ -1282,7 +1428,7 @@ ReturnedValue DatePrototype::method_setMonth(const FunctionObject *b, const Valu
     double date = (argc < 2) ? DateFromTime(t) : argv[1].toNumber();
     if (v4->hasException)
         return QV4::Encode::undefined();
-    t = TimeClip(UTC(MakeDate(MakeDay(YearFromTime(t), month, date), TimeWithinDay(t)), v4->localTZA));
+    t = UTC(MakeDate(MakeDay(YearFromTime(t), month, date), TimeWithinDay(t)), v4->localTZA);
     self->setDate(t);
     return Encode(self->date());
 }
@@ -1297,7 +1443,7 @@ ReturnedValue DatePrototype::method_setUTCMonth(const FunctionObject *b, const V
     double t = self->date();
     double month = argc ? argv[0].toNumber() : qt_qnan();
     double date = (argc < 2) ? DateFromTime(t) : argv[1].toNumber();
-    t = TimeClip(MakeDate(MakeDay(YearFromTime(t), month, date), TimeWithinDay(t)));
+    t = MakeDate(MakeDay(YearFromTime(t), month, date), TimeWithinDay(t));
     self->setDate(t);
     return Encode(self->date());
 }
@@ -1323,7 +1469,6 @@ ReturnedValue DatePrototype::method_setYear(const FunctionObject *b, const Value
             year += 1900;
         r = MakeDay(year, MonthFromTime(t), DateFromTime(t));
         r = UTC(MakeDate(r, TimeWithinDay(t)), v4->localTZA);
-        r = TimeClip(r);
     }
     self->setDate(r);
     return Encode(self->date());
@@ -1340,7 +1485,7 @@ ReturnedValue DatePrototype::method_setUTCFullYear(const FunctionObject *b, cons
     double year = argc ? argv[0].toNumber() : qt_qnan();
     double month = (argc < 2) ? MonthFromTime(t) : argv[1].toNumber();
     double date = (argc < 3) ? DateFromTime(t) : argv[2].toNumber();
-    t = TimeClip(MakeDate(MakeDay(year, month, date), TimeWithinDay(t)));
+    t = MakeDate(MakeDay(year, month, date), TimeWithinDay(t));
     self->setDate(t);
     return Encode(self->date());
 }
@@ -1366,7 +1511,7 @@ ReturnedValue DatePrototype::method_setFullYear(const FunctionObject *b, const V
     double date = (argc < 3) ? DateFromTime(t) : argv[2].toNumber();
     if (v4->hasException)
         return QV4::Encode::undefined();
-    t = TimeClip(UTC(MakeDate(MakeDay(year, month, date), TimeWithinDay(t)), v4->localTZA));
+    t = UTC(MakeDate(MakeDay(year, month, date), TimeWithinDay(t)), v4->localTZA);
     self->setDate(t);
     return Encode(self->date());
 }
@@ -1378,6 +1523,9 @@ ReturnedValue DatePrototype::method_toUTCString(const FunctionObject *b, const V
     if (!self)
         return v4->throwTypeError();
 
+    if (self->d()->isAttachedToProperty())
+        self->d()->readReference();
+
     double t = self->date();
     return Encode(v4->newString(ToUTCString(t)));
 }
@@ -1402,6 +1550,9 @@ ReturnedValue DatePrototype::method_toISOString(const FunctionObject *b, const V
     if (!self)
         return v4->throwTypeError();
 
+    if (self->d()->isAttachedToProperty())
+        self->d()->readReference();
+
     double t = self->date();
     if (!std::isfinite(t))
         RETURN_RESULT(v4->throwRangeError(*thisObject));
