@@ -27,17 +27,26 @@ QT_BEGIN_NAMESPACE
 
 Q_DECLARE_LOGGING_CATEGORY(lcQmlTypeCompiler);
 
+// This class primarily resolves component boundaries in a document.
+// With the information about boundaries, it then goes on to resolve aliases and generalized
+// group properties. Both rely on IDs as first part of their expressions and the IDs have
+// to be located in surrounding components. That's why we have to do this with the component
+// boundaries in mind.
+
+class QQmlComponentAndAliasResolverBase
+{
+    Q_DECLARE_TR_FUNCTIONS(QQmlComponentAndAliasResolverBase)
+};
+
 template<typename ObjectContainer>
-class QQmlComponentAndAliasResolver
+class QQmlComponentAndAliasResolver : public QQmlComponentAndAliasResolverBase
 {
-    Q_DECLARE_TR_FUNCTIONS(QQmlComponentAndAliasResolver)
 public:
     using CompiledObject = typename ObjectContainer::CompiledObject;
     using CompiledBinding = typename ObjectContainer::CompiledBinding;
 
     QQmlComponentAndAliasResolver(
             ObjectContainer *compiler,
-            QQmlEnginePrivate *enginePrivate,
             QQmlPropertyCacheVector *propertyCaches);
 
     [[nodiscard]] QQmlError resolve(int root = 0);
@@ -54,13 +63,28 @@ private:
     void setObjectId(int index) const;
     [[nodiscard]] bool markAsComponent(int index) const;
     [[nodiscard]] AliasResolutionResult resolveAliasesInObject(
-            const CompiledObject &component, int objectIndex, QQmlError *error);
+            const CompiledObject &component, int objectIndex,
+            QQmlPropertyCacheAliasCreator<ObjectContainer> *aliasCacheCreator, QQmlError *error);
+    [[nodiscard]] bool appendAliasToPropertyCache(
+            const CompiledObject *component, const QV4::CompiledData::Alias *alias, int objectIndex,
+            int aliasIndex, int encodedPropertyIndex,
+            QQmlPropertyCacheAliasCreator<ObjectContainer> *aliasCacheCreator, QQmlError *error)
+    {
+        *error = aliasCacheCreator->appendAliasToPropertyCache(
+                *component, *alias, objectIndex, aliasIndex, encodedPropertyIndex);
+        resolvedAliases.insert(alias);
+        return !error->isValid();
+    }
+
+
+    void resolveGeneralizedGroupProperty(const CompiledObject &component, CompiledBinding *binding);
     [[nodiscard]] bool wrapImplicitComponent(CompiledBinding *binding);
 
     [[nodiscard]] QQmlError findAndRegisterImplicitComponents(
             const CompiledObject *obj, const QQmlPropertyCache::ConstPtr &propertyCache);
     [[nodiscard]] QQmlError collectIdsAndAliases(int objectIndex);
     [[nodiscard]] QQmlError resolveAliases(int componentIndex);
+    void resolveGeneralizedGroupProperties(int componentIndex);
     [[nodiscard]] QQmlError resolveComponentsInInlineComponentRoot(int root);
 
     QString stringAt(int idx) const { return m_compiler->stringAt(idx); }
@@ -102,7 +126,6 @@ private:
     }
 
     ObjectContainer *m_compiler = nullptr;
-    QQmlEnginePrivate *m_enginePrivate = nullptr;
 
     // Implicit component insertion may have added objects and thus we also need
     // to extend the symmetric propertyCaches. Therefore, non-const propertyCaches.
@@ -111,16 +134,16 @@ private:
     // indices of the objects that are actually Component {}
     QVector<quint32> m_componentRoots;
     QVector<int> m_objectsWithAliases;
+    QVector<CompiledBinding *> m_generalizedGroupProperties;
+    QSet<const QV4::CompiledData::Alias *> resolvedAliases;
     typename ObjectContainer::IdToObjectMap m_idToObjectIndex;
 };
 
 template<typename ObjectContainer>
 QQmlComponentAndAliasResolver<ObjectContainer>::QQmlComponentAndAliasResolver(
         ObjectContainer *compiler,
-        QQmlEnginePrivate *enginePrivate,
         QQmlPropertyCacheVector *propertyCaches)
     : m_compiler(compiler)
-    , m_enginePrivate(enginePrivate)
     , m_propertyCaches(propertyCaches)
 {
 }
@@ -182,7 +205,7 @@ QQmlError QQmlComponentAndAliasResolver<ObjectContainer>::findAndRegisterImplici
             continue;
 
         if (!wrapImplicitComponent(binding))
-            return error(binding, tr("Cannot wrap implicit component"));
+            return error(binding, QQmlComponentAndAliasResolverBase::tr("Cannot wrap implicit component"));
     }
 
     return QQmlError();
@@ -278,7 +301,7 @@ QQmlError QQmlComponentAndAliasResolver<ObjectContainer>::resolve(int root)
         }
 
         if (!markAsComponent(i))
-            return error(obj, tr("Cannot mark object as component"));
+            return error(obj, QQmlComponentAndAliasResolverBase::tr("Cannot mark object as component"));
 
         // check if this object is the root
         if (i == 0) {
@@ -292,14 +315,14 @@ QQmlError QQmlComponentAndAliasResolver<ObjectContainer>::resolve(int root)
         }
 
         if (obj->functionCount() > 0)
-            return error(obj, tr("Component objects cannot declare new functions."));
+            return error(obj, QQmlComponentAndAliasResolverBase::tr("Component objects cannot declare new functions."));
         if (obj->propertyCount() > 0 || obj->aliasCount() > 0)
-            return error(obj, tr("Component objects cannot declare new properties."));
+            return error(obj, QQmlComponentAndAliasResolverBase::tr("Component objects cannot declare new properties."));
         if (obj->signalCount() > 0)
-            return error(obj, tr("Component objects cannot declare new signals."));
+            return error(obj, QQmlComponentAndAliasResolverBase::tr("Component objects cannot declare new signals."));
 
         if (obj->bindingCount() == 0)
-            return error(obj, tr("Cannot create empty component specification"));
+            return error(obj, QQmlComponentAndAliasResolverBase::tr("Cannot create empty component specification"));
 
         const auto rootBinding = obj->bindingsBegin();
         const auto bindingsEnd = obj->bindingsEnd();
@@ -310,12 +333,12 @@ QQmlError QQmlComponentAndAliasResolver<ObjectContainer>::resolve(int root)
             if (b->propertyNameIndex == 0)
                 continue;
 
-            return error(b, tr("Component elements may not contain properties other than id"));
+            return error(b, QQmlComponentAndAliasResolverBase::tr("Component elements may not contain properties other than id"));
         }
 
         if (auto b = rootBinding;
                 b->type() != QV4::CompiledData::Binding::Type_Object || ++b != bindingsEnd) {
-            return error(obj, tr("Invalid component body specification"));
+            return error(obj, QQmlComponentAndAliasResolverBase::tr("Invalid component body specification"));
         }
 
         // For the root object, we are going to collect ids/aliases and resolve them for as a
@@ -330,6 +353,7 @@ QQmlError QQmlComponentAndAliasResolver<ObjectContainer>::resolve(int root)
 
         m_idToObjectIndex.clear();
         m_objectsWithAliases.clear();
+        m_generalizedGroupProperties.clear();
 
         if (const QQmlError error = collectIdsAndAliases(rootBinding->value.objectIndex);
                 error.isValid()) {
@@ -340,17 +364,24 @@ QQmlError QQmlComponentAndAliasResolver<ObjectContainer>::resolve(int root)
 
         if (const QQmlError error = resolveAliases(m_componentRoots.at(i)); error.isValid())
             return error;
+
+        resolveGeneralizedGroupProperties(m_componentRoots.at(i));
     }
 
     // Collect ids and aliases for root
     m_idToObjectIndex.clear();
     m_objectsWithAliases.clear();
+    m_generalizedGroupProperties.clear();
 
     if (const QQmlError error = collectIdsAndAliases(root); error.isValid())
         return error;
 
     allocateNamedObjects(m_compiler->objectAt(root));
-    return resolveAliases(root);
+    if (const QQmlError error = resolveAliases(root); error.isValid())
+        return error;
+
+    resolveGeneralizedGroupProperties(root);
+    return QQmlError();
 }
 
 template<typename ObjectContainer>
@@ -360,7 +391,7 @@ QQmlError QQmlComponentAndAliasResolver<ObjectContainer>::collectIdsAndAliases(i
 
     if (obj->idNameIndex != 0) {
         if (m_idToObjectIndex.contains(obj->idNameIndex))
-            return error(obj->locationOfIdProperty, tr("id is not unique"));
+            return error(obj->locationOfIdProperty, QQmlComponentAndAliasResolverBase::tr("id is not unique"));
         setObjectId(objectIndex);
         m_idToObjectIndex.insert(obj->idNameIndex, objectIndex);
     }
@@ -375,9 +406,19 @@ QQmlError QQmlComponentAndAliasResolver<ObjectContainer>::collectIdsAndAliases(i
     for (auto binding = obj->bindingsBegin(), end = obj->bindingsEnd();
          binding != end; ++binding) {
         switch (binding->type()) {
+        case QV4::CompiledData::Binding::Type_GroupProperty: {
+            const auto *inner = m_compiler->objectAt(binding->value.objectIndex);
+            if (m_compiler->stringAt(inner->inheritedTypeNameIndex).isEmpty()) {
+                const auto cache = m_propertyCaches->at(objectIndex);
+                if (!cache || !cache->property(
+                            m_compiler->stringAt(binding->propertyNameIndex), nullptr, nullptr)) {
+                    m_generalizedGroupProperties.append(binding);
+                }
+            }
+        }
+        Q_FALLTHROUGH();
         case QV4::CompiledData::Binding::Type_Object:
         case QV4::CompiledData::Binding::Type_AttachedProperty:
-        case QV4::CompiledData::Binding::Type_GroupProperty:
             if (const QQmlError error = collectIdsAndAliases(binding->value.objectIndex);
                     error.isValid()) {
                 return error;
@@ -408,15 +449,12 @@ QQmlError QQmlComponentAndAliasResolver<ObjectContainer>::resolveAliases(int com
 
             QQmlError error;
             const auto &component = *m_compiler->objectAt(componentIndex);
-            const auto result = resolveAliasesInObject(component, objectIndex, &error);
+            const auto result
+                    = resolveAliasesInObject(component, objectIndex, &aliasCacheCreator, &error);
             if (error.isValid())
                 return error;
 
             if (result == AllAliasesResolved) {
-                QQmlError error = aliasCacheCreator.appendAliasesToPropertyCache(
-                            component, objectIndex, m_enginePrivate);
-                if (error.isValid())
-                    return error;
                 atLeastOneAliasResolved = true;
             } else if (result == SomeAliasesResolved) {
                 atLeastOneAliasResolved = true;
@@ -431,14 +469,23 @@ QQmlError QQmlComponentAndAliasResolver<ObjectContainer>::resolveAliases(int com
     if (!atLeastOneAliasResolved && !m_objectsWithAliases.isEmpty()) {
         const CompiledObject *obj = m_compiler->objectAt(m_objectsWithAliases.first());
         for (auto alias = obj->aliasesBegin(), end = obj->aliasesEnd(); alias != end; ++alias) {
-            if (!alias->hasFlag(QV4::CompiledData::Alias::Resolved))
-                return error(alias->location, tr("Circular alias reference detected"));
+            if (!resolvedAliases.contains(alias))
+                return error(alias->location, QQmlComponentAndAliasResolverBase::tr("Circular alias reference detected"));
         }
     }
 
     return QQmlError();
 }
 
+template<typename ObjectContainer>
+void QQmlComponentAndAliasResolver<ObjectContainer>::resolveGeneralizedGroupProperties(
+        int componentIndex)
+{
+    const auto &component = *m_compiler->objectAt(componentIndex);
+    for (CompiledBinding *binding : m_generalizedGroupProperties)
+        resolveGeneralizedGroupProperty(component, binding);
+}
+
 QT_END_NAMESPACE
 
 #endif // QQMLCOMPONENTANDALIASRESOLVER_P_H
