@@ -50,6 +50,10 @@
 #  endif
 #endif // Q_OS_WIN
 
+#ifdef Q_OS_DARWIN
+#  include <unistd.h>
+#endif
+
 QT_BEGIN_NAMESPACE
 
 using namespace Qt::StringLiterals;
@@ -619,9 +623,7 @@ void QQmlEngine::clearComponentCache()
     mm->gcStateMachine->timeLimit = std::move(oldLimit);
 
     handle()->trimCompilationUnits();
-    d->typeLoader.lock();
     d->typeLoader.clearCache();
-    d->typeLoader.unlock();
     QQmlMetaType::freeUnusedTypesAndCaches();
 }
 
@@ -694,7 +696,7 @@ QQmlContext *QQmlEngine::rootContext() const
 QQmlAbstractUrlInterceptor *QQmlEngine::urlInterceptor() const
 {
     Q_D(const QQmlEngine);
-    return d->urlInterceptors.last();
+    return d->typeLoader.urlInterceptors().last();
 }
 #endif
 
@@ -710,7 +712,7 @@ QQmlAbstractUrlInterceptor *QQmlEngine::urlInterceptor() const
 void QQmlEngine::addUrlInterceptor(QQmlAbstractUrlInterceptor *urlInterceptor)
 {
     Q_D(QQmlEngine);
-    d->urlInterceptors.append(urlInterceptor);
+    d->typeLoader.addUrlInterceptor(urlInterceptor);
 }
 
 /*!
@@ -724,7 +726,7 @@ void QQmlEngine::addUrlInterceptor(QQmlAbstractUrlInterceptor *urlInterceptor)
 void QQmlEngine::removeUrlInterceptor(QQmlAbstractUrlInterceptor *urlInterceptor)
 {
     Q_D(QQmlEngine);
-    d->urlInterceptors.removeOne(urlInterceptor);
+    d->typeLoader.removeUrlInterceptor(urlInterceptor);
 }
 
 /*!
@@ -734,10 +736,7 @@ void QQmlEngine::removeUrlInterceptor(QQmlAbstractUrlInterceptor *urlInterceptor
 QUrl QQmlEngine::interceptUrl(const QUrl &url, QQmlAbstractUrlInterceptor::DataType type) const
 {
     Q_D(const QQmlEngine);
-    QUrl result = url;
-    for (QQmlAbstractUrlInterceptor *interceptor : d->urlInterceptors)
-        result = interceptor->intercept(result, type);
-    return result;
+    return d->typeLoader.interceptUrl(url, type);
 }
 
 /*!
@@ -746,7 +745,7 @@ QUrl QQmlEngine::interceptUrl(const QUrl &url, QQmlAbstractUrlInterceptor::DataT
 QList<QQmlAbstractUrlInterceptor *> QQmlEngine::urlInterceptors() const
 {
     Q_D(const QQmlEngine);
-    return d->urlInterceptors;
+    return d->typeLoader.urlInterceptors();
 }
 
 QSharedPointer<QQmlImageProviderBase> QQmlEnginePrivate::imageProvider(const QString &providerId) const
@@ -772,10 +771,11 @@ QSharedPointer<QQmlImageProviderBase> QQmlEnginePrivate::imageProvider(const QSt
 void QQmlEngine::setNetworkAccessManagerFactory(QQmlNetworkAccessManagerFactory *factory)
 {
     Q_D(QQmlEngine);
-    QMutexLocker locker(&d->networkAccessManagerMutex);
-    d->networkAccessManagerFactory = factory;
+    d->typeLoader.setNetworkAccessManagerFactory(factory);
 }
 
+class QQmlEnginePublicAPIToken {};
+
 /*!
   Returns the current QQmlNetworkAccessManagerFactory.
 
@@ -784,27 +784,14 @@ void QQmlEngine::setNetworkAccessManagerFactory(QQmlNetworkAccessManagerFactory
 QQmlNetworkAccessManagerFactory *QQmlEngine::networkAccessManagerFactory() const
 {
     Q_D(const QQmlEngine);
-    return d->networkAccessManagerFactory;
+    return d->typeLoader.networkAccessManagerFactory().get(QQmlEnginePublicAPIToken());
 }
 
-QNetworkAccessManager *QQmlEnginePrivate::createNetworkAccessManager(QObject *parent) const
+QNetworkAccessManager *QQmlEnginePrivate::getNetworkAccessManager()
 {
-    QMutexLocker locker(&networkAccessManagerMutex);
-    QNetworkAccessManager *nam;
-    if (networkAccessManagerFactory) {
-        nam = networkAccessManagerFactory->create(parent);
-    } else {
-        nam = new QNetworkAccessManager(parent);
-    }
-
-    return nam;
-}
-
-QNetworkAccessManager *QQmlEnginePrivate::getNetworkAccessManager() const
-{
-    Q_Q(const QQmlEngine);
+    Q_Q(QQmlEngine);
     if (!networkAccessManager)
-        networkAccessManager = createNetworkAccessManager(const_cast<QQmlEngine*>(q));
+        networkAccessManager = typeLoader.createNetworkAccessManager(q);
     return networkAccessManager;
 }
 
@@ -822,8 +809,9 @@ QNetworkAccessManager *QQmlEnginePrivate::getNetworkAccessManager() const
 */
 QNetworkAccessManager *QQmlEngine::networkAccessManager() const
 {
+    // ### Qt7: This method is clearly not const since it _creates_ the network access manager.
     Q_D(const QQmlEngine);
-    return d->getNetworkAccessManager();
+    return const_cast<QQmlEnginePrivate *>(d)->getNetworkAccessManager();
 }
 #endif // qml_network
 
@@ -1623,7 +1611,7 @@ void QQmlEnginePrivate::cleanupScarceResources()
 void QQmlEngine::addImportPath(const QString& path)
 {
     Q_D(QQmlEngine);
-    d->importDatabase.addImportPath(path);
+    d->typeLoader.addImportPath(path);
 }
 
 /*!
@@ -1644,7 +1632,7 @@ void QQmlEngine::addImportPath(const QString& path)
 QStringList QQmlEngine::importPathList() const
 {
     Q_D(const QQmlEngine);
-    return d->importDatabase.importPathList();
+    return d->typeLoader.importPathList();
 }
 
 /*!
@@ -1662,7 +1650,7 @@ QStringList QQmlEngine::importPathList() const
 void QQmlEngine::setImportPathList(const QStringList &paths)
 {
     Q_D(QQmlEngine);
-    d->importDatabase.setImportPathList(paths);
+    d->typeLoader.setImportPathList(paths);
 }
 
 
@@ -1680,7 +1668,7 @@ void QQmlEngine::setImportPathList(const QStringList &paths)
 void QQmlEngine::addPluginPath(const QString& path)
 {
     Q_D(QQmlEngine);
-    d->importDatabase.addPluginPath(path);
+    d->typeLoader.addPluginPath(path);
 }
 
 /*!
@@ -1695,7 +1683,7 @@ void QQmlEngine::addPluginPath(const QString& path)
 QStringList QQmlEngine::pluginPathList() const
 {
     Q_D(const QQmlEngine);
-    return d->importDatabase.pluginPathList();
+    return d->typeLoader.pluginPathList();
 }
 
 /*!
@@ -1711,7 +1699,7 @@ QStringList QQmlEngine::pluginPathList() const
 void QQmlEngine::setPluginPathList(const QStringList &paths)
 {
     Q_D(QQmlEngine);
-    d->importDatabase.setPluginPathList(paths);
+    d->typeLoader.setPluginPathList(paths);
 }
 
 #if QT_CONFIG(library)
@@ -1734,8 +1722,7 @@ bool QQmlEngine::importPlugin(const QString &filePath, const QString &uri, QList
 {
     Q_D(QQmlEngine);
     QQmlTypeLoaderQmldirContent qmldir;
-    QQmlPluginImporter importer(
-                uri, QTypeRevision(), &d->importDatabase, &qmldir, &d->typeLoader, errors);
+    QQmlPluginImporter importer(uri, QTypeRevision(),  &qmldir, &d->typeLoader, errors);
     return importer.importDynamicPlugin(filePath, uri, false).isValid();
 }
 #endif
@@ -1895,7 +1882,7 @@ QJSValue QQmlEnginePrivate::singletonInstance<QJSValue>(const QQmlType &type)
         }
         QObject *o = component.beginCreate(q->rootContext());
         auto *compPriv = QQmlComponentPrivate::get(&component);
-        if (compPriv->state.hasUnsetRequiredProperties()) {
+        if (compPriv->hasUnsetRequiredProperties()) {
             /* We would only get the errors from the component after (complete)Create.
                 We can't call create, as we need to convertAndInsert before completeCreate (otherwise
                 tst_qqmllanguage::compositeSingletonCircular fails).
@@ -1903,7 +1890,7 @@ QJSValue QQmlEnginePrivate::singletonInstance<QJSValue>(const QQmlType &type)
                 So create the unset required component errors manually.
             */
             delete o;
-            const auto requiredProperties = compPriv->state.requiredProperties();
+            const auto requiredProperties = compPriv->requiredProperties();
             QList<QQmlError> errors (requiredProperties->size());
             for (const auto &reqProp: *requiredProperties)
                 errors.push_back(QQmlComponentPrivate::unsetRequiredPropertyToQQmlError(reqProp));
@@ -2173,9 +2160,7 @@ LoadHelper::LoadHelper(
     , m_typeName(typeName.toString())
     , m_mode(mode)
 {
-    m_typeLoader->lock();
     m_typeLoader->loadWithStaticData(this, QByteArray(), m_mode);
-    m_typeLoader->unlock();
 }
 
 void LoadHelper::registerCallback(QQmlComponentPrivate *callback)
