@@ -58,50 +58,125 @@ private:
 
 Q_GLOBAL_STATIC(PluginMap, qmlPluginsById); // stores the uri and the PluginLoaders
 
-static QVector<QStaticPlugin> makePlugins()
+struct StaticPluginMapping
 {
-    QVector<QStaticPlugin> plugins;
-    // To avoid traversing all static plugins for all imports, we cut down
-    // the list the first time called to only contain QML plugins:
-    const auto staticPlugins = QPluginLoader::staticPlugins();
-    for (const QStaticPlugin &plugin : staticPlugins) {
-        const QString iid = plugin.metaData().value(QLatin1String("IID")).toString();
-        if (iid == QLatin1String(QQmlEngineExtensionInterface_iid)
+    QStaticPlugin plugin;
+    QString metadataURI;
+};
+
+struct VersionedURI
+{
+    QString id;
+    QTypeRevision version;
+};
+
+/*
+ * This function verifies if staticPlugin is valid QML plugin by
+ * verifying the IID member of Metadata and
+ * checks whether the plugin instance has the correct parent plugin extension class.
+ * Checks the presence of URI member of Metadata and returns QJsonArray,
+ * which in majority cases contains only 1 element.
+ */
+static QJsonArray tryExtractQmlPluginURIs(const QStaticPlugin &plugin)
+{
+    const auto isQmlPlugin = [](const QStaticPlugin &plugin, auto &&pluginMetadata) -> bool {
+        const QString iid = pluginMetadata.value(QLatin1String("IID")).toString();
+        const bool isQmlExtensionIID = iid == QLatin1String(QQmlEngineExtensionInterface_iid)
                 || iid == QLatin1String(QQmlExtensionInterface_iid)
-                || iid == QLatin1String(QQmlExtensionInterface_iid_old)) {
-            if (Q_UNLIKELY(iid == QLatin1String(QQmlExtensionInterface_iid_old))) {
-                qWarning()
-                        << "Found plugin with old IID, this will be unsupported in upcoming Qt releases:"
-                        << plugin.metaData();
-            }
-            plugins.append(plugin);
+                || iid == QLatin1String(QQmlExtensionInterface_iid_old);
+        if (Q_UNLIKELY(iid == QLatin1String(QQmlExtensionInterface_iid_old))) {
+            qWarning() << QQmlImports::tr(
+                    "Found plugin with old IID, this will be unsupported in upcoming Qt "
+                    "releases:")
+                       << pluginMetadata;
+        }
+        if (!isQmlExtensionIID) {
+            return false;
+        }
+
+        // plugin.instance must not be called earlier to avoid instantiating
+        // non-QML plugins, potentially from an unexpected thread(typeloader vs
+        // main thread)
+        const auto *pluginInstance = plugin.instance();
+        return qobject_cast<const QQmlEngineExtensionPlugin *>(pluginInstance)
+                || qobject_cast<const QQmlExtensionPlugin *>(pluginInstance);
+    };
+
+    const auto &pluginMetadata = plugin.metaData();
+    if (!isQmlPlugin(plugin, pluginMetadata)) {
+        return {};
+    }
+
+    const QJsonArray metadataUriList = pluginMetadata.value(QStringLiteral("uri")).toArray();
+    if (metadataUriList.isEmpty()) {
+        qWarning() << QQmlImports::tr("qml static plugin with name \"%2\" has no metadata URI")
+                              .arg(plugin.instance()->metaObject()->className())
+                   << pluginMetadata;
+        return {};
+    }
+    return metadataUriList;
+}
+
+static QVector<StaticPluginMapping> staticQmlPlugins()
+{
+    QVector<StaticPluginMapping> qmlPlugins;
+    const auto staticPlugins = QPluginLoader::staticPlugins();
+    qmlPlugins.reserve(staticPlugins.size());
+
+    for (const auto &plugin : staticPlugins) {
+        // Filter out static plugins which are not Qml plugins
+        for (const QJsonValueConstRef &pluginURI : tryExtractQmlPluginURIs(plugin)) {
+            qmlPlugins.append({ plugin, pluginURI.toString() });
         }
     }
-    return plugins;
+    return qmlPlugins;
 }
 
 /*
     Returns the list of possible versioned URI combinations. For example, if \a uri is
-    QtQml.Models, \a vmaj is 2, and \a vmin is 0, this method returns the following:
+    (id = QtQml.Models, (vmaj = 2, vmin = 0)), this method returns the following:
     [QtQml.Models.2.0, QtQml.2.0.Models, QtQml.Models.2, QtQml.2.Models, QtQml.Models]
  */
-static QStringList versionUriList(const QString &uri, QTypeRevision version)
+static QStringList versionUriList(const VersionedURI &uri)
 {
     QStringList result;
     for (int mode = QQmlImports::FullyVersioned; mode <= QQmlImports::Unversioned; ++mode) {
-        int index = uri.size();
+        int index = uri.id.size();
         do {
-            QString versionUri = uri;
-            versionUri.insert(index, QQmlImports::versionString(
-                                  version, QQmlImports::ImportVersion(mode)));
+            QString versionUri = uri.id;
+            versionUri.insert(
+                    index,
+                    QQmlImports::versionString(uri.version, QQmlImports::ImportVersion(mode)));
             result += versionUri;
 
-            index = uri.lastIndexOf(u'.', index - 1);
+            index = uri.id.lastIndexOf(u'.', index - 1);
         } while (index > 0 && mode != QQmlImports::Unversioned);
     }
     return result;
 }
 
+/*
+    Get all static plugins that are QML plugins and has a meta data URI that matches with one of
+    \a versionUris, which is a list of all possible versioned URI combinations - see
+   versionUriList() above.
+ */
+static QVector<StaticPluginMapping> staticQmlPluginsMatchingURI(const VersionedURI &uri)
+{
+    static const auto qmlPlugins = staticQmlPlugins();
+
+    // Since plugin metadata URIs can be anything from
+    // fully versioned to unversioned, we need to compare with differnt version strings.
+    // If a module has several plugins, they must all have the same version. Start by
+    // populating pluginPairs with relevant plugins to cut the list short early on:
+    const QStringList versionedURIs = versionUriList(uri);
+    QVector<StaticPluginMapping> matches;
+    std::copy_if(qmlPlugins.begin(), qmlPlugins.end(), std::back_inserter(matches),
+                 [&](const auto &pluginMapping) {
+                     return versionedURIs.contains(pluginMapping.metadataURI);
+                 });
+    return matches;
+}
+
 static bool unloadPlugin(const std::pair<const QString, QmlPlugin> &plugin)
 {
     const auto &loader = plugin.second.loader;
@@ -124,6 +199,32 @@ static bool unloadPlugin(const std::pair<const QString, QmlPlugin> &plugin)
     return true;
 }
 
+/*!
+    \internal
+*/
+static QTypeRevision lockModule(const QString &uri, const QString &typeNamespace,
+                                             QTypeRevision version, QList<QQmlError> *errors)
+{
+    if (!version.hasMajorVersion()) {
+        version = QQmlMetaType::latestModuleVersion(uri);
+        if (!version.isValid())
+            errors->prepend(QQmlImports::moduleNotFoundError(uri, version));
+    }
+    if (version.hasMajorVersion() && !typeNamespace.isEmpty()
+        && !QQmlMetaType::protectModule(uri, version, true)) {
+        // Not being able to protect the module means there are not types registered for it,
+        // means the plugin we loaded didn't provide any, means we didn't find the module.
+        // We output the generic error message as depending on the load order of imports we may
+        // hit this path or another one that only figures "plugin is already loaded but module
+        // unavailable" and doesn't try to protect it anymore.
+        errors->prepend(QQmlImports::moduleNotFoundError(uri, version));
+        return QTypeRevision();
+    }
+
+    return version;
+}
+
+
 void qmlClearEnginePlugins()
 {
     PluginMapPtr plugins(qmlPluginsById());
@@ -168,7 +269,7 @@ void QQmlPluginImporter::finalizePlugin(QObject *instance, const QString &plugin
     // only called from the engine specific loader thread and importDynamicPlugin as well as
     // importStaticPlugin are the only places of access.
 
-    database->initializedPlugins.insert(pluginId);
+    typeLoader->setPluginInitialized(pluginId);
     if (auto *extensionIface = qobject_cast<QQmlExtensionInterface *>(instance))
         typeLoader->initializeEngine(extensionIface, uri.toUtf8().constData());
     else if (auto *engineIface = qobject_cast<QQmlEngineExtensionInterface *>(instance))
@@ -195,8 +296,7 @@ QTypeRevision QQmlPluginImporter::importStaticPlugin(QObject *instance, const QS
                 return QTypeRevision();
             }
 
-            importVersion = QQmlImportDatabase::lockModule(
-                        uri, qmldir->typeNamespace(), importVersion, errors);
+            importVersion = lockModule(uri, qmldir->typeNamespace(), importVersion, errors);
             if (!importVersion.isValid())
                 return QTypeRevision();
         }
@@ -207,7 +307,7 @@ QTypeRevision QQmlPluginImporter::importStaticPlugin(QObject *instance, const QS
         // other QML loader threads and thus not process the initializeEngine call).
     }
 
-    if (!database->initializedPlugins.contains(pluginId))
+    if (!typeLoader->isPluginInitialized(pluginId))
         finalizePlugin(instance, pluginId);
 
     return QQmlImports::validVersion(importVersion);
@@ -219,7 +319,7 @@ QTypeRevision QQmlPluginImporter::importDynamicPlugin(
     QObject *instance = nullptr;
     QTypeRevision importVersion = version;
 
-    const bool engineInitialized = database->initializedPlugins.contains(pluginId);
+    const bool engineInitialized = typeLoader->isPluginInitialized(pluginId);
     {
         PluginMapPtr plugins(qmlPluginsById());
         bool typesRegistered = plugins->find(pluginId) != plugins->end();
@@ -234,14 +334,13 @@ QTypeRevision QQmlPluginImporter::importDynamicPlugin(
                     // try again with plugin
                     break;
                 case QQmlMetaType::RegistrationResult::Success:
-                    importVersion = QQmlImportDatabase::lockModule(
-                                uri, qmldir->typeNamespace(), importVersion, errors);
+                    importVersion = lockModule(uri, qmldir->typeNamespace(), importVersion, errors);
                     if (!importVersion.isValid())
                         return QTypeRevision();
                     // instance and loader intentionally left at nullptr
                     plugins->insert(std::make_pair(pluginId, QmlPlugin()));
                     // Not calling initializeEngine with null instance
-                    database->initializedPlugins.insert(pluginId);
+                    typeLoader->setPluginInitialized(pluginId);
                     return importVersion;
                 case QQmlMetaType::RegistrationResult::Failure:
                     return QTypeRevision();
@@ -263,7 +362,7 @@ QTypeRevision QQmlPluginImporter::importDynamicPlugin(
                     if (errors) {
                         QQmlError error;
                         error.setDescription(
-                                    QQmlImportDatabase::tr("File name case mismatch for \"%1\"")
+                                    QQmlImports::tr("File name case mismatch for \"%1\"")
                                     .arg(absoluteFilePath));
                         errors->prepend(error);
                     }
@@ -292,8 +391,7 @@ QTypeRevision QQmlPluginImporter::importDynamicPlugin(
                     return QTypeRevision();
                 }
 
-                importVersion = QQmlImportDatabase::lockModule(
-                            uri, qmldir->typeNamespace(), importVersion, errors);
+                importVersion = lockModule(uri, qmldir->typeNamespace(), importVersion, errors);
                 if (!importVersion.isValid())
                     return QTypeRevision();
             } else {
@@ -326,7 +424,7 @@ QTypeRevision QQmlPluginImporter::importDynamicPlugin(
   \internal
 
   Searches for a plugin called \a baseName in \a qmldirPluginPath, taking the
-  path of the qmldir file itself, and the plugin paths of the QQmlImportDatabase
+  path of the qmldir file itself, and the plugin paths of the QQmlTypeLoader
   into account.
 
   The baseName is amended with a platform-dependent prefix and suffix to
@@ -341,7 +439,7 @@ QTypeRevision QQmlPluginImporter::importDynamicPlugin(
   \endtable
 
   If the \a qmldirPluginPath is absolute, it is searched first. Then each of the
-  filePluginPath entries in the QQmlImportDatabase is checked in turn. If the
+  filePluginPath entries in the QQmlTypeLoader is checked in turn. If the
   entry is relative, it is resolved on top of the path of the qmldir file,
   otherwise it is taken verbatim. If a "." is found in the filePluginPath, and
   \a qmldirPluginPath is relative, then \a qmldirPluginPath is used in its
@@ -388,7 +486,7 @@ QString QQmlPluginImporter::resolvePlugin(const QString &qmldirPluginPath, const
     };
 #endif
 
-    QStringList searchPaths = database->filePluginPath;
+    QStringList searchPaths = typeLoader->pluginPathList();
     bool qmldirPluginPathIsRelative = QDir::isRelativePath(qmldirPluginPath);
     if (!qmldirPluginPathIsRelative)
         searchPaths.prepend(qmldirPluginPath);
@@ -454,47 +552,6 @@ QString QQmlPluginImporter::resolvePlugin(const QString &qmldirPluginPath, const
     return QString();
 }
 
-/*
-    Get all static plugins that are QML plugins and has a meta data URI that matches with one of
-    \a versionUris, which is a list of all possible versioned URI combinations - see versionUriList()
-    above.
- */
-bool QQmlPluginImporter::populatePluginDataVector(QVector<StaticPluginData> &result, const QStringList &versionUris)
-{
-    static const QVector<QStaticPlugin> plugins = makePlugins();
-    for (const QStaticPlugin &plugin : plugins) {
-        // Since a module can list more than one plugin,
-        // we keep iterating even after we found a match.
-        QObject *instance = plugin.instance();
-        if (qobject_cast<QQmlEngineExtensionPlugin *>(instance)
-                || qobject_cast<QQmlExtensionPlugin *>(instance)) {
-            const QJsonArray metaTagsUriList =
-                    plugin.metaData().value(QStringLiteral("uri")).toArray();
-            if (metaTagsUriList.isEmpty()) {
-                if (errors) {
-                    QQmlError error;
-                    error.setDescription(QQmlImportDatabase::tr(
-                                             "static plugin for module \"%1\" with name \"%2\" "
-                                             "has no metadata URI")
-                                         .arg(uri, QString::fromUtf8(
-                                                  instance->metaObject()->className())));
-                    error.setUrl(QUrl::fromLocalFile(qmldir->qmldirLocation()));
-                    errors->prepend(error);
-                }
-                return false;
-            }
-            // A plugin can be set up to handle multiple URIs, so go through the list:
-            for (const QJsonValueConstRef metaTagUri : metaTagsUriList) {
-                if (versionUris.contains(metaTagUri.toString())) {
-                    result.append({ plugin, metaTagsUriList });
-                    break;
-                }
-            }
-        }
-    }
-    return true;
-}
-
 QTypeRevision QQmlPluginImporter::importPlugins() {
     const auto qmldirPlugins = qmldir->plugins();
     const int qmldirPluginCount = qmldirPlugins.size();
@@ -508,92 +565,75 @@ QTypeRevision QQmlPluginImporter::importPlugins() {
             && qmldirPath.endsWith(u'/' + QString(uri).replace(u'.', u'/'));
     const QString moduleId = canUseUris ? uri : qmldir->qmldirLocation();
 
-    if (!database->modulesForWhichPluginsHaveBeenLoaded.contains(moduleId)) {
-        // First search for listed qmldir plugins dynamically. If we cannot resolve them all, we
-        // continue searching static plugins that has correct metadata uri. Note that since we
-        // only know the uri for a static plugin, and not the filename, we cannot know which
-        // static plugin belongs to which listed plugin inside qmldir. And for this reason,
-        // mixing dynamic and static plugins inside a single module is not recommended.
+    if (typeLoader->isModulePluginProcessingDone(moduleId)) {
+        return QQmlImports::validVersion(importVersion);
+    }
 
-        int dynamicPluginsFound = 0;
-        int staticPluginsLoaded = 0;
+    // First search for listed qmldir plugins dynamically. If we cannot resolve them all, we
+    // continue searching static plugins that has correct metadata uri. Note that since we
+    // only know the uri for a static plugin, and not the filename, we cannot know which
+    // static plugin belongs to which listed plugin inside qmldir. And for this reason,
+    // mixing dynamic and static plugins inside a single module is not recommended.
 
-        for (const QQmlDirParser::Plugin &plugin : qmldirPlugins) {
-            const QString resolvedFilePath = resolvePlugin(plugin.path, plugin.name);
+    int dynamicPluginsFound = 0;
+    int staticPluginsLoaded = 0;
 
-            if (!canUseUris && resolvedFilePath.isEmpty())
+    for (const QQmlDirParser::Plugin &plugin : qmldirPlugins) {
+        const QString resolvedFilePath = resolvePlugin(plugin.path, plugin.name);
+
+        if (!canUseUris && resolvedFilePath.isEmpty())
+            continue;
+
+        importVersion = importDynamicPlugin(
+                resolvedFilePath, canUseUris ? uri : QFileInfo(resolvedFilePath).absoluteFilePath(),
+                plugin.optional);
+        if (importVersion.isValid())
+            ++dynamicPluginsFound;
+        else if (!resolvedFilePath.isEmpty())
+            return QTypeRevision();
+    }
+
+    if (dynamicPluginsFound < qmldirPluginCount) {
+        // Check if the missing plugins can be resolved statically. We do this by looking at
+        // the URIs embedded in a plugins meta data.
+        const auto pluginPairs = staticQmlPluginsMatchingURI({ uri, importVersion });
+
+        for (const auto &pluginWithURI : std::as_const(pluginPairs)) {
+            QObject *instance = pluginWithURI.plugin.instance();
+            importVersion = importStaticPlugin(
+                    instance, canUseUris ? uri : QString::asprintf("%p", instance));
+            if (!importVersion.isValid()) {
                 continue;
-
-            importVersion = importDynamicPlugin(
-                        resolvedFilePath,
-                        canUseUris ? uri : QFileInfo(resolvedFilePath).absoluteFilePath(),
-                        plugin.optional);
-            if (importVersion.isValid())
-                ++dynamicPluginsFound;
-            else if (!resolvedFilePath.isEmpty())
-                return QTypeRevision();
-        }
-
-        if (dynamicPluginsFound < qmldirPluginCount) {
-            // Check if the missing plugins can be resolved statically. We do this by looking at
-            // the URIs embedded in a plugins meta data. Since those URIs can be anything from
-            // fully versioned to unversioned, we need to compare with differnt version strings.
-            // If a module has several plugins, they must all have the same version. Start by
-            // populating pluginPairs with relevant plugins to cut the list short early on:
-            const QStringList versionUris = versionUriList(uri, importVersion);
-            QVector<StaticPluginData> pluginPairs;
-            if (!populatePluginDataVector(pluginPairs, versionUris))
-                return QTypeRevision();
-
-            for (const QString &versionUri : versionUris) {
-                int staticPluginsFound = 0;
-                for (const StaticPluginData &pair : std::as_const(pluginPairs)) {
-                    for (const QJsonValueConstRef metaTagUri : pair.uriList) {
-                        if (versionUri == metaTagUri.toString()) {
-                            staticPluginsFound++;
-                            QObject *instance = pair.plugin.instance();
-                            importVersion = importStaticPlugin(
-                                        instance,
-                                        canUseUris ? uri : QString::asprintf("%p", instance));
-                            if (!importVersion.isValid()) {
-                                continue;
-                            }
-                            staticPluginsLoaded++;
-                            qCDebug(lcQmlImport)
-                                    << "importExtension" << "loaded static plugin " << versionUri;
-                            break;
-                        }
-                    }
-                }
-                if (staticPluginsLoaded > 0)
-                    break;
-                if (staticPluginsFound > 0) {
-                    // found matched plugins, but failed to load, early exit to preserve the error
-                    return QTypeRevision();
-                }
             }
+            staticPluginsLoaded++;
+            qCDebug(lcQmlImport) << "importExtension" << "loaded static plugin "
+                                 << pluginWithURI.metadataURI;
         }
-
-        if ((dynamicPluginsFound + staticPluginsLoaded) < qmldirPluginCount) {
-            if (errors) {
-                QQmlError error;
-                if (qmldirPluginCount > 1 && staticPluginsLoaded > 0) {
-                    error.setDescription(QQmlImportDatabase::tr(
-                                                 "could not resolve all plugins for module \"%1\"")
-                                                 .arg(uri));
-                } else {
-                    error.setDescription(
-                            QQmlImportDatabase::tr("module \"%1\" plugin \"%2\" not found")
-                                    .arg(uri, qmldirPlugins[dynamicPluginsFound].name));
-                }
-                error.setUrl(QUrl::fromLocalFile(qmldir->qmldirLocation()));
-                errors->prepend(error);
-            }
+        if (!pluginPairs.empty() && staticPluginsLoaded == 0) {
+            // found matched plugins, but failed to load, early exit to preserve the error
             return QTypeRevision();
         }
-
-        database->modulesForWhichPluginsHaveBeenLoaded.insert(moduleId);
     }
+
+    if ((dynamicPluginsFound + staticPluginsLoaded) < qmldirPluginCount) {
+        if (errors) {
+            QQmlError error;
+            if (qmldirPluginCount > 1 && staticPluginsLoaded > 0) {
+                error.setDescription(
+                        QQmlImports::tr("could not resolve all plugins for module \"%1\"")
+                                .arg(uri));
+            } else {
+                error.setDescription(QQmlImports::tr("module \"%1\" plugin \"%2\" not found")
+                                             .arg(uri, qmldirPlugins[dynamicPluginsFound].name));
+            }
+            error.setUrl(QUrl::fromLocalFile(qmldir->qmldirLocation()));
+            errors->prepend(error);
+        }
+        return QTypeRevision();
+    }
+
+    typeLoader->setModulePluginProcessingDone(moduleId);
+
     return QQmlImports::validVersion(importVersion);
 }
 
