@@ -10,17 +10,18 @@
 
 #include <private/qqmlglobal_p.h>
 
-#include <private/qv4object_p.h>
-#include <private/qv4variantobject_p.h>
-#include <private/qv4variantassociationobject_p.h>
-#include <private/qv4functionobject_p.h>
-#include <private/qv4scopedvalue_p.h>
-#include <private/qv4jscall_p.h>
-#include <private/qv4qobjectwrapper_p.h>
-#include <private/qv4sequenceobject_p.h>
 #include <private/qqmlpropertycachecreator_p.h>
 #include <private/qqmlpropertycachemethodarguments_p.h>
 #include <private/qqmlvaluetypewrapper_p.h>
+#include <private/qv4functionobject_p.h>
+#include <private/qv4jscall_p.h>
+#include <private/qv4object_p.h>
+#include <private/qv4qobjectwrapper_p.h>
+#include <private/qv4runtime_p.h>
+#include <private/qv4scopedvalue_p.h>
+#include <private/qv4sequenceobject_p.h>
+#include <private/qv4variantassociationobject_p.h>
+#include <private/qv4variantobject_p.h>
 
 #include <QtCore/qsequentialiterable.h>
 
@@ -220,14 +221,19 @@ void QQmlVMEMetaObjectEndpoint::tryConnect()
         int sigIdx = aliasId + metaObject->propCount();
         metaObject->activate(metaObject->object, sigIdx, nullptr);
     } else if (const QV4::CompiledData::Object *compiledObject = metaObject->findCompiledObject()) {
-        const QV4::CompiledData::Alias *aliasData = &compiledObject->aliasTable()[aliasId];
-        if (!aliasData->isObjectAlias()) {
+        const QQmlPropertyData *aliasProperty
+                = metaObject->cache->property(metaObject->aliasOffset() + aliasId);
+        const int targetPropertyIndex = aliasProperty ? aliasProperty->aliasTarget() : -1;
+
+        if (targetPropertyIndex != -1) {
+            const QV4::CompiledData::Alias *aliasData = &compiledObject->aliasTable()[aliasId];
+
             QQmlRefPointer<QQmlContextData> ctxt = metaObject->ctxt;
             QObject *target = ctxt->idValue(aliasData->targetObjectId());
             if (!target)
                 return;
 
-            QQmlPropertyIndex encodedIndex = QQmlPropertyIndex::fromEncoded(aliasData->encodedMetaPropertyIndex);
+            QQmlPropertyIndex encodedIndex = QQmlPropertyIndex::fromEncoded(targetPropertyIndex);
             int coreIndex = encodedIndex.coreIndex();
             int valueTypeIndex = encodedIndex.valueTypeIndex();
             const QQmlPropertyData *pd = QQmlData::ensurePropertyCache(target)->property(coreIndex);
@@ -374,7 +380,10 @@ bool QQmlInterceptorMetaObject::doIntercept(QMetaObject::Call c, int id, void **
                     // current value is explicitly set.
                     // So, we cannot return here if prevComponentValue == newComponentValue.
                     valueType->writeOnGadget(valueProp, std::move(prevComponentValue));
-                    valueType->write(object, id, QQmlPropertyData::DontRemoveBinding | QQmlPropertyData::BypassInterceptor);
+                    valueType->write(
+                        object, id,
+                        QQmlPropertyData::DontRemoveBinding | QQmlPropertyData::BypassInterceptor,
+                        QV4::ReferenceObject::AllProperties);
 
                     vi->write(newComponentValue);
                     return true;
@@ -1079,7 +1088,10 @@ int QQmlVMEMetaObject::metaCall(QObject *o, QMetaObject::Call c, int _id, void *
 
                 connectAlias(compiledObject, id);
 
-                if (aliasData->isObjectAlias()) {
+                const QQmlPropertyData *aliasProperty = cache->property(aliasOffset() + id);
+                const int targetPropertyIndex = aliasProperty ? aliasProperty->aliasTarget() : -1;
+
+                if (targetPropertyIndex == -1) {
                     *reinterpret_cast<QObject **>(a[0]) = target;
                     return -1;
                 }
@@ -1088,7 +1100,8 @@ int QQmlVMEMetaObject::metaCall(QObject *o, QMetaObject::Call c, int _id, void *
                 if (!targetDData)
                     return -1;
 
-                QQmlPropertyIndex encodedIndex = QQmlPropertyIndex::fromEncoded(aliasData->encodedMetaPropertyIndex);
+                QQmlPropertyIndex encodedIndex
+                        = QQmlPropertyIndex::fromEncoded(targetPropertyIndex);
                 int coreIndex = encodedIndex.coreIndex();
                 const int valueTypePropertyIndex = encodedIndex.valueTypeIndex();
 
@@ -1100,8 +1113,10 @@ int QQmlVMEMetaObject::metaCall(QObject *o, QMetaObject::Call c, int _id, void *
                         if (flags & QQmlPropertyData::RemoveBindingOnAliasWrite) {
                             QQmlData *targetData = QQmlData::get(target);
                             if (targetData && targetData->hasBindingBit(coreIndex)) {
-                                QQmlPropertyPrivate::removeBinding(target, encodedIndex);
-                                targetData->clearBindingBit(coreIndex);
+                                if (QQmlPropertyPrivate::removeBinding(
+                                            target, encodedIndex, QQmlPropertyPrivate::None)) {
+                                    targetData->clearBindingBit(coreIndex);
+                                }
                             }
                         }
                     }
@@ -1265,9 +1280,13 @@ void QQmlVMEMetaObject::writeVarProperty(int id, const QV4::Value &value)
     if (!md)
         return;
 
+    const QV4::Value &oldValue = (*md)[id];
+    if (QV4::RuntimeHelpers::strictEqual(oldValue, value))
+        return;
+
     // Importantly, if the current value is a scarce resource, we need to ensure that it
     // gets automatically released by the engine if no other references to it exist.
-    const QV4::VariantObject *oldVariant = (md->data() + id)->as<QV4::VariantObject>();
+    const QV4::VariantObject *oldVariant = oldValue.as<QV4::VariantObject>();
     if (oldVariant)
         oldVariant->removeVmePropertyReference();
 
@@ -1430,8 +1449,11 @@ bool QQmlVMEMetaObject::aliasTarget(int index, QObject **target, int *coreIndex,
     if (!*target)
         return false;
 
-    if (!aliasData->isObjectAlias()) {
-        QQmlPropertyIndex encodedIndex = QQmlPropertyIndex::fromEncoded(aliasData->encodedMetaPropertyIndex);
+    const QQmlPropertyData *aliasProperty = cache->property(aliasOffset() + aliasId);
+    const int targetPropertyIndex = aliasProperty ? aliasProperty->aliasTarget() : -1;
+
+    if (targetPropertyIndex != -1) {
+        QQmlPropertyIndex encodedIndex = QQmlPropertyIndex::fromEncoded(targetPropertyIndex);
         *coreIndex = encodedIndex.coreIndex();
         *valueTypeIndex = encodedIndex.valueTypeIndex();
     }
