@@ -1030,7 +1030,7 @@ private:
     QByteArray m_data;
     int m_redirectCount;
 
-    typedef QPair<QByteArray, QByteArray> HeaderPair;
+    typedef std::pair<QByteArray, QByteArray> HeaderPair;
     typedef QList<HeaderPair> HeadersList;
     HeadersList m_headersList;
     void fillHeadersList();
@@ -1234,6 +1234,13 @@ void QQmlXMLHttpRequest::requestFromUrl(const QUrl &url)
         }
     }
 
+    if (!m_nam) {
+        qWarning() << "XMLHttpRequest:" << qPrintable(m_method)
+                   << "No network accessmanager available.";
+        m_state = Done;
+        return;
+    }
+
     if (m_method == QLatin1String("GET")) {
         m_network = networkAccessManager()->get(request);
     } else if (m_method == QLatin1String("HEAD")) {
