@@ -50,6 +50,7 @@ QQmlDirParser QQmlJSImporter::createQmldirParserForFile(const QString &filename,
     QQmlDirParser parser;
     if (f.open(QFile::ReadOnly)) {
         parser.parse(QString::fromUtf8(f.readAll()));
+        import->warnings.append(parser.errors(filename));
     } else {
         import->warnings.append({
             QStringLiteral("Could not open qmldir file: ") + filename,
@@ -417,6 +418,19 @@ static bool isVersionAllowed(const QQmlJSScope::Export &exportEntry,
             || exportVersion.minorVersion() <= importVersion.minorVersion();
 }
 
+/* This is a _rough_ heuristic; only meant for qmllint to avoid warnings about commonconstructs.
+   We might want to improve it in the future if it causes issues
+*/
+static bool fileSelectedScopesAreCompatibleHeuristic(const QQmlJSScope::ConstPtr &scope1, const QQmlJSScope::ConstPtr &scope2) {
+    for (const auto &[propertyName, prop]: scope1->properties().asKeyValueRange())
+        if (!scope2->hasProperty(propertyName))
+            return false;
+    for (const auto &[methodName, method]: scope1->methods().asKeyValueRange())
+        if (!scope2->hasMethod(methodName))
+            return false;
+    return true;
+}
+
 void QQmlJSImporter::processImport(
         const QQmlJS::Import &importDescription, const QQmlJSImporter::Import &import,
         QQmlJSImporter::AvailableTypes *types)
@@ -480,6 +494,36 @@ void QQmlJSImporter::processImport(
                 case LowerVersion:
                     break;
                 case SameVersion: {
+                    if (m_flags & QQmlJSImporterFlag::TolerateFileSelectors) {
+                        auto isFileSelected = [](const QQmlJSScope::ConstPtr &scope) -> bool
+                        {
+                            return scope->filePath().contains(u"+");
+                        };
+                        auto warnAboutFileSelector = [&](const QString &path) {
+                            types->warnings.append({
+                                QStringLiteral("Type %1 is ambiguous due to file selector usage, ignoring %2.")
+                                        .arg(qmlName, path),
+                                QtInfoMsg,
+                                QQmlJS::SourceLocation()
+                            });
+                        };
+                        if (it->scope) {
+                            if (isFileSelected(val.scope)) {
+                                // new entry is file selected, skip if it looks compatible
+                                if (fileSelectedScopesAreCompatibleHeuristic(it->scope, val.scope)) {
+                                    warnAboutFileSelector(val.scope->filePath());
+                                    continue;
+                                }
+                            } else if (isFileSelected(it->scope)) {
+                                // the first scope we saw is file selected. If they are compatible
+                                // we update to the new one without file selector
+                                if (fileSelectedScopesAreCompatibleHeuristic(it->scope, val.scope)) {
+                                    warnAboutFileSelector(it->scope->filePath());
+                                    break;
+                                }
+                            }
+                        }
+                    }
                     types->warnings.append({
                         QStringLiteral("Ambiguous type detected. "
                                        "%1 %2.%3 is defined multiple times.")
@@ -597,6 +641,11 @@ void QQmlJSImporter::processImport(
         // Otherwise we have already done it in localFile2ScopeTree()
         if (!val.scope.factory() && val.scope->baseType().isNull()) {
 
+            // ignore the scope currently analyzed by QQmlJSImportVisitor, as its only populated
+            // after importing the implicit directory.
+            if (val.scope->baseTypeName() == "$InProcess$"_L1)
+                continue;
+
             // Composite types use QML names, and we should have resolved those already.
             // ... except that old qmltypes files might specify composite types with C++ names.
             // Warn about those.
@@ -774,7 +823,7 @@ bool QQmlJSImporter::importHelper(const QString &module, AvailableTypes *types,
                     QQmlJS::ContextualTypes::INTERNAL, {}, {}, types->cppNames.arrayType())));
     m_cachedImportTypes[cacheKey] = cacheTypes;
 
-    const QPair<QString, QTypeRevision> importId { module, version };
+    const std::pair<QString, QTypeRevision> importId { module, version };
     const auto it = m_seenImports.constFind(importId);
 
     if (it != m_seenImports.constEnd()) {
@@ -811,7 +860,7 @@ bool QQmlJSImporter::importHelper(const QString &module, AvailableTypes *types,
         modulePaths = qQmlResolveImportPaths(module, m_importPaths, version);
     }
 
-    for (auto const &modulePath : modulePaths) {
+    for (auto const &modulePath : std::as_const(modulePaths)) {
         QString qmldirPath = modulePath + SlashQmldir;
         if (modulePath.startsWith(u':')) {
             if (module == "QML"_L1) {
@@ -887,6 +936,25 @@ QQmlJSScope::Ptr QQmlJSImporter::localFile2ScopeTree(const QString &filePath)
                       this, sourceFolderFile)) });
 }
 
+/*!
+\internal
+Add scopes manually created and QQmlJSImportVisited to QQmlJSImporter.
+This allows theses scopes to not get loaded twice during linting, for example.
+
+Returns false if the importer contains a scope different than \a scope for the same
+QQmlJSScope::filePath.
+*/
+bool QQmlJSImporter::registerScope(const QQmlJSScope::Ptr &scope)
+{
+    Q_ASSERT(!scope.factory());
+
+    QQmlJSScope::Ptr &existing = m_importedFiles[scope->filePath()];
+    if (existing)
+        return existing == scope;
+    existing = scope;
+    return true;
+}
+
 QQmlJSScope::Ptr QQmlJSImporter::importFile(const QString &file)
 {
     return localFile2ScopeTree(file);
