@@ -19,6 +19,8 @@
 #include <QtCore/qscopedpointer.h>
 
 #include <QtQmlCompiler/private/qqmlsa_p.h>
+#include <QtQmlCompiler/private/qqmljsloggingutils_p.h>
+#include <QtQmlCompiler/private/qqmljslintervisitor_p.h>
 
 #if QT_CONFIG(library)
 #    include <QtCore/qdiriterator.h>
@@ -37,40 +39,109 @@ QT_BEGIN_NAMESPACE
 
 using namespace Qt::StringLiterals;
 
+class HasFunctionDefinitionVisitor final : public QQmlJS::AST::Visitor
+{
+public:
+    bool visit(QQmlJS::AST::FunctionDeclaration *functionDeclaration) override
+    {
+        m_result = !functionDeclaration->name.isEmpty();
+        return false;
+    }
+
+    void throwRecursionDepthError() override { }
+    bool result() const { return m_result; }
+    void reset() { m_result = false; }
+
+private:
+    bool m_result = false;
+};
+
+class UnreachableVisitor final : public QQmlJS::AST::Visitor
+{
+public:
+    UnreachableVisitor(QQmlJSLogger *logger) : m_logger(logger) { }
+
+    bool containsFunctionDeclaration(QQmlJS::AST::Node *node)
+    {
+        m_hasFunctionDefinition.reset();
+        node->accept(&m_hasFunctionDefinition);
+        return m_hasFunctionDefinition.result();
+    }
+
+    bool visit(QQmlJS::AST::StatementList *unreachable) override
+    {
+        QQmlJS::SourceLocation location;
+        auto report = [this, &location]() {
+            if (location.isValid()) {
+                m_logger->log(u"Unreachable code"_s, qmlUnreachableCode, location);
+            }
+            location = QQmlJS::SourceLocation{};
+        };
+
+        for (auto it = unreachable; it && it->statement; it = it->next) {
+            if (containsFunctionDeclaration(it->statement)) {
+                report();
+                continue; // don't warn about the location of the function declaration
+            }
+            location = combine(location,
+                               combine(it->statement->firstSourceLocation(),
+                                       it->statement->lastSourceLocation()));
+        }
+        report();
+        return false;
+    }
+    void throwRecursionDepthError() override { }
+
+private:
+    QQmlJSLogger *m_logger = nullptr;
+    HasFunctionDefinitionVisitor m_hasFunctionDefinition;
+};
+
 class CodegenWarningInterface final : public QV4::Compiler::CodegenWarningInterface
 {
 public:
-    CodegenWarningInterface(QQmlJSLogger *logger) : m_logger(logger) { }
+    CodegenWarningInterface(QQmlJSLogger *logger) : m_logger(logger), m_unreachableVisitor(logger)
+    {
+    }
 
     void reportVarUsedBeforeDeclaration(const QString &name, const QString &fileName,
                                         QQmlJS::SourceLocation declarationLocation,
                                         QQmlJS::SourceLocation accessLocation) override
     {
         Q_UNUSED(fileName)
-        m_logger->log(
-                u"Variable \"%1\" is used here before its declaration. The declaration is at %2:%3."_s
-                        .arg(name)
-                        .arg(declarationLocation.startLine)
-                        .arg(declarationLocation.startColumn),
-                qmlVarUsedBeforeDeclaration, accessLocation);
+
+        m_logger->log("Identifier '%1' is used here before its declaration."_L1.arg(name),
+                      qmlVarUsedBeforeDeclaration, accessLocation);
+        m_logger->log("Note: declaration of '%1' here"_L1.arg(name), qmlVarUsedBeforeDeclaration,
+                      declarationLocation, true, true, {}, {}, accessLocation.startLine);
     }
 
+    void reportFunctionUsedBeforeDeclaration(const QString &name, const QString &fileName,
+                                             QQmlJS::SourceLocation declarationLocation,
+                                             QQmlJS::SourceLocation accessLocation) override
+    {
+        Q_UNUSED(fileName)
+
+        m_logger->log("Function '%1' is used here before its declaration."_L1.arg(name),
+                      qmlFunctionUsedBeforeDeclaration, accessLocation);
+        m_logger->log("Note: declaration of '%1' here"_L1.arg(name),
+                      qmlFunctionUsedBeforeDeclaration, declarationLocation);
+    }
+
+    UnreachableVisitor *unreachableVisitor() override { return &m_unreachableVisitor; }
+
 private:
     QQmlJSLogger *m_logger;
+    UnreachableVisitor m_unreachableVisitor;
 };
 
-QString QQmlJSLinter::defaultPluginPath()
-{
-    return QLibraryInfo::path(QLibraryInfo::PluginsPath) + QDir::separator() + u"qmllint";
-}
-
-QQmlJSLinter::QQmlJSLinter(const QStringList &importPaths, const QStringList &pluginPaths,
+QQmlJSLinter::QQmlJSLinter(const QStringList &importPaths, const QStringList &extraPluginPaths,
                            bool useAbsolutePath)
     : m_useAbsolutePath(useAbsolutePath),
       m_enablePlugins(true),
-      m_importer(importPaths, nullptr, true)
+      m_importer(importPaths, nullptr, QQmlJSImporterFlags(UseOptionalImports | TolerateFileSelectors))
 {
-    m_plugins = loadPlugins(pluginPaths);
+    m_plugins = loadPlugins(extraPluginPaths);
 }
 
 QQmlJSLinter::Plugin::Plugin(QQmlJSLinter::Plugin &&plugin) noexcept
@@ -163,7 +234,7 @@ bool QQmlJSLinter::Plugin::parseMetaData(const QJsonObject &metaData, QString pl
 
     QJsonArray categories = pluginMetaData[u"loggingCategories"].toArray();
 
-    for (const QJsonValue value : categories) {
+    for (const QJsonValue &value : std::as_const(categories)) {
         if (!value.isObject()) {
             qWarning() << pluginName << "has invalid loggingCategories entries, skipping";
             return false;
@@ -182,28 +253,39 @@ bool QQmlJSLinter::Plugin::parseMetaData(const QJsonObject &metaData, QString pl
         const auto it = object.find("enabled"_L1);
         const bool ignored = (it != object.end() && !it->toBool());
 
+        const QString prefix = (m_isInternal ? u""_s : u"Plugin."_s).append(m_name).append(u'.');
         const QString categoryId =
-                (m_isInternal ? u""_s : u"Plugin."_s) + m_name + u'.' + object[u"name"].toString();
-        m_categories << QQmlJSLogger::Category {
-                categoryId,
-                categoryId,
-                object["description"_L1].toString(),
-                QtWarningMsg,
-                ignored
-        };
+                prefix + object[u"name"].toString();
+        const auto settingsNameIt = object.constFind(u"settingsName");
+        const QString settingsName = (settingsNameIt == object.constEnd())
+                ? categoryId
+                : prefix + settingsNameIt->toString(categoryId);
+        m_categories << QQmlJS::LoggerCategory{ categoryId, settingsName,
+                                                object["description"_L1].toString(), QtWarningMsg,
+                                                ignored };
+        const auto itLevel = object.find("defaultLevel"_L1);
+        if (itLevel == object.end())
+            continue;
+
+        const QString level = itLevel->toString();
+        if (!QQmlJS::LoggingUtils::applyLevelToCategory(level, m_categories.last())) {
+            qWarning() << "Invalid logging level" << level << "provided for"
+                       << m_categories.last().id().name().toString()
+                       << "(allowed are: disable, info, warning, error) found in plugin metadata.";
+        }
     }
 
     return true;
 }
 
-std::vector<QQmlJSLinter::Plugin> QQmlJSLinter::loadPlugins(QStringList paths)
+std::vector<QQmlJSLinter::Plugin> QQmlJSLinter::loadPlugins(QStringList extraPluginPaths)
 {
-
     std::vector<Plugin> plugins;
 
     QDuplicateTracker<QString> seenPlugins;
 
-    for (const QStaticPlugin &staticPlugin : QPluginLoader::staticPlugins()) {
+    const auto &staticPlugins = QPluginLoader::staticPlugins();
+    for (const QStaticPlugin &staticPlugin : staticPlugins) {
         Plugin plugin(staticPlugin);
         if (!plugin.isValid())
             continue;
@@ -219,8 +301,16 @@ std::vector<QQmlJSLinter::Plugin> QQmlJSLinter::loadPlugins(QStringList paths)
     }
 
 #if QT_CONFIG(library)
+    const QStringList paths = [&extraPluginPaths]() {
+        QStringList result{ extraPluginPaths };
+        const QStringList libraryPaths = QCoreApplication::libraryPaths();
+        for (const auto &path : libraryPaths) {
+            result.append(path + u"/qmllint"_s);
+        }
+        return result;
+    }();
     for (const QString &pluginDir : paths) {
-        QDirIterator it { pluginDir };
+        QDirIterator it{ pluginDir, QDir::Files };
 
         while (it.hasNext()) {
             auto potentialPlugin = it.next();
@@ -244,7 +334,7 @@ std::vector<QQmlJSLinter::Plugin> QQmlJSLinter::loadPlugins(QStringList paths)
         }
     }
 #endif
-
+    Q_UNUSED(extraPluginPaths)
     return plugins;
 }
 
@@ -273,7 +363,7 @@ void QQmlJSLinter::parseComments(QQmlJSLogger *logger,
             const QString category = words.at(i);
             const auto categoryExists = std::any_of(
                     loggerCategories.cbegin(), loggerCategories.cend(),
-                    [&](const QQmlJSLogger::Category &cat) { return cat.id().name() == category; });
+                    [&](const QQmlJS::LoggerCategory &cat) { return cat.id().name() == category; });
 
             if (categoryExists)
                 categories << category;
@@ -283,7 +373,8 @@ void QQmlJSLinter::parseComments(QQmlJSLogger *logger,
         }
 
         if (categories.isEmpty()) {
-            for (const auto &option : logger->categories())
+            const auto &loggerCategories = logger->categories();
+            for (const auto &option : loggerCategories)
                 categories << option.id().name().toString();
         }
 
@@ -380,41 +471,29 @@ static void addJsonWarning(QJsonArray &warnings, const QQmlJS::DiagnosticMessage
         QJsonObject jsonFix {
             { "message"_L1, suggestion->fixDescription() },
             { "replacement"_L1, suggestion->replacement() },
-            { "isHint"_L1, !suggestion->isAutoApplicable() },
+            { "isAutoApplicable"_L1, suggestion->isAutoApplicable() },
+            { "hint"_L1, suggestion->hint() },
         };
         convertLocation(suggestion->location(), &jsonFix);
         const QString filename = suggestion->filename();
         if (!filename.isEmpty())
             jsonFix.insert("fileName"_L1, filename);
         suggestions << jsonFix;
-
-        const QString hint = suggestion->hint();
-        if (!hint.isEmpty()) {
-            // We need to keep compatibility with the JSON format.
-            // Therefore the overly verbose encoding of the hint.
-            QJsonObject jsonHint {
-                { "message"_L1,  hint },
-                { "replacement"_L1, QString() },
-                { "isHint"_L1, true }
-            };
-            convertLocation(QQmlJS::SourceLocation(), &jsonHint);
-            suggestions << jsonHint;
-        }
     }
     jsonMessage[u"suggestions"] = suggestions;
 
     warnings << jsonMessage;
-
 }
 
 void QQmlJSLinter::processMessages(QJsonArray &warnings)
 {
-    for (const auto &error : m_logger->errors())
-        addJsonWarning(warnings, error, error.id, error.fixSuggestion);
-    for (const auto &warning : m_logger->warnings())
-        addJsonWarning(warnings, warning, warning.id, warning.fixSuggestion);
-    for (const auto &info : m_logger->infos())
-        addJsonWarning(warnings, info, info.id, info.fixSuggestion);
+    m_logger->iterateAllMessages([&](const Message &message) {
+        addJsonWarning(warnings, message, message.id, message.fixSuggestion);
+    });
+}
+
+static bool scopeIsBinding(const QQmlJSScope::ConstPtr& scope) {
+    return scope->scopeType() == QQmlJSScope::ScopeType::JSFunctionScope && scope->baseTypeName() == u"binding";
 }
 
 QQmlJSLinter::LintResult QQmlJSLinter::lintFile(const QString &filename,
@@ -422,7 +501,8 @@ QQmlJSLinter::LintResult QQmlJSLinter::lintFile(const QString &filename,
                                                 QJsonArray *json, const QStringList &qmlImportPaths,
                                                 const QStringList &qmldirFiles,
                                                 const QStringList &resourceFiles,
-                                                const QList<QQmlJSLogger::Category> &categories)
+                                                const QList<QQmlJS::LoggerCategory> &categories,
+                                                const QQmlJS::ContextProperties &contextProperties)
 {
     // Make sure that we don't expose an old logger if we return before a new one is created.
     m_logger.reset();
@@ -430,7 +510,7 @@ QQmlJSLinter::LintResult QQmlJSLinter::lintFile(const QString &filename,
     QJsonArray warnings;
     QJsonObject result;
 
-    bool success = true;
+    LintResult success = LintSuccess;
 
     QScopeGuard jsonOutput([&] {
         if (!json)
@@ -438,7 +518,7 @@ QQmlJSLinter::LintResult QQmlJSLinter::lintFile(const QString &filename,
 
         result[u"filename"_s] = QFileInfo(filename).absoluteFilePath();
         result[u"warnings"] = warnings;
-        result[u"success"] = success;
+        result[u"success"] = success == LintSuccess;
 
         json->append(result);
     });
@@ -455,11 +535,11 @@ QQmlJSLinter::LintResult QQmlJSLinter::lintFile(const QString &filename,
                                                             .arg(filename, file.errorString()),
                                                     QtCriticalMsg, QQmlJS::SourceLocation() },
                         qmlImport.name());
-                success = false;
             } else if (!silent) {
                 qWarning() << "Failed to open file" << filename << file.error();
             }
-            return FailedToOpen;
+            success = FailedToOpen;
+            return success;
         }
 
         code = QString::fromUtf8(file.readAll());
@@ -478,150 +558,183 @@ QQmlJSLinter::LintResult QQmlJSLinter::lintFile(const QString &filename,
     const bool isESModule = lowerSuffix == QLatin1String("mjs");
     const bool isJavaScript = isESModule || lowerSuffix == QLatin1String("js");
 
+    m_logger.reset(new QQmlJSLogger);
+    m_logger->setFilePath(m_useAbsolutePath ? info.absoluteFilePath() : filename);
+    m_logger->setCode(code);
+    m_logger->setSilent(silent || json);
+
     lexer.setCode(code, /*lineno = */ 1, /*qmlMode=*/!isJavaScript);
     QQmlJS::Parser parser(&engine);
 
-    success = isJavaScript ? (isESModule ? parser.parseModule() : parser.parseProgram())
-                           : parser.parse();
-
-    if (!success) {
+    if (!(isJavaScript ? (isESModule ? parser.parseModule() : parser.parseProgram())
+                       : parser.parse())) {
+        success = FailedToParse;
         const auto diagnosticMessages = parser.diagnosticMessages();
         for (const QQmlJS::DiagnosticMessage &m : diagnosticMessages) {
-            if (json) {
-                addJsonWarning(warnings, m, qmlSyntax.name());
-            } else if (!silent) {
-                qWarning().noquote() << QString::fromLatin1("%1:%2:%3: %4")
-                                                .arg(filename)
-                                                .arg(m.loc.startLine)
-                                                .arg(m.loc.startColumn)
-                                                .arg(m.message);
-            }
-        }
-        return FailedToParse;
-    }
-
-    if (success && !isJavaScript) {
-        const auto check = [&](QQmlJSResourceFileMapper *mapper) {
-            if (m_importer.importPaths() != qmlImportPaths)
-                m_importer.setImportPaths(qmlImportPaths);
-
-            m_importer.setResourceFileMapper(mapper);
-
-            m_logger.reset(new QQmlJSLogger);
-            m_logger->setFileName(m_useAbsolutePath ? info.absoluteFilePath() : filename);
-            m_logger->setCode(code);
-            m_logger->setSilent(silent || json);
-            QQmlJSScope::Ptr target = QQmlJSScope::create();
-            QQmlJSImportVisitor v { target, &m_importer, m_logger.get(),
-                                    QQmlJSImportVisitor::implicitImportDirectory(
-                                            m_logger->fileName(), m_importer.resourceFileMapper()),
-                                    qmldirFiles };
-
-            if (m_enablePlugins) {
-                for (const Plugin &plugin : m_plugins) {
-                    for (const QQmlJSLogger::Category &category : plugin.categories())
-                        m_logger->registerCategory(category);
-                }
-            }
-
-            for (auto it = categories.cbegin(); it != categories.cend(); ++it) {
-                if (!it->changed)
-                    continue;
-
-                m_logger->setCategoryIgnored(it->id(), it->ignored);
-                m_logger->setCategoryLevel(it->id(), it->level);
-            }
-
-            parseComments(m_logger.get(), engine.comments());
-
-            QQmlJSTypeResolver typeResolver(&m_importer);
-
-            // Type resolving is using document parent mode here so that it produces fewer false
-            // positives on the "parent" property of QQuickItem. It does produce a few false
-            // negatives this way because items can be reparented. Furthermore, even if items are
-            // not reparented, the document parent may indeed not be their visual parent. See
-            // QTBUG-95530. Eventually, we'll need cleverer logic to deal with this.
-            typeResolver.setParentMode(QQmlJSTypeResolver::UseDocumentParent);
-            // We don't need to create tracked types and such as we are just linting the code here
-            // and not actually compiling it. The duplicated scopes would cause issues during
-            // linting.
-            typeResolver.setCloneMode(QQmlJSTypeResolver::DoNotCloneTypes);
-
-            typeResolver.init(&v, parser.rootNode());
-
-            QQmlJSLiteralBindingCheck literalCheck;
-            literalCheck.run(&v, &typeResolver);
-
-            QScopedPointer<QQmlSA::PassManager> passMan;
-
-            if (m_enablePlugins) {
-                passMan.reset(new QQmlSA::PassManager(&v, &typeResolver));
-
-                for (const Plugin &plugin : m_plugins) {
-                    if (!plugin.isValid() || !plugin.isEnabled())
-                        continue;
-
-                    QQmlSA::LintPlugin *instance = plugin.m_instance;
-                    Q_ASSERT(instance);
-                    instance->registerPasses(passMan.get(), v.result());
-                }
-
-                passMan->analyze(v.result());
-            }
-
-            success = !m_logger->hasWarnings() && !m_logger->hasErrors();
-
-            if (m_logger->hasErrors()) {
-                if (json)
-                    processMessages(warnings);
-                return;
-            }
-
-            const QStringList resourcePaths = mapper
-                    ? mapper->resourcePaths(QQmlJSResourceFileMapper::localFileFilter(filename))
-                    : QStringList();
-            const QString resolvedPath =
-                    (resourcePaths.size() == 1) ? u':' + resourcePaths.first() : filename;
-
-            QQmlJSLinterCodegen codegen { &m_importer, resolvedPath, qmldirFiles, m_logger.get() };
-            codegen.setTypeResolver(std::move(typeResolver));
-            if (passMan)
-                codegen.setPassManager(passMan.get());
-            QQmlJSSaveFunction saveFunction = [](const QV4::CompiledData::SaveableUnitPointer &,
-                                                 const QQmlJSAotFunctionMap &,
-                                                 QString *) { return true; };
-
-            QQmlJSCompileError error;
-
-            QLoggingCategory::setFilterRules(u"qt.qml.compiler=false"_s);
-
-            CodegenWarningInterface interface(m_logger.get());
-            qCompileQmlFile(filename, saveFunction, &codegen, &error, true, &interface,
-                            fileContents);
-
-            QList<QQmlJS::DiagnosticMessage> globalWarnings = m_importer.takeGlobalWarnings();
-
-            if (!globalWarnings.isEmpty()) {
-                m_logger->log(QStringLiteral("Type warnings occurred while evaluating file:"),
-                              qmlImport, QQmlJS::SourceLocation());
-                m_logger->processMessages(globalWarnings, qmlImport);
-            }
-
-            success &= !m_logger->hasWarnings() && !m_logger->hasErrors();
-
             if (json)
-                processMessages(warnings);
-        };
+                addJsonWarning(warnings, m, qmlSyntax.name());
+            m_logger->log(m.message, qmlSyntax, m.loc);
+        }
+        return success;
+    }
 
-        if (resourceFiles.isEmpty()) {
-            check(nullptr);
-        } else {
-            QQmlJSResourceFileMapper mapper(resourceFiles);
-            check(&mapper);
+    if (isJavaScript)
+        return success;
+
+    if (m_importer.importPaths() != qmlImportPaths)
+        m_importer.setImportPaths(qmlImportPaths);
+
+    std::optional<QQmlJSResourceFileMapper> mapper;
+    if (!resourceFiles.isEmpty())
+        mapper.emplace(resourceFiles);
+    m_importer.setResourceFileMapper(mapper.has_value() ? &*mapper : nullptr);
+
+    QQmlJSScope::Ptr target = QQmlJSScope::create();
+    QQmlJS::LinterVisitor v{ target, &m_importer, m_logger.get(),
+                             QQmlJSImportVisitor::implicitImportDirectory(
+                                     m_logger->filePath(), m_importer.resourceFileMapper()),
+                             qmldirFiles, &engine };
+
+    if (m_enablePlugins) {
+        for (const Plugin &plugin : m_plugins) {
+            for (const QQmlJS::LoggerCategory &category : plugin.categories())
+                m_logger->registerCategory(category);
         }
     }
 
-    return success ? LintSuccess : HasWarnings;
+    for (auto it = categories.cbegin(); it != categories.cend(); ++it) {
+        if (auto logger = *it; !QQmlJS::LoggerCategoryPrivate::get(&logger)->hasChanged())
+            continue;
+
+        m_logger->setCategoryIgnored(it->id(), it->isIgnored());
+        m_logger->setCategoryLevel(it->id(), it->level());
+    }
+
+    parseComments(m_logger.get(), engine.comments());
+
+    QQmlJSTypeResolver typeResolver(&m_importer);
+
+    // Type resolving is using document parent mode here so that it produces fewer false
+    // positives on the "parent" property of QQuickItem. It does produce a few false
+    // negatives this way because items can be reparented. Furthermore, even if items
+    // are not reparented, the document parent may indeed not be their visual parent.
+    // See QTBUG-95530. Eventually, we'll need cleverer logic to deal with this.
+    typeResolver.setParentMode(QQmlJSTypeResolver::UseDocumentParent);
+    // We don't need to create tracked types and such as we are just linting the code
+    // here and not actually compiling it. The duplicated scopes would cause issues
+    // during linting.
+    typeResolver.setCloneMode(QQmlJSTypeResolver::DoNotCloneTypes);
+
+    typeResolver.init(&v, parser.rootNode());
+
+    const QStringList resourcePaths = mapper
+            ? mapper->resourcePaths(QQmlJSResourceFileMapper::localFileFilter(filename))
+            : QStringList();
+    const QString resolvedPath =
+            (resourcePaths.size() == 1) ? u':' + resourcePaths.first() : filename;
+
+    QQmlJSLinterCodegen codegen{ &m_importer, resolvedPath, qmldirFiles, m_logger.get(),
+                                 contextProperties };
+    codegen.setTypeResolver(std::move(typeResolver));
+
+    using PassManagerPtr =
+            std::unique_ptr<QQmlSA::PassManager,
+                            decltype(&QQmlSA::PassManagerPrivate::deletePassManager)>;
+    PassManagerPtr passMan(
+            QQmlSA::PassManagerPrivate::createPassManager(&v, codegen.typeResolver()),
+            &QQmlSA::PassManagerPrivate::deletePassManager);
+    passMan->registerPropertyPass(std::make_unique<QQmlJSLiteralBindingCheck>(passMan.get()),
+                                  QString(), QString(), QString());
+
+    QQmlSA::PropertyPassBuilder(passMan.get())
+            .withOnCall([](QQmlSA::PropertyPass *self, const QQmlSA::Element &, const QString &,
+                           const QQmlSA::Element &, QQmlSA::SourceLocation location) {
+                self->emitWarning("Do not use 'eval'", qmlEval, location);
+            })
+            .registerOnBuiltin("GlobalObject", "eval");
+
+    QQmlSA::PropertyPassBuilder(passMan.get())
+            .withOnRead([](QQmlSA::PropertyPass *self, const QQmlSA::Element &element,
+                           const QString &propName, const QQmlSA::Element &readScope_,
+                           QQmlSA::SourceLocation location) {
+
+                const auto &elementScope = QQmlJSScope::scope(element);
+                const auto &owner = QQmlJSScope::ownerOfProperty(elementScope, propName).scope;
+                if (!owner || owner->isComposite() || owner->isValueType())
+                    return;
+                const auto &prop = QQmlSA::PropertyPrivate::property(element.property(propName));
+                if (prop.index() != -1 && !prop.isPropertyConstant()
+                        && prop.notify().isEmpty() && prop.bindable().isEmpty()) {
+                    const QQmlJSScope::ConstPtr &readScope = QQmlJSScope::scope(readScope_);
+                    // FIXME: we currently get the closest QML Scope as readScope, instead of
+                    // the innermost scope. We try locate it here via source location
+                    Q_ASSERT(readScope->scopeType() == QQmlJSScope::ScopeType::QMLScope);
+                    for (auto it = readScope->childScopesBegin(); it != readScope->childScopesEnd(); ++it) {
+                        QQmlJS::SourceLocation childLocation = (*it)->sourceLocation();
+                        if ( childLocation.offset <= location.offset() &&
+                            (childLocation.offset + childLocation.length <= location.offset() + location.length())  ) {
+                            if (!scopeIsBinding(*it))
+                                return;
+                        }
+                    }
+                    const QString msg =
+                            "Reading non-constant and non-notifiable property %1. "_L1
+                            "Binding might not update when the property changes."_L1.arg(propName);
+                    self->emitWarning(msg, qmlStalePropertyRead, location);
+                }
+            }).registerOn({}, {}, {});
+
+    if (m_enablePlugins) {
+        for (const Plugin &plugin : m_plugins) {
+            if (!plugin.isValid() || !plugin.isEnabled())
+                continue;
+
+            QQmlSA::LintPlugin *instance = plugin.m_instance;
+            Q_ASSERT(instance);
+            instance->registerPasses(passMan.get(), QQmlJSScope::createQQmlSAElement(v.result()));
+        }
+    }
+    passMan->analyze(QQmlJSScope::createQQmlSAElement(v.result()));
+
+    if (m_logger->hasErrors()) {
+        success = HasErrors;
+        if (json)
+            processMessages(warnings);
+        return success;
+    } else if (m_logger->hasWarnings())
+        success = HasWarnings;
+
+    if (passMan) {
+        // passMan now has a pointer to the moved from type resolver
+        // we fix this in setPassManager
+        codegen.setPassManager(passMan.get());
+    }
+    QQmlJSSaveFunction saveFunction = [](const QV4::CompiledData::SaveableUnitPointer &,
+                                         const QQmlJSAotFunctionMap &, QString *) { return true; };
+
+    QQmlJSCompileError error;
+
+    QLoggingCategory::setFilterRules(u"qt.qml.compiler=false"_s);
+
+    CodegenWarningInterface interface(m_logger.get());
+    qCompileQmlFile(filename, saveFunction, &codegen, &error, true, &interface, fileContents);
+
+    QList<QQmlJS::DiagnosticMessage> globalWarnings = m_importer.takeGlobalWarnings();
+
+    if (!globalWarnings.isEmpty()) {
+        m_logger->log(QStringLiteral("Type warnings occurred while evaluating file:"), qmlImport,
+                      QQmlJS::SourceLocation());
+        m_logger->processMessages(globalWarnings, qmlImport);
+    }
+
+    if (m_logger->hasErrors())
+        success = HasErrors;
+    else if (m_logger->hasWarnings())
+        success = HasWarnings;
+
+    if (json)
+        processMessages(warnings);
+    return success;
 }
 
 QQmlJSLinter::LintResult QQmlJSLinter::lintModule(
@@ -661,14 +774,14 @@ QQmlJSLinter::LintResult QQmlJSLinter::lintModule(
     });
 
     m_logger.reset(new QQmlJSLogger);
-    m_logger->setFileName(module);
+    m_logger->setFilePath(module);
     m_logger->setCode(u""_s);
     m_logger->setSilent(silent || json);
 
     const QQmlJSImporter::ImportedTypes types = m_importer.importModule(module);
 
     QList<QQmlJS::DiagnosticMessage> importWarnings =
-            m_importer.takeGlobalWarnings() + m_importer.takeWarnings();
+            m_importer.takeGlobalWarnings() + types.warnings();
 
     if (!importWarnings.isEmpty()) {
         m_logger->log(QStringLiteral("Warnings occurred while importing module:"), qmlImport,
@@ -703,7 +816,8 @@ QQmlJSLinter::LintResult QQmlJSLinter::lintModule(
             if (!partiallyResolvedTypes.contains(name))
                 partiallyResolvedTypes[name] = {};
         }
-        for (const auto &property : scope->ownProperties()) {
+        const auto &ownProperties = scope->ownProperties();
+        for (const auto &property : ownProperties) {
             if (property.typeName().isEmpty()) {
                 // If the type name is empty, then it's an intentional vaguery i.e. for some
                 // builtins
@@ -724,7 +838,8 @@ QQmlJSLinter::LintResult QQmlJSLinter::lintModule(
                           qmlUnresolvedType, scope->sourceLocation());
         }
 
-        for (const auto &method : scope->ownMethods()) {
+        const auto &ownMethods = scope->ownMethods();
+        for (const auto &method : ownMethods) {
             if (method.returnTypeName().isEmpty())
                 continue;
             if (method.returnType().isNull()) {
@@ -801,7 +916,7 @@ QQmlJSLinter::FixResult QQmlJSLinter::applyFixes(QString *fixedCode, bool silent
 
     QList<QQmlJSFixSuggestion> fixesToApply;
 
-    QFileInfo info(m_logger->fileName());
+    QFileInfo info(m_logger->filePath());
     const QString currentFileAbsolutePath = info.absoluteFilePath();
 
     const QString lowerSuffix = info.suffix().toLower();
@@ -811,21 +926,20 @@ QQmlJSLinter::FixResult QQmlJSLinter::applyFixes(QString *fixedCode, bool silent
     if (isESModule || isJavaScript)
         return NothingToFix;
 
-    for (const auto &messages : { m_logger->infos(), m_logger->warnings(), m_logger->errors() })
-        for (const Message &msg : messages) {
-            if (!msg.fixSuggestion.has_value() || !msg.fixSuggestion->isAutoApplicable())
-                continue;
+    m_logger->iterateAllMessages([&](const Message &msg) {
+        if (!msg.fixSuggestion.has_value() || !msg.fixSuggestion->isAutoApplicable())
+            return;
 
-            // Ignore fix suggestions for other files
-            const QString filename = msg.fixSuggestion->filename();
-            if (!filename.isEmpty()
-                    && QFileInfo(filename).absoluteFilePath() != currentFileAbsolutePath) {
-                continue;
-            }
-
-            fixesToApply << msg.fixSuggestion.value();
+        // Ignore fix suggestions for other files
+        const QString filename = msg.fixSuggestion->filename();
+        if (!filename.isEmpty()
+                && QFileInfo(filename).absoluteFilePath() != currentFileAbsolutePath) {
+            return;
         }
 
+        fixesToApply << msg.fixSuggestion.value();
+    });
+
     if (fixesToApply.isEmpty())
         return NothingToFix;
 
@@ -834,6 +948,9 @@ QQmlJSLinter::FixResult QQmlJSLinter::applyFixes(QString *fixedCode, bool silent
                   return a.location().offset < b.location().offset;
               });
 
+    const auto dupes = std::unique(fixesToApply.begin(), fixesToApply.end());
+    fixesToApply.erase(dupes, fixesToApply.end());
+
     for (auto it = fixesToApply.begin(); it + 1 != fixesToApply.end(); it++) {
         const QQmlJS::SourceLocation srcLocA = it->location();
         const QQmlJS::SourceLocation srcLocB = (it + 1)->location();
@@ -847,7 +964,7 @@ QQmlJSLinter::FixResult QQmlJSLinter::applyFixes(QString *fixedCode, bool silent
 
     int offsetChange = 0;
 
-    for (const auto &fix : fixesToApply) {
+    for (const auto &fix : std::as_const(fixesToApply)) {
         const QQmlJS::SourceLocation fixLocation = fix.location();
         qsizetype cutLocation = fixLocation.offset + offsetChange;
         const QString before = code.left(cutLocation);
@@ -878,7 +995,7 @@ QQmlJSLinter::FixResult QQmlJSLinter::applyFixes(QString *fixedCode, bool silent
 
         for (const QQmlJS::DiagnosticMessage &m : diagnosticMessages) {
             qWarning().noquote() << QString::fromLatin1("%1:%2:%3: %4")
-                                            .arg(m_logger->fileName())
+                                            .arg(m_logger->filePath())
                                             .arg(m.loc.startLine)
                                             .arg(m.loc.startColumn)
                                             .arg(m.message);
