@@ -15,7 +15,6 @@
 #include <QtQmlCompiler/private/qqmlsasourcelocation_p.h>
 
 #include <memory>
-#include <new>
 
 QT_BEGIN_NAMESPACE
 
@@ -71,7 +70,7 @@ static_assert(QQmlJSScope::sizeofQQmlSAElement() == sizeof(Element));
     \value TranslationById  The binding is a \l{Text ID based translations}{translation} by id
     \value Script           The binding is a regular script
     \value Object           The binging is an \l{QML Object Types}{Object}
-    \value Interceptor      The binding is an interceptor that can intercept writes to properties such as \l{Behavior QML Type}{Behavior}
+    \value Interceptor      The binding is an interceptor that can intercept writes to properties such as \l[Quick]{Behavior}
     \value ValueSource      The binging is a \l{Defining QML Types from C++#Property Value Sources}{property value source}
     \value AttachedProperty The binding is an \l{QML Object Attributes#Attached Properties and Attached Signal Handlers}{attached object}
     \value GroupProperty    The binding is a \l{QML Object Attributes#Grouped Properties}{grouped property}
@@ -253,8 +252,11 @@ Binding &Binding::operator=(const Binding &other)
     if (*this == other)
         return *this;
 
-    d_func()->m_binding = other.d_func()->m_binding;
-    d_func()->q_ptr = this;
+    Q_D(Binding);
+    d->m_binding = other.d_func()->m_binding;
+    d->m_bindingScope = other.d_func()->m_bindingScope;
+    d->q_ptr = this;
+    d->m_isAttached = other.d_func()->m_isAttached;
     return *this;
 }
 
@@ -266,8 +268,11 @@ Binding &Binding::operator=(Binding &&other) noexcept
     if (*this == other)
         return *this;
 
-    d_func()->m_binding = std::move(other.d_func()->m_binding);
-    d_func()->q_ptr = this;
+    Q_D(Binding);
+    d->m_binding = std::move(other.d_func()->m_binding);
+    d->m_bindingScope = std::move(other.d_func()->m_bindingScope);
+    d->q_ptr = this;
+    d->m_isAttached = other.d_func()->m_isAttached;
     return *this;
 }
 
@@ -278,11 +283,14 @@ Binding::~Binding() = default;
 
 bool Binding::operatorEqualsImpl(const Binding &lhs, const Binding &rhs)
 {
-    return lhs.d_func()->m_binding == rhs.d_func()->m_binding;
+    return lhs.d_func()->m_binding == rhs.d_func()->m_binding
+            && lhs.d_func()->m_bindingScope == rhs.d_func()->m_bindingScope
+            && lhs.d_func()->m_isAttached == rhs.d_func()->m_isAttached;
 }
 
 BindingPrivate::BindingPrivate(Binding *interface, const BindingPrivate &other)
-    : m_binding{ other.m_binding }, q_ptr{ interface }
+    : m_binding{ other.m_binding }, m_bindingScope{ other.m_bindingScope }, q_ptr{ interface },
+      m_isAttached{ other.m_isAttached }
 {
 }
 
@@ -312,6 +320,14 @@ Element Binding::groupType() const
     return QQmlJSScope::createQQmlSAElement(BindingPrivate::binding(*this).groupType());
 }
 
+/*!
+    Returns the Element scope in which the binding is defined.
+ */
+Element Binding::bindingScope() const
+{
+    return BindingPrivate::get(this)->m_bindingScope;
+}
+
 /*!
     Returns the type of this binding.
  */
@@ -337,14 +353,35 @@ QString Binding::propertyName() const
     return BindingPrivate::binding(*this).propertyName();
 }
 
+/*!
+    Returns \c true if this type is attached to another one, \c false otherwise.
+ */
+bool Binding::isAttached() const
+{
+    return BindingPrivate::get(this)->m_isAttached;
+}
+
 /*!
     Returns the attached type if the content type of this binding is
     AttachedProperty, otherwise returns an invalid Element.
  */
+Element Binding::attachedType() const
+{
+    return QQmlJSScope::createQQmlSAElement(BindingPrivate::binding(*this).attachedType());
+}
+
+#if QT_DEPRECATED_SINCE(6, 9)
+/*!
+    Returns the attached type if the content type of this binding is
+    AttachedProperty, otherwise returns an invalid Element.
+
+    \deprecated [6.9] Use the better named attachedType() method.
+ */
 Element Binding::attachingType() const
 {
-    return QQmlJSScope::createQQmlSAElement(BindingPrivate::binding(*this).attachingType());
+    return attachedType();
 }
+#endif
 
 /*!
     Returns the location in the QML code where this binding is defined.
@@ -402,6 +439,19 @@ bool Binding::hasUndefinedScriptValue() const
             && jsBinding.scriptValueType() == ScriptValue_Undefined;
 }
 
+/*!
+    Returns whether this binding has script value type function like when it
+    is assigned a (lambda) method, an arrow function or a statement block. If
+    the content type of this binding is not \l{QQmlSA::BindingType::Script},
+    returns \c false.
+ */
+bool Binding::hasFunctionScriptValue() const
+{
+    const auto &jsBinding = BindingPrivate::binding(*this);
+    return jsBinding.bindingType() == BindingType::Script
+            && jsBinding.scriptValueType() == ScriptValue_Function;
+}
+
 /*!
     Returns \c true if \a bindingType is a literal type, and \c false
     otherwise.
@@ -891,6 +941,14 @@ bool Element::inherits(const Element &element) const
     return QQmlJSScope::scope(*this)->inherits(QQmlJSScope::scope(element));
 }
 
+/*!
+    Returns whether this Element is the root component of its QML file.
+ */
+bool Element::isFileRootComponent() const
+{
+    return QQmlJSScope::scope(*this)->isFileRootComponent();
+}
+
 /*!
     Returns \c true if this element is null, \c false otherwise.
  */
@@ -1001,6 +1059,15 @@ QQmlSA::SourceLocation Element::sourceLocation() const
             QQmlJSScope::scope(*this)->sourceLocation());
 }
 
+/*!
+    Returns the location in the QML code where this Element is assigned its id, if it has one.
+ */
+QQmlSA::SourceLocation Element::idSourceLocation() const
+{
+    return QQmlSA::SourceLocationPrivate::createQQmlSASourceLocation(
+            QQmlJSScope::scope(*this)->idSourceLocation());
+}
+
 /*!
     Returns the file path of the QML code that defines this Element.
  */
@@ -1073,7 +1140,7 @@ BindingsPrivate::createBindings(const QMultiHash<QString, QQmlJSMetaPropertyBind
 }
 
 QQmlSA::Binding::Bindings BindingsPrivate::createBindings(
-        QPair<QMultiHash<QString, QQmlJSMetaPropertyBinding>::const_iterator,
+        std::pair<QMultiHash<QString, QQmlJSMetaPropertyBinding>::const_iterator,
               QMultiHash<QString, QQmlJSMetaPropertyBinding>::const_iterator> iterators)
 {
     QMultiHash<QString, QQmlSA::Binding> saBindings;
@@ -1159,9 +1226,9 @@ public:
     }
 
 private:
-    PassManager *m_manager;
+    PassManager *m_manager = nullptr;
 
-    GenericPass *q_ptr;
+    GenericPass *q_ptr = nullptr;
 };
 
 /*!
@@ -1444,10 +1511,29 @@ bool PassManagerPrivate::registerPropertyPass(std::shared_ptr<PropertyPass> pass
 
         name = lookupName(element, Register);
     }
-    const QQmlSA::PropertyPassInfo passInfo{ propertyName.isEmpty()
-                                                     ? QStringList{}
-                                                     : QStringList{ propertyName.toString() },
-                                             std::move(pass), allowInheritance };
+    const QQmlSA::PropertyPassInvocation passInfo{ propertyName.toString(), std::move(pass),
+                                                   allowInheritance };
+    m_propertyPasses.insert({ name, passInfo });
+
+    return true;
+}
+
+bool PassManagerPrivate::registerPropertyPassOnBuiltinType(std::shared_ptr<PropertyPass> pass,
+                                                           QAnyStringView builtinTypeName,
+                                                           QAnyStringView propertyName,
+                                                           bool allowInheritance)
+{
+    auto typeImporter = m_visitor->importer();
+    const auto scope = typeImporter->builtinInternalNames().type(builtinTypeName.toString()).scope;
+    const auto element = QQmlJSScope::createQQmlSAElement(scope);
+
+    if (element.isNull())
+        return false;
+
+    const QString name = lookupName(element, Register);
+
+    const QQmlSA::PropertyPassInvocation passInfo{ propertyName.toString(), std::move(pass),
+                                                   allowInheritance };
     m_propertyPasses.insert({ name, passInfo });
 
     return true;
@@ -1457,25 +1543,37 @@ void PassManagerPrivate::addBindingSourceLocations(const Element &element, const
                                                    const QString prefix, bool isAttached)
 {
     const Element &currentScope = scope.isNull() ? element : scope;
-    const auto ownBindings = currentScope.ownPropertyBindings();
-    for (const auto &binding : ownBindings) {
+    auto ownBindings = currentScope.ownPropertyBindings();
+    for (auto &binding : ownBindings) {
         switch (binding.bindingType()) {
         case QQmlSA::BindingType::GroupProperty:
             addBindingSourceLocations(element, Element{ binding.groupType() },
                                       prefix + binding.propertyName() + u'.');
-            break;
+            continue; // don't insert into m_bindingsByLocation
         case QQmlSA::BindingType::AttachedProperty:
-            addBindingSourceLocations(element, Element{ binding.attachingType() },
+            addBindingSourceLocations(element, Element{ binding.attachedType() },
                                       prefix + binding.propertyName() + u'.', true);
+            continue; // don't insert into m_bindingsByLocation
+        case QQmlSA::BindingType::Translation: {
+            analyzeCall(QQmlJSScope::createQQmlSAElement(m_typeResolver->jsGlobalObject()),
+                        u"qsTr"_s, element, binding.sourceLocation());
             break;
-        default:
-            m_bindingsByLocation.insert({ binding.sourceLocation().offset(),
-                                          BindingInfo{ prefix + binding.propertyName(), binding,
-                                                       currentScope, isAttached } });
-
-            if (binding.bindingType() != QQmlSA::BindingType::Script)
-                analyzeBinding(element, QQmlSA::Element(), binding.sourceLocation());
         }
+        case QQmlSA::BindingType::TranslationById: {
+            analyzeCall(QQmlJSScope::createQQmlSAElement(m_typeResolver->jsGlobalObject()),
+                        u"qsTrId"_s, element, binding.sourceLocation());
+            break;
+        }
+        default:
+            break;
+        }
+
+        BindingPrivate::get(&binding)->setBindingScope(currentScope);
+        BindingPrivate::get(&binding)->setPropertyName(prefix + binding.propertyName());
+        BindingPrivate::get(&binding)->setIsAttached(isAttached);
+        m_bindingsByLocation.insert({ binding.sourceLocation().offset(), binding });
+        if (binding.bindingType() != QQmlSA::BindingType::Script)
+            analyzeBinding(element, QQmlSA::Element(), binding.sourceLocation());
     }
 }
 
@@ -1488,12 +1586,12 @@ void PassManager::analyze(const Element &root)
     d->analyze(root);
 }
 
-static QQmlJS::ConstPtrWrapperIterator childScopesBegin(const Element &element)
+static QQmlJS::ChildScopesIterator childScopesBegin(const Element &element)
 {
     return QQmlJSScope::scope(element)->childScopesBegin();
 }
 
-static QQmlJS::ConstPtrWrapperIterator childScopesEnd(const Element &element)
+static QQmlJS::ChildScopesIterator childScopesEnd(const Element &element)
 {
     return QQmlJSScope::scope(element)->childScopesEnd();
 }
@@ -1516,40 +1614,50 @@ void PassManagerPrivate::analyze(const Element &root)
     }
 }
 
-void PassManagerPrivate::analyzeWrite(const Element &element, QString propertyName,
+void PassManagerPrivate::analyzeWrite(const Element &element, const QString &propertyName,
                                       const Element &value, const Element &writeScope,
-                                      QQmlSA::SourceLocation location)
+                                      const QQmlSA::SourceLocation &location)
 {
-    for (PropertyPass *pass : findPropertyUsePasses(element, propertyName))
+    const auto &passes = findPropertyUsePasses(element, propertyName);
+    for (PropertyPass *pass : passes)
         pass->onWrite(element, propertyName, value, writeScope, location);
 }
 
-void PassManagerPrivate::analyzeRead(const Element &element, QString propertyName,
-                                     const Element &readScope, QQmlSA::SourceLocation location)
+void PassManagerPrivate::analyzeRead(const Element &element, const QString &propertyName,
+                                     const Element &readScope, const QQmlSA::SourceLocation &location)
 {
-    for (PropertyPass *pass : findPropertyUsePasses(element, propertyName))
+    const auto &passes = findPropertyUsePasses(element, propertyName);
+    for (PropertyPass *pass : passes)
         pass->onRead(element, propertyName, readScope, location);
 }
 
-void PassManagerPrivate::analyzeBinding(const Element &element, const QQmlSA::Element &value,
-                                        QQmlSA::SourceLocation location)
+void PassManagerPrivate::analyzeCall(const Element &element, const QString &propertyName,
+                                     const Element &readScope, const QQmlSA::SourceLocation &location)
 {
-    const auto info = m_bindingsByLocation.find(location.offset());
+    const auto &passes = findPropertyUsePasses(element, propertyName);
+    for (PropertyPass *pass : passes)
+        pass->onCall(element, propertyName, readScope, location);
+}
+
+void PassManagerPrivate::analyzeBinding(const Element &element, const QQmlSA::Element &value,
+                                        const QQmlSA::SourceLocation &location)
+{
+    const auto it = m_bindingsByLocation.find(location.offset());
 
     // If there's no matching binding that means we're in a nested Ret somewhere inside an
     // expression
-    if (info == m_bindingsByLocation.end())
+    if (it == m_bindingsByLocation.cend())
         return;
 
-    const QQmlSA::Element &bindingScope = info->second.bindingScope;
-    const QQmlSA::Binding &binding = info->second.binding;
-    const QString &propertyName = info->second.fullPropertyName;
+    const auto &[offset, binding] = *it;
+    const QQmlSA::Element &bindingScope = binding.bindingScope();
+    const QString &propertyName = binding.propertyName();
 
     const auto elementPasses = findPropertyUsePasses(element, propertyName);
     for (PropertyPass *pass : elementPasses)
         pass->onBinding(element, propertyName, binding, bindingScope, value);
 
-    if (!info->second.isAttached || bindingScope.baseType().isNull())
+    if (!binding.isAttached() || bindingScope.baseType().isNull())
         return;
 
     const auto bindingScopePasses = findPropertyUsePasses(bindingScope.baseType(), propertyName);
@@ -1601,15 +1709,10 @@ QQmlJSTypeResolver *QQmlSA::PassManagerPrivate::resolver(const QQmlSA::PassManag
 QSet<PropertyPass *> PassManagerPrivate::findPropertyUsePasses(const QQmlSA::Element &element,
                                                                const QString &propertyName)
 {
-    QStringList typeNames { lookupName(element) };
+    QStringList typeNames;
 
-    QQmlJSUtils::searchBaseAndExtensionTypes(
-            QQmlJSScope::scope(element),
-            [&](const QQmlJSScope::ConstPtr &scope, QQmlJSScope::ExtensionKind mode) {
-                Q_UNUSED(mode);
-                typeNames.append(lookupName(QQmlJSScope::createQQmlSAElement(scope)));
-                return false;
-            });
+    for (auto it = QQmlJSScope::scope(element); it; it = it->baseType())
+        typeNames.append(lookupName(QQmlJSScope::createQQmlSAElement(it)));
 
     QSet<PropertyPass *> passes;
 
@@ -1622,10 +1725,8 @@ QSet<PropertyPass *> PassManagerPrivate::findPropertyUsePasses(const QQmlSA::Ele
             for (auto it = pass.first; it != pass.second; it++) {
                 if (typeName != typeNames.constFirst() && !it->second.allowInheritance)
                     continue;
-                if (it->second.properties.isEmpty()
-                    || it->second.properties.contains(propertyName)) {
+                if (it->second.property.isEmpty() || it->second.property == propertyName)
                     passes.insert(it->second.pass.get());
-                }
             }
         }
     }
@@ -1778,6 +1879,9 @@ void PropertyPass::onBinding(const Element &element, const QString &propertyName
 
     The property \a propertyName of \a element is read by an instruction within
     \a readScope defined at \a location.
+
+    This is also executed if the property \a propertyName is called as a function as that requires
+    the property to be read first.
  */
 void PropertyPass::onRead(const Element &element, const QString &propertyName,
                           const Element &readScope, QQmlSA::SourceLocation location)
@@ -1788,6 +1892,25 @@ void PropertyPass::onRead(const Element &element, const QString &propertyName,
     Q_UNUSED(location);
 }
 
+/*!
+    Executes whenever a property or method is called.
+
+    The property or method \a propertyName of \a element is called as a function by an instruction
+    within \a readScope defined at \a location.
+
+    \note Currently only direct calls of methods or properties are supported, indirect calls, for
+    example by storing a method into a JavaScript variable and then calling the variable, are not
+    recognized.
+ */
+void PropertyPass::onCall(const Element &element, const QString &propertyName,
+                          const Element &readScope, QQmlSA::SourceLocation location)
+{
+    Q_UNUSED(element);
+    Q_UNUSED(propertyName);
+    Q_UNUSED(readScope);
+    Q_UNUSED(location);
+}
+
 /*!
     Executes whenever a property is written to.
 
@@ -1858,28 +1981,10 @@ void DebugPropertyPass::onWrite(const QQmlSA::Element &element, const QString &p
                 qmlPlugin, location);
 }
 
-/*!
-    Returns the list of element passes.
- */
-std::vector<std::shared_ptr<ElementPass>> PassManager::elementPasses() const
-{
-    Q_D(const PassManager);
-    return d->m_elementPasses;
-}
-
-/*!
-    Returns the list of property passes.
- */
-std::multimap<QString, PropertyPassInfo> PassManager::propertyPasses() const
-{
-    Q_D(const PassManager);
-    return d->m_propertyPasses;
-}
-
 /*!
     Returns bindings by their source location.
  */
-std::unordered_map<quint32, BindingInfo> PassManager::bindingsByLocation() const
+std::unordered_map<quint32, Binding> PassManager::bindingsByLocation() const
 {
     Q_D(const PassManager);
     return d->m_bindingsByLocation;
@@ -2112,6 +2217,27 @@ bool FixSuggestion::operatorEqualsImpl(const FixSuggestion &lhs, const FixSugges
     return lhs.d_func()->m_fixSuggestion == rhs.d_func()->m_fixSuggestion;
 }
 
+void emitWarningWithOptionalFix(GenericPass &pass, QAnyStringView diagnostic,
+                                const LoggerWarningId &id,
+                                const QQmlSA::SourceLocation &srcLocation,
+                                const std::optional<QQmlJSFixSuggestion> &fix)
+{
+    if (!fix.has_value()) {
+        pass.emitWarning(diagnostic, id, srcLocation);
+        return;
+    }
+
+    const QQmlSA::SourceLocation location =
+            QQmlSA::SourceLocationPrivate::createQQmlSASourceLocation(fix->location());
+    const QQmlSA::FixSuggestion saFix{ fix->fixDescription(), location, fix->replacement() };
+    pass.emitWarning(diagnostic, id, srcLocation, saFix);
+}
+
+bool isRegularBindingType(BindingType type)
+{
+    return type >= BindingType::BoolLiteral && type <= BindingType::Object;
+}
+
 } // namespace QQmlSA
 
 QT_END_NAMESPACE
