@@ -50,14 +50,14 @@
 
 QT_BEGIN_NAMESPACE
 
+QT_DECLARE_EXPORTED_QT_LOGGING_CATEGORY(writeOutLog, QMLDOM_EXPORT);
+
 namespace QQmlJS {
 // we didn't have enough 'O's to properly name everything...
 namespace Dom {
 
 class Path;
 
-Q_DECLARE_LOGGING_CATEGORY(writeOutLog);
-
 constexpr bool domTypeIsObjWrap(DomType k);
 constexpr bool domTypeIsValueWrap(DomType k);
 constexpr bool domTypeIsDomElement(DomType);
@@ -674,9 +674,9 @@ public:
     Path referredObjectPath;
 };
 
-template<typename Info>
-class AttachedInfoT;
-class FileLocations;
+namespace FileLocations {
+struct Info;
+}
 
 /*!
     \internal
@@ -695,8 +695,8 @@ struct ScriptElement : public DomElement
     using PointerType = std::shared_ptr<T>;
 
     using DomElement::DomElement;
-    virtual void createFileLocations(
-            const std::shared_ptr<AttachedInfoT<FileLocations>> &fileLocationOfOwner) = 0;
+    virtual void
+    createFileLocations(const std::shared_ptr<FileLocations::Node> &fileLocationOfOwner) = 0;
 
     QQmlJSScope::ConstPtr semanticScope();
     void setSemanticScope(const QQmlJSScope::ConstPtr &scope);
@@ -793,64 +793,30 @@ private:
 
 // TODO: create more "groups" to simplify this variant? Maybe into Internal, ScriptExpression, ???
 using ElementT =
-        std::variant<
-                ConstantData,
-                Empty,
-                List,
-                ListP,
-                Map,
-                Reference,
-                ScriptElementDomWrapper,
-                SimpleObjectWrap,
-                const AstComments *,
-                const AttachedInfo *,
-                const DomEnvironment *,
-                const DomUniverse *,
-                const EnumDecl *,
-                const ExternalItemInfoBase *,
-                const ExternalItemPairBase *,
-                const GlobalComponent *,
-                const GlobalScope *,
-                const JsFile *,
-                const JsResource *,
-                const LoadInfo *,
-                const MockObject *,
-                const MockOwner *,
-                const ModuleIndex *,
-                const ModuleScope *,
-                const QmlComponent *,
-                const QmlDirectory *,
-                const QmlFile *,
-                const QmlObject *,
-                const QmldirFile *,
-                const QmltypesComponent *,
-                const QmltypesFile *,
-                const ScriptExpression *
-        >;
+        std::variant<ConstantData, Empty, List, ListP, Map, Reference, ScriptElementDomWrapper,
+                     SimpleObjectWrap, const AstComments *, const FileLocations::Node *,
+                     const DomEnvironment *, const DomUniverse *, const EnumDecl *,
+                     const ExternalItemInfoBase *, const ExternalItemPairBase *,
+                     const GlobalComponent *, const GlobalScope *, const JsFile *,
+                     const JsResource *, const LoadInfo *, const MockObject *, const MockOwner *,
+                     const ModuleIndex *, const ModuleScope *, const QmlComponent *,
+                     const QmlDirectory *, const QmlFile *, const QmlObject *, const QmldirFile *,
+                     const QmltypesComponent *, const QmltypesFile *, const ScriptExpression *>;
 
 using TopT = std::variant<
         std::monostate,
         std::shared_ptr<DomEnvironment>,
         std::shared_ptr<DomUniverse>>;
 
-using OwnerT = std::variant<
-        std::monostate,
-        std::shared_ptr<ModuleIndex>,
-        std::shared_ptr<MockOwner>,
-        std::shared_ptr<ExternalItemInfoBase>,
-        std::shared_ptr<ExternalItemPairBase>,
-        std::shared_ptr<QmlDirectory>,
-        std::shared_ptr<QmldirFile>,
-        std::shared_ptr<JsFile>,
-        std::shared_ptr<QmlFile>,
-        std::shared_ptr<QmltypesFile>,
-        std::shared_ptr<GlobalScope>,
-        std::shared_ptr<ScriptExpression>,
-        std::shared_ptr<AstComments>,
-        std::shared_ptr<LoadInfo>,
-        std::shared_ptr<AttachedInfo>,
-        std::shared_ptr<DomEnvironment>,
-        std::shared_ptr<DomUniverse>>;
+using OwnerT =
+        std::variant<std::monostate, std::shared_ptr<ModuleIndex>, std::shared_ptr<MockOwner>,
+                     std::shared_ptr<ExternalItemInfoBase>, std::shared_ptr<ExternalItemPairBase>,
+                     std::shared_ptr<QmlDirectory>, std::shared_ptr<QmldirFile>,
+                     std::shared_ptr<JsFile>, std::shared_ptr<QmlFile>,
+                     std::shared_ptr<QmltypesFile>, std::shared_ptr<GlobalScope>,
+                     std::shared_ptr<ScriptExpression>, std::shared_ptr<AstComments>,
+                     std::shared_ptr<LoadInfo>, std::shared_ptr<FileLocations::Node>,
+                     std::shared_ptr<DomEnvironment>, std::shared_ptr<DomUniverse>>;
 
 inline bool emptyChildrenVisitor(Path, const DomItem &, bool)
 {
@@ -880,6 +846,8 @@ public:
     std::weak_ptr<DomEnvironment> environment() const { return m_environment; }
     QString canonicalPath() const { return m_canonicalPath; }
     QString logicalPath() const { return m_logicalPath; }
+    void setCanonicalPath(const QString &canonicalPath) { m_canonicalPath = canonicalPath; }
+    void setLogicalPath(const QString &logicalPath) { m_logicalPath = logicalPath; }
     std::optional<InMemoryContents> content() const { return m_content; }
 
 private:
@@ -1071,8 +1039,6 @@ public:
     quintptr id() const;
     Path pathFromOwner() const;
     QString canonicalFilePath() const;
-    DomItem fileLocationsTree() const;
-    DomItem fileLocations() const;
     MutableDomItem makeCopy(CopyOption option = CopyOption::EnvConnected) const;
     bool commitToBase(const std::shared_ptr<DomEnvironment> &validPtr = nullptr) const;
     DomItem refreshed() const { return top().path(canonicalPath()); }
@@ -1190,7 +1156,7 @@ public:
     DomItem subOwnerItem(const PathEls::PathComponent &c, Owner o) const
     {
         if constexpr (domTypeIsUnattachedOwningItem(Owner::element_type::kindValue))
-            return DomItem(m_top, o, canonicalPath().appendComponent(c), o.get());
+            return DomItem(m_top, o, canonicalPath().withComponent(c), o.get());
         else
             return DomItem(m_top, o, Path(), o.get());
     }
@@ -1282,7 +1248,7 @@ public:
 
 private:
     enum class WriteOutCheckResult { Success, Failed };
-    WriteOutCheckResult performWriteOutChecks(const DomItem &, const DomItem &, OutWriter &, WriteOutChecks) const;
+    WriteOutCheckResult performWriteOutChecks(const DomItem &, OutWriter &, WriteOutChecks) const;
     const DomBase *base() const;
 
     template<typename Env, typename Owner>
@@ -1327,7 +1293,7 @@ private:
     friend class MutableDomItem;
     friend class ScriptExpression;
     friend class AstComments;
-    friend class AttachedInfo;
+    friend class FileLocations::Node;
     friend class TestDomItem;
     friend QMLDOM_EXPORT bool operator==(const DomItem &, const DomItem &);
     DomType m_kind = DomType::Empty;
@@ -1357,7 +1323,7 @@ static DomItem keyMultiMapHelper(const DomItem &self, const QString &key,
         QList<const T *> values;
         while (it != end && it.key() == key)
             values.append(&(*it++));
-        ListP ll(self.pathFromOwner().appendComponent(PathEls::Key(key)), values, QString(),
+        ListP ll(self.pathFromOwner().withComponent(PathEls::Key(key)), values, QString(),
                  ListOptions::Reverse);
         return self.copy(ll);
     }
@@ -1562,10 +1528,10 @@ std::shared_ptr<T> DomItem::ownerAs() const
 {
     if constexpr (domTypeIsOwningItem(T::kindValue)) {
         if (!std::holds_alternative<std::monostate>(m_owner)) {
-            if constexpr (T::kindValue == DomType::AttachedInfo) {
-                if (std::holds_alternative<std::shared_ptr<AttachedInfo>>(m_owner))
+            if constexpr (T::kindValue == DomType::FileLocationsNode) {
+                if (std::holds_alternative<std::shared_ptr<FileLocations::Node>>(m_owner))
                     return std::static_pointer_cast<T>(
-                            std::get<std::shared_ptr<AttachedInfo>>(m_owner));
+                            std::get<std::shared_ptr<FileLocations::Node>>(m_owner));
             } else if constexpr (T::kindValue == DomType::ExternalItemInfo) {
                 if (std::holds_alternative<std::shared_ptr<ExternalItemInfoBase>>(m_owner))
                     return std::static_pointer_cast<T>(
@@ -1635,7 +1601,7 @@ public:
     QString internalKindStr() { return domTypeToString(internalKind()); }
     DomKind domKind() { return kind2domKind(internalKind()); }
 
-    Path canonicalPath() const { return m_owner.canonicalPath().path(m_pathFromOwner); }
+    Path canonicalPath() const { return m_owner.canonicalPath().withPath(m_pathFromOwner); }
     MutableDomItem containingObject()
     {
         if (m_pathFromOwner)
@@ -1726,7 +1692,6 @@ public:
         return item().writeOut(path, nBackups, opt, fw);
     }
 
-    MutableDomItem fileLocations() { return MutableDomItem(item().fileLocations()); }
     MutableDomItem makeCopy(CopyOption option = CopyOption::EnvConnected)
     {
         return item().makeCopy(option);
@@ -1862,7 +1827,7 @@ Path insertUpdatableElementInMultiMap(const Path &mapPathFromOwner, QMultiMap<K,
                 qWarning() << " requested overwrite of " << key
                            << " that contains aleready multiple entries in" << mapPathFromOwner;
             }
-            Path newPath = mapPathFromOwner.key(key).index(0);
+            Path newPath = mapPathFromOwner.withKey(key).withIndex(0);
             v.updatePathFromOwner(newPath);
             if (valuePtr)
                 *valuePtr = &v;
@@ -1877,7 +1842,7 @@ Path insertUpdatableElementInMultiMap(const Path &mapPathFromOwner, QMultiMap<K,
        ++nVal;
         ++it2;
     }
-    Path newPath = mapPathFromOwner.key(key).index(nVal-1);
+    Path newPath = mapPathFromOwner.withKey(key).withIndex(nVal-1);
     T &v = *it;
     v.updatePathFromOwner(newPath);
     if (valuePtr)
@@ -1891,7 +1856,7 @@ Path appendUpdatableElementInQList(const Path &listPathFromOwner, QList<T> &list
 {
     int idx = list.size();
     list.append(value);
-    Path newPath = listPathFromOwner.index(idx);
+    Path newPath = listPathFromOwner.withIndex(idx);
     T &targetV = list[idx];
     targetV.updatePathFromOwner(newPath);
     if (vPtr)
@@ -1909,9 +1874,9 @@ void updatePathFromOwnerMultiMap(QMultiMap<K, T> &mmap, const Path &newPath)
     QList<T*> els;
     while (it != end) {
         if (i > 0 && name != it.key()) {
-            Path pName = newPath.key(QString(name));
+            Path pName = newPath.withKey(QString(name));
             for (T *el : els)
-                el->updatePathFromOwner(pName.index(--i));
+                el->updatePathFromOwner(pName.withIndex(--i));
             els.clear();
             els.append(&(*it));
             name = it.key();
@@ -1923,9 +1888,9 @@ void updatePathFromOwnerMultiMap(QMultiMap<K, T> &mmap, const Path &newPath)
         }
         ++it;
     }
-    Path pName = newPath.key(name);
+    Path pName = newPath.withKey(name);
     for (T *el : els)
-        el->updatePathFromOwner(pName.index(--i));
+        el->updatePathFromOwner(pName.withIndex(--i));
 }
 
 template <typename T>
@@ -1935,7 +1900,7 @@ void updatePathFromOwnerQList(QList<T> &list, const Path &newPath)
     auto end = list.end();
     index_type i = 0;
     while (it != end)
-        (it++)->updatePathFromOwner(newPath.index(i++));
+        (it++)->updatePathFromOwner(newPath.withIndex(i++));
 }
 
 constexpr bool domTypeIsObjWrap(DomType k)
@@ -1957,8 +1922,7 @@ constexpr bool domTypeIsObjWrap(DomType k)
     case DomType::Comment:
     case DomType::CommentedElement:
     case DomType::RegionComments:
-    case DomType::FileLocations:
-    case DomType::UpdatedScriptExpression:
+    case DomType::FileLocationsInfo:
         return true;
     default:
         return false;
@@ -2019,7 +1983,7 @@ constexpr bool domTypeIsOwningItem(DomType k)
     case DomType::AstComments:
 
     case DomType::LoadInfo:
-    case DomType::AttachedInfo:
+    case DomType::FileLocationsNode:
 
     case DomType::DomEnvironment:
     case DomType::DomUniverse:
@@ -2034,7 +1998,7 @@ constexpr bool domTypeIsUnattachedOwningItem(DomType k)
     switch (k) {
     case DomType::ScriptExpression:
     case DomType::AstComments:
-    case DomType::AttachedInfo:
+    case DomType::FileLocationsNode:
         return true;
     default:
         return false;
@@ -2056,13 +2020,13 @@ DomItem DomItem::subValueItem(const PathEls::PathComponent &c, const T &value,
                     QCborValue,
                     BaseT> || std::is_base_of_v<QCborArray, BaseT> || std::is_base_of_v<QCborMap, BaseT>) {
         return DomItem(m_top, m_owner, m_ownerPath,
-                       ConstantData(pathFromOwner().appendComponent(c), value, options));
+                       ConstantData(pathFromOwner().withComponent(c), value, options));
     } else if constexpr (std::is_same_v<DomItem, BaseT>) {
         Q_UNUSED(options);
         return value;
     } else if constexpr (IsList<T>::value && !std::is_convertible_v<BaseT, QStringView>) {
         return subListItem(List::fromQList<typename BaseT::value_type>(
-                pathFromOwner().appendComponent(c), value,
+                pathFromOwner().withComponent(c), value,
                 [options](const DomItem &list, const PathEls::PathComponent &p,
                           const typename T::value_type &v) { return list.subValueItem(p, v, options); }));
     } else if constexpr (IsSharedPointerToDomObject<BaseT>::value) {
@@ -2082,11 +2046,11 @@ DomItem DomItem::subDataItem(const PathEls::PathComponent &c, const T &value,
         return this->copy(value);
     } else if constexpr (std::is_base_of_v<QCborValue, BaseT>) {
         return DomItem(m_top, m_owner, m_ownerPath,
-                       ConstantData(pathFromOwner().appendComponent(c), value, options));
+                       ConstantData(pathFromOwner().withComponent(c), value, options));
     } else {
         return DomItem(
                 m_top, m_owner, m_ownerPath,
-                ConstantData(pathFromOwner().appendComponent(c), QCborValue(value), options));
+                ConstantData(pathFromOwner().withComponent(c), QCborValue(value), options));
     }
 }
 
@@ -2135,7 +2099,7 @@ DomItem DomItem::wrap(const PathEls::PathComponent &c, const T &obj) const
     } else if constexpr (IsDomObject<BaseT>::value) {
         if constexpr (domTypeIsObjWrap(BaseT::kindValue) || domTypeIsValueWrap(BaseT::kindValue)) {
             return this->subObjectWrapItem(
-                    SimpleObjectWrap::fromObjectRef(this->pathFromOwner().appendComponent(c), obj));
+                    SimpleObjectWrap::fromObjectRef(this->pathFromOwner().withComponent(c), obj));
         } else if constexpr (domTypeIsDomElement(BaseT::kindValue)) {
             return this->copy(&obj);
         } else {
@@ -2154,14 +2118,14 @@ DomItem DomItem::wrap(const PathEls::PathComponent &c, const T &obj) const
     } else if constexpr (IsMultiMap<BaseT>::value) {
         if constexpr (std::is_same_v<typename BaseT::key_type, QString>) {
             return subMapItem(Map::fromMultiMapRef<typename BaseT::mapped_type>(
-                    pathFromOwner().appendComponent(c), obj));
+                    pathFromOwner().withComponent(c), obj));
         } else {
             Q_ASSERT_X(false, "DomItem::wrap", "non string keys not supported (try .toString()?)");
         }
     } else if constexpr (IsMap<BaseT>::value) {
         if constexpr (std::is_same_v<typename BaseT::key_type, QString>) {
             return subMapItem(Map::fromMapRef<typename BaseT::mapped_type>(
-                    pathFromOwner().appendComponent(c), obj,
+                    pathFromOwner().withComponent(c), obj,
                     [](const DomItem &map, const PathEls::PathComponent &p,
                        const typename BaseT::mapped_type &el) { return map.wrap(p, el); }));
         } else {
@@ -2170,7 +2134,7 @@ DomItem DomItem::wrap(const PathEls::PathComponent &c, const T &obj) const
     } else if constexpr (IsList<BaseT>::value) {
         if constexpr (IsDomObject<typename BaseT::value_type>::value) {
             return subListItem(List::fromQListRef<typename BaseT::value_type>(
-                    pathFromOwner().appendComponent(c), obj,
+                    pathFromOwner().withComponent(c), obj,
                     [](const DomItem &list, const PathEls::PathComponent &p,
                        const typename BaseT::value_type &el) { return list.wrap(p, el); }));
         } else {
