@@ -3,14 +3,14 @@
 
 #include "qqmllscompletion_p.h"
 
-using namespace QLspSpecification;
-using namespace QQmlJS::Dom;
-using namespace Qt::StringLiterals;
-
 QT_BEGIN_NAMESPACE
 
 Q_LOGGING_CATEGORY(QQmlLSCompletionLog, "qt.languageserver.completions")
 
+using namespace QLspSpecification;
+using namespace QQmlJS::Dom;
+using namespace Qt::StringLiterals;
+
 /*!
 \class QQmlLSCompletion
 \internal
@@ -645,7 +645,7 @@ bool QQmlLSCompletion::cursorInFrontOfItem(const DomItem &parentForContext,
                                            const QQmlLSCompletionPosition &positionInfo)
 {
     auto fileLocations = FileLocations::treeOf(parentForContext)->info().fullRegion;
-    return positionInfo.offset() <= fileLocations.offset;
+    return positionInfo.offset() <= fileLocations.begin();
 }
 
 bool QQmlLSCompletion::cursorAfterColon(const DomItem &currentItem,
@@ -657,7 +657,7 @@ bool QQmlLSCompletion::cursorAfterColon(const DomItem &currentItem,
     if (region == location.regions.constEnd())
         return false;
 
-    if (region.value().isValid() && region.value().offset < positionInfo.offset()) {
+    if (region.value().isValid() && region.value().begin() < positionInfo.offset()) {
         return true;
     }
     return false;
@@ -760,10 +760,12 @@ void QQmlLSCompletion::insideQmlObjectCompletion(const DomItem &parentForContext
     }
 
     if (betweenLocations(leftBrace, positionInfo, rightBrace)) {
-        // default/required property completion
-        for (QUtf8StringView view :
-             std::array<QUtf8StringView, 6>{ "", "readonly ", "default ", "default required ",
-                                             "required default ", "required " }) {
+        // default/final/required/readonly property completion
+        constexpr static std::array completions {
+            ""_L1, "default "_L1, "final "_L1, "required "_L1, "readonly "_L1, "default final "_L1,
+            "default required "_L1, "final required "_L1, "final readonly "_L1,
+            "default final required "_L1 };
+        for (QLatin1StringView view : completions) {
             // readonly properties require an initializer
             if (view != QUtf8StringView("readonly ")) {
                 result = makeSnippet(
@@ -815,33 +817,40 @@ void QQmlLSCompletion::insidePropertyDefinitionCompletion(
     const QQmlJS::SourceLocation propertyKeyword = info.regions[PropertyKeywordRegion];
 
     // do completions for the keywords
-    if (positionInfo.offset() < propertyKeyword.offset + propertyKeyword.length) {
+    if (positionInfo.offset() < propertyKeyword.end()) {
         const QQmlJS::SourceLocation readonlyKeyword = info.regions[ReadonlyKeywordRegion];
         const QQmlJS::SourceLocation defaultKeyword = info.regions[DefaultKeywordRegion];
         const QQmlJS::SourceLocation requiredKeyword = info.regions[RequiredKeywordRegion];
+        const QQmlJS::SourceLocation finalKeyword = info.regions[FinalKeywordRegion];
 
         bool completeReadonly = true;
         bool completeRequired = true;
         bool completeDefault = true;
+        bool completeFinal = true;
 
         // if there is already a readonly keyword before the cursor: do not auto complete it again
-        if (readonlyKeyword.isValid() && readonlyKeyword.offset < positionInfo.offset()) {
+        if (readonlyKeyword.isValid() && readonlyKeyword.begin() < positionInfo.offset()) {
             completeReadonly = false;
             // also, required keywords do not like readonly keywords
             completeRequired = false;
         }
 
         // same for required
-        if (requiredKeyword.isValid() && requiredKeyword.offset < positionInfo.offset()) {
+        if (requiredKeyword.isValid() && requiredKeyword.begin() < positionInfo.offset()) {
             completeRequired = false;
             // also, required keywords do not like readonly keywords
             completeReadonly = false;
         }
 
         // same for default
-        if (defaultKeyword.isValid() && defaultKeyword.offset < positionInfo.offset()) {
+        if (defaultKeyword.isValid() && defaultKeyword.begin() < positionInfo.offset()) {
             completeDefault = false;
         }
+
+        // same for final
+        if (finalKeyword.isValid() && finalKeyword.begin() < positionInfo.offset())
+            completeFinal = false;
+
         auto addCompletionKeyword = [&result](QUtf8StringView view, bool complete) {
             if (!complete)
                 return;
@@ -853,6 +862,7 @@ void QQmlLSCompletion::insidePropertyDefinitionCompletion(
         addCompletionKeyword(u8"readonly", completeReadonly);
         addCompletionKeyword(u8"required", completeRequired);
         addCompletionKeyword(u8"default", completeDefault);
+        addCompletionKeyword(u8"final", completeFinal);
         addCompletionKeyword(u8"property", true);
 
         return;
@@ -860,7 +870,7 @@ void QQmlLSCompletion::insidePropertyDefinitionCompletion(
 
     const QQmlJS::SourceLocation propertyIdentifier = info.regions[IdentifierRegion];
     if (propertyKeyword.end() <= positionInfo.offset()
-        && positionInfo.offset() < propertyIdentifier.offset) {
+        && positionInfo.offset() < propertyIdentifier.begin()) {
         suggestReachableTypes(currentItem,
                               LocalSymbolsType::ObjectType | LocalSymbolsType::ValueType,
                               CompletionItemKind::Class, result);
@@ -1937,6 +1947,9 @@ void QQmlLSCompletion::collectCompletions(const DomItem &currentItem,
         case DomType::ScriptSuperLiteral:
             // suppress completions on `super`
             return;
+        case DomType::Comment:
+            // no completion inside of comments
+            return;
 
         // TODO: Implement those statements.
         // In the meanwhile, suppress completions to avoid weird behaviors.
