@@ -17,22 +17,17 @@
 #include <QtQmlCompiler/private/qqmljsutils_p.h>
 
 #include <algorithm>
-#include <iterator>
 #include <memory>
 #include <optional>
-#include <set>
-#include <stack>
-#include <type_traits>
 #include <utility>
-#include <variant>
-
-using namespace QQmlJS::Dom;
-using namespace Qt::StringLiterals;
 
 QT_BEGIN_NAMESPACE
 
 Q_LOGGING_CATEGORY(QQmlLSUtilsLog, "qt.languageserver.utils")
 
+using namespace QQmlJS::Dom;
+using namespace Qt::StringLiterals;
+
 namespace QQmlLSUtils {
 QString qualifiersFrom(const DomItem &el)
 {
@@ -192,29 +187,7 @@ QLspSpecification::Range qmlLocationToLspLocation(Location qmlLocation)
 */
 qsizetype textOffsetFrom(const QString &text, int row, int column)
 {
-    int targetLine = row;
-    qsizetype i = 0;
-    while (i != text.size() && targetLine != 0) {
-        QChar c = text.at(i++);
-        if (c == u'\n') {
-            --targetLine;
-        }
-        if (c == u'\r') {
-            if (i != text.size() && text.at(i) == u'\n')
-                ++i;
-            --targetLine;
-        }
-    }
-    qsizetype leftChars = column;
-    while (i != text.size() && leftChars) {
-        QChar c = text.at(i);
-        if (c == u'\n' || c == u'\r')
-            break;
-        ++i;
-        if (!c.isLowSurrogate())
-            --leftChars;
-    }
-    return i;
+    return QQmlJS::SourceLocation::offsetFrom(text, row + 1, column + 1);
 }
 
 /*!
@@ -229,22 +202,13 @@ qsizetype textOffsetFrom(const QString &text, int row, int column)
 */
 TextPosition textRowAndColumnFrom(const QString &text, qsizetype offset)
 {
-    int row = 0;
-    int column = 0;
-    qsizetype currentLineOffset = 0;
-    for (qsizetype i = 0; i < offset; i++) {
-        QChar c = text[i];
-        if (c == u'\n') {
-            row++;
-            currentLineOffset = i + 1;
-        } else if (c == u'\r') {
-            if (i > 0 && text[i - 1] == u'\n')
-                currentLineOffset++;
-        }
-    }
-    column = offset - currentLineOffset;
+    auto [row, column] = QQmlJS::SourceLocation::rowAndColumnFrom(text, offset);
 
-    return { row, column };
+    // special case: return last character when accessing after end of file
+    if (offset >= text.size())
+        --column;
+
+    return TextPosition{ int(row - 1), int(column - 1) };
 }
 
 static QList<ItemLocation>::const_iterator
@@ -290,7 +254,7 @@ handlePropertyDefinitionAndBindingOverlap(const QList<ItemLocation> &items, qsiz
         // sanity check: is it the definition of the current binding? check if they both have their
         // ':' at the same location
         if (propertyDefinitionColon.isValid() && propertyDefinitionColon == smallestColon
-            && offsetInFile < smallestColon.offset) {
+            && offsetInFile < smallestColon.begin()) {
             return smallestPropertyDefinition;
         }
     }
@@ -316,16 +280,16 @@ static QList<ItemLocation> filterItemsFromTextLocation(const QList<ItemLocation>
     filteredItems.append(*smallest);
 
     const QQmlJS::SourceLocation smallestLoc = smallest->fileLocation->info().fullRegion;
-    const quint32 smallestBegin = smallestLoc.begin();
-    const quint32 smallestEnd = smallestLoc.end();
+    const qsizetype smallestBegin = smallestLoc.begin();
+    const qsizetype smallestEnd = smallestLoc.end();
 
     for (auto it = items.begin(); it != items.end(); it++) {
         if (it == smallest)
             continue;
 
         const QQmlJS::SourceLocation itLoc = it->fileLocation->info().fullRegion;
-        const quint32 itBegin = itLoc.begin();
-        const quint32 itEnd = itLoc.end();
+        const qsizetype itBegin = itLoc.begin();
+        const qsizetype itEnd = itLoc.end();
         if (itBegin == smallestEnd || smallestBegin == itEnd) {
             filteredItems.append(*it);
         }
@@ -364,11 +328,7 @@ QList<ItemLocation> itemsFromTextLocation(const DomItem &file, int line, int cha
 
     enum ComparisonOption { Normal, ExcludePositionAfterLast };
     auto containsTarget = [targetPos](QQmlJS::SourceLocation l, ComparisonOption c) {
-        if constexpr (sizeof(qsizetype) <= sizeof(quint32)) {
-            return l.begin() <= quint32(targetPos) && quint32(targetPos) < l.end() + (c == Normal ? 1 : 0) ;
-        } else {
-            return l.begin() <= targetPos && targetPos < l.end() + (c == Normal ? 1 : 0);
-        }
+        return l.begin() <= targetPos && targetPos < l.end() + (c == Normal ? 1 : 0);
     };
     if (containsTarget(t->info().fullRegion, Normal)) {
         ItemLocation loc;
@@ -385,14 +345,18 @@ QList<ItemLocation> itemsFromTextLocation(const DomItem &file, int line, int cha
         // Exclude the position behind the source location in ScriptBinaryExpressions to avoid
         // returning `owner` in `owner.member` when completion is triggered on the \c{.}. This
         // tells the code for the completion if the completion was triggered on `owner` or on `.`.
+        // Same is true for templateliterals, where ScriptTemplateExpressionParts and
+        // ScriptTemplateStringParts stop overlapping when using ExcludePositionAfterLast.
         const ComparisonOption comparisonOption =
-                iLoc.domItem.internalKind() == QQmlJS::Dom::DomType::ScriptBinaryExpression
+                iLoc.domItem.internalKind() == DomType::ScriptBinaryExpression
+                        || iLoc.domItem.directParent().internalKind()
+                                == DomType::ScriptTemplateLiteral
                 ? ExcludePositionAfterLast
                 : Normal;
 
         auto subEls = iLoc.fileLocation->subItems();
         for (auto it = subEls.begin(); it != subEls.end(); ++it) {
-            auto subLoc = std::static_pointer_cast<AttachedInfoT<FileLocations>>(it.value());
+            auto subLoc = it.value();
             Q_ASSERT(subLoc);
 
             if (containsTarget(subLoc->info().fullRegion, comparisonOption)) {
@@ -764,10 +728,12 @@ static QStringList namesOfPossibleUsages(const QString &name,
 }
 
 template<typename Predicate>
-QQmlJSScope::ConstPtr findDefiningScopeIf(QQmlJSScope::ConstPtr referrerScope, Predicate &&check)
+QQmlJSScope::ConstPtr findDefiningScopeIf(
+        const QQmlJSScope::ConstPtr &referrerScope, Predicate &&check)
 {
     QQmlJSScope::ConstPtr result;
-    QQmlJSUtils::searchBaseAndExtensionTypes(referrerScope, [&](QQmlJSScope::ConstPtr scope) {
+    QQmlJSUtils::searchBaseAndExtensionTypes(
+            referrerScope, [&](const QQmlJSScope::ConstPtr &scope) {
         if (check(scope)) {
             result = scope;
             return true;
@@ -858,9 +824,9 @@ static FieldFilter filterForFindUsages()
 {
     FieldFilter filter{ {},
                         {
-                                { QString(), QString::fromUtf16(Fields::propertyInfos) },
-                                { QString(), QString::fromUtf16(Fields::defaultPropertyName) },
-                                { QString(), QString::fromUtf16(Fields::get) },
+                                { QString(), Fields::propertyInfos.toString() },
+                                { QString(), Fields::defaultPropertyName.toString() },
+                                { QString(), Fields::get.toString() },
                         } };
     return filter;
 };
@@ -1191,18 +1157,16 @@ static std::optional<ExpressionType>
 propertyBindingFromReferrerScope(const QQmlJSScope::ConstPtr &referrerScope, const QString &name,
                                  ResolveOptions options, QQmlJSTypeResolver *resolverForIds)
 {
-    auto bindings = referrerScope->propertyBindings(name);
+    const auto bindings = referrerScope->propertyBindings(name);
     if (bindings.isEmpty())
         return {};
 
-    const auto binding = bindings.front();
-
-    if ((binding.bindingType() != QQmlSA::BindingType::AttachedProperty)
-        && (binding.bindingType() != QQmlSA::BindingType::GroupProperty))
+    const auto binding = bindings.begin();
+    const auto bindingType = binding->bindingType();
+    const bool bindingIsAttached = bindingType == QQmlSA::BindingType::AttachedProperty;
+    if (!bindingIsAttached && bindingType != QQmlSA::BindingType::GroupProperty)
         return {};
 
-    const bool bindingIsAttached = binding.bindingType() == QQmlSA::BindingType::AttachedProperty;
-
     // Generalized grouped properties, like Bindings or PropertyChanges, for example, have bindings
     // starting in an id (like `someId.someProperty: ...`).
     // If `someid` is not a property and is a deferred name, then it should be an id.
@@ -1211,23 +1175,21 @@ propertyBindingFromReferrerScope(const QQmlJSScope::ConstPtr &referrerScope, con
         if (!resolverForIds)
             return {};
 
-        QQmlJSRegisterContent fromId = resolverForIds->scopedType(
-                referrerScope, name, QQmlJSRegisterContent::InvalidLookupIndex,
-                AssumeComponentsAreBound);
-        if (fromId.variant() == QQmlJSRegisterContent::ObjectById)
-            return ExpressionType{ name, fromId.type(), QmlObjectIdIdentifier };
-
-        return ExpressionType{ name, {}, QmlObjectIdIdentifier };
+        return ExpressionType {
+            name,
+            resolverForIds->typeForId(referrerScope, name, AssumeComponentsAreBound),
+            QmlObjectIdIdentifier
+        };
     }
 
     const auto typeIdentifier =
             bindingIsAttached ? AttachedTypeIdentifier : GroupedPropertyIdentifier;
 
-    const auto getScope = [&bindingIsAttached, &binding]() -> QQmlJSScope::ConstPtr {
+    const auto getScope = [bindingIsAttached, binding]() -> QQmlJSScope::ConstPtr {
         if (bindingIsAttached)
-            return binding.attachingType();
+            return binding->attachedType();
 
-        return binding.groupType();
+        return binding->groupType();
     };
 
     switch (options) {
@@ -1251,7 +1213,8 @@ propertyBindingFromReferrerScope(const QQmlJSScope::ConstPtr &referrerScope, con
     Finds the scope within the special elements like Connections,
     PropertyChanges, Bindings or AnchorChanges.
 */
-static QQmlJSScope::ConstPtr findScopeOfSpecialItems(QQmlJSScope::ConstPtr scope, const DomItem &item)
+static QQmlJSScope::ConstPtr findScopeOfSpecialItems(
+        const QQmlJSScope::ConstPtr &scope, const DomItem &item)
 {
     if (!scope)
         return {};
@@ -1262,7 +1225,7 @@ static QQmlJSScope::ConstPtr findScopeOfSpecialItems(QQmlJSScope::ConstPtr scope
                                         u"QQuickAnchorChanges"_s};
 
     const auto special = QQmlJSUtils::searchBaseAndExtensionTypes(
-            scope, [&specialItems](QQmlJSScope::ConstPtr visitedScope) {
+            scope, [&specialItems](const QQmlJSScope::ConstPtr &visitedScope) {
                 const auto typeName = visitedScope->internalName();
                 if (specialItems.contains(typeName))
                     return true;
@@ -1296,18 +1259,16 @@ static QQmlJSScope::ConstPtr findScopeOfSpecialItems(QQmlJSScope::ConstPtr scope
             return {};
 
         // Note: It does not have to be an ID. It can be a property.
-        return resolver->containedType(resolver->scopedType(scope, targetName));
-    } else {
-        if (item.internalKind() == DomType::Binding &&
-            item.field(Fields::bindingType).value().toInteger() == int(BindingType::OnBinding)) {
-                // Binding on sth : {} syntax
-                // Target scope is the current scope
-                return scope;
-        }
-        return scope->parentScope();
+        return resolver->scopedType(scope, targetName);
     }
 
-    return {};
+    if (item.internalKind() == DomType::Binding &&
+        item.field(Fields::bindingType).value().toInteger() == int(BindingType::OnBinding)) {
+            // Binding on sth : {} syntax
+            // Target scope is the current scope
+            return scope;
+    }
+    return scope->parentScope();
 }
 
 /*!
@@ -1368,9 +1329,12 @@ static std::optional<ExpressionType> resolveFieldMemberExpressionType(const DomI
         if (owner->type == JavaScriptIdentifier) {
             return ExpressionType{ name, {}, JavaScriptIdentifier };
         } else if (owner->type == QualifiedModuleIdentifier) {
-            auto resolver = item.fileObject().as<QmlFile>()->typeResolver();
-            if (auto scope = resolveTypeName(resolver, u"%1.%2"_s.arg(*owner->name, name), item,
-                                             options)) {
+            const QmlFile *qmlFile = item.fileObject().as<QmlFile>();
+            if (!qmlFile)
+                return {};
+            if (auto scope = resolveTypeName(
+                        qmlFile->typeResolver(), u"%1.%2"_s.arg(*owner->name, name), item,
+                        options)) {
                 // remove the qualified module name from the type name
                 scope->name = name;
                 return scope;
@@ -1393,15 +1357,16 @@ static std::optional<ExpressionType> resolveFieldMemberExpressionType(const DomI
         // Enumerations should live under the root element scope of the file that defines the enum,
         // therefore use the DomItem to find the root element of the qml file instead of directly
         // using owner->semanticScope.
-        const auto scope = item.goToFile(owner->semanticScope->filePath())
-                                   .rootQmlObject(GoTo::MostLikely)
-                                   .semanticScope();
-        if (scope->hasEnumerationKey(name)) {
-            return ExpressionType{ name, scope, EnumeratorValueIdentifier };
-        }
-        // Or it is a enum name <TypeName>.<EnumName>.<EnumValue>
-        else if (scope->hasEnumeration(name)) {
-            return ExpressionType{ name, scope, EnumeratorIdentifier };
+        if (const auto scope = item.goToFile(owner->semanticScope->filePath())
+                                       .rootQmlObject(GoTo::MostLikely)
+                                       .semanticScope()) {
+            if (scope->hasEnumerationKey(name)) {
+                return ExpressionType{ name, scope, EnumeratorValueIdentifier };
+            }
+            // Or it is a enum name <TypeName>.<EnumName>.<EnumValue>
+            else if (scope->hasEnumeration(name)) {
+                return ExpressionType{ name, scope, EnumeratorIdentifier };
+            }
         }
 
         // check inline components <TypeName>.<InlineComponentName>
@@ -1528,7 +1493,7 @@ static std::optional<ExpressionType> resolveIdentifierExpressionType(const DomIt
 
     const QString name = item.field(Fields::identifier).value().toString();
 
-    if (DomItem definitionOfItem = findJSIdentifierDefinition(item, name)) {
+    if (const DomItem definitionOfItem = findJSIdentifierDefinition(item, name)) {
         Q_ASSERT_X(!definitionOfItem.semanticScope().isNull()
                             && definitionOfItem.semanticScope()->ownJSIdentifier(name),
                     "QQmlLSUtils::findDefinitionOf",
@@ -1554,16 +1519,22 @@ static std::optional<ExpressionType> resolveIdentifierExpressionType(const DomIt
     if (auto scope = methodFromReferrerScope(referrerScope, name, options))
         return scope;
 
-    const auto resolver = item.containingFile().ownerAs<QmlFile>()->typeResolver();
+    const auto qmlFile = item.containingFile().ownerAs<QmlFile>();
+    if (!qmlFile)
+        return {};
+
+    const auto resolver = qmlFile->typeResolver();
     if (!resolver)
         return {};
 
     // check if its found as a property binding
-    if (auto scope = propertyBindingFromReferrerScope(referrerScope, name, options, resolver.get()))
+    if (const auto scope = propertyBindingFromReferrerScope(
+                referrerScope, name, options, resolver.get())) {
         return *scope;
+    }
 
     // check if its an (unqualified) property
-    if (auto scope = propertyFromReferrerScope(referrerScope, name, options))
+    if (const auto scope = propertyFromReferrerScope(referrerScope, name, options))
         return *scope;
 
     if (resolver->seenModuleQualifiers().contains(name))
@@ -1573,11 +1544,10 @@ static std::optional<ExpressionType> resolveIdentifierExpressionType(const DomIt
         return scope;
 
     // check if its an id
-    QQmlJSRegisterContent fromId =
-            resolver->scopedType(referrerScope, name, QQmlJSRegisterContent::InvalidLookupIndex,
-                                 AssumeComponentsAreBound);
-    if (fromId.variant() == QQmlJSRegisterContent::ObjectById)
-        return ExpressionType{ name, fromId.type(), QmlObjectIdIdentifier };
+    if (const QQmlJSScope::ConstPtr fromId
+            = resolver->typeForId(referrerScope, name, AssumeComponentsAreBound)) {
+        return ExpressionType{ name, fromId, QmlObjectIdIdentifier };
+    }
 
     const QQmlJSScope::ConstPtr jsGlobal = resolver->jsGlobalObject();
     // check if its a JS global method
@@ -1599,7 +1569,7 @@ static std::optional<ExpressionType>
 resolveSignalOrPropertyExpressionType(const QString &name, const QQmlJSScope::ConstPtr &scope,
                                       ResolveOptions options)
 {
-    auto signalOrProperty = resolveNameInQmlScope(name, scope);
+    const auto signalOrProperty = resolveNameInQmlScope(name, scope);
     if (!signalOrProperty)
         return {};
 
@@ -1686,7 +1656,8 @@ std::optional<ExpressionType> resolveExpressionType(const QQmlJS::Dom::DomItem &
             if (name == u"id")
                 return ExpressionType{ name, owner.value(), QmlObjectIdIdentifier };
 
-            if (QQmlJSScope::ConstPtr targetScope = findScopeOfSpecialItems(owner.value(), item)) {
+            if (const QQmlJSScope::ConstPtr targetScope
+                    = findScopeOfSpecialItems(owner.value(), item)) {
                 const auto signalOrProperty = resolveNameInQmlScope(name, targetScope);
                 if (!signalOrProperty)
                     return {};
@@ -1753,13 +1724,13 @@ std::optional<ExpressionType> resolveExpressionType(const QQmlJS::Dom::DomItem &
         return ExpressionType{ {}, item.semanticScope(), LambdaMethodIdentifier };
     }
     case DomType::MethodInfo: {
-        auto object = item.as<MethodInfo>();
+        const auto object = item.as<MethodInfo>();
         if (object && object->semanticScope()) {
             std::optional<QQmlJSScope::ConstPtr> scope = object->semanticScope();
             if (!scope)
                 return {};
 
-            if (QQmlJSScope::ConstPtr targetScope =
+            if (const QQmlJSScope::ConstPtr targetScope =
                         findScopeOfSpecialItems(scope.value()->parentScope(), item)) {
                 const auto signalOrProperty = resolveNameInQmlScope(object->name, targetScope);
                 if (!signalOrProperty)
@@ -1782,8 +1753,8 @@ std::optional<ExpressionType> resolveExpressionType(const QQmlJS::Dom::DomItem &
             if (scope.value()->scopeType() == QQmlJSScope::ScopeType::JSFunctionScope)
                 scope = scope.value()->parentScope();
 
-            if (auto result = resolveSignalOrPropertyExpressionType(object->name, scope.value(),
-                                                                    options)) {
+            if (const auto result = resolveSignalOrPropertyExpressionType(
+                        object->name, scope.value(), options)) {
                 return result;
             }
             qDebug(QQmlLSUtilsLog) << "QQmlLSUtils::resolveExpressionType() could not resolve the"
@@ -1804,7 +1775,7 @@ std::optional<ExpressionType> resolveExpressionType(const QQmlJS::Dom::DomItem &
         */
         const auto scope = item.qmlObject().semanticScope();
         const auto name = item.field(Fields::value).value().toString();
-        if (QQmlJSScope::ConstPtr targetScope = findScopeOfSpecialItems(scope, item)) {
+        if (const QQmlJSScope::ConstPtr targetScope = findScopeOfSpecialItems(scope, item)) {
             const auto signalOrProperty = resolveNameInQmlScope(name, targetScope);
             if (!signalOrProperty)
                 return {};
@@ -1824,32 +1795,38 @@ std::optional<ExpressionType> resolveExpressionType(const QQmlJS::Dom::DomItem &
     }
     case DomType::EnumItem: {
         const QString enumValue = item.field(Fields::name).value().toString();
-         QQmlJSScope::ConstPtr referrerScope = item.rootQmlObject(GoTo::MostLikely).semanticScope();
-         if (!referrerScope->hasEnumerationKey(enumValue))
-             return {};
-         switch (options) {
-             // special case: use the owner's scope here, as enums do not have their own
-             // QQmlJSScope.
-         case ResolveActualTypeForFieldMemberExpression:
-         case ResolveOwnerType:
-             return ExpressionType{ enumValue,
-                                    findDefiningScopeForEnumerationKey(referrerScope, enumValue),
-                                    EnumeratorValueIdentifier };
+        const QQmlJSScope::ConstPtr referrerScope
+                = item.rootQmlObject(GoTo::MostLikely).semanticScope();
+        if (!referrerScope->hasEnumerationKey(enumValue))
+            return {};
+        switch (options) {
+            // special case: use the owner's scope here, as enums do not have their own
+            // QQmlJSScope.
+        case ResolveActualTypeForFieldMemberExpression:
+        case ResolveOwnerType:
+            return ExpressionType {
+                enumValue,
+                findDefiningScopeForEnumerationKey(referrerScope, enumValue),
+                EnumeratorValueIdentifier
+            };
          }
          Q_UNREACHABLE_RETURN({});
     }
     case DomType::EnumDecl: {
         const QString enumName = item.field(Fields::name).value().toString();
-        QQmlJSScope::ConstPtr referrerScope = item.rootQmlObject(GoTo::MostLikely).semanticScope();
+        const QQmlJSScope::ConstPtr referrerScope
+                = item.rootQmlObject(GoTo::MostLikely).semanticScope();
         if (!referrerScope->hasEnumeration(enumName))
             return {};
         switch (options) {
         // special case: use the owner's scope here, as enums do not have their own QQmlJSScope.
         case ResolveActualTypeForFieldMemberExpression:
         case ResolveOwnerType:
-            return ExpressionType{ enumName,
-                                   findDefiningScopeForEnumeration(referrerScope, enumName),
-                                   EnumeratorIdentifier };
+            return ExpressionType {
+                enumName,
+                findDefiningScopeForEnumeration(referrerScope, enumName),
+                EnumeratorIdentifier
+            };
         }
 
         Q_UNREACHABLE_RETURN({});
@@ -2120,8 +2097,8 @@ static QQmlJSScope::ConstPtr expressionTypeWithDefinition(const ExpressionType &
     case AttachedTypeIdentifier:
     case GroupedPropertyIdentifier:
     case QmlComponentIdentifier:
-    case QualifiedModuleIdentifier:
     case LambdaMethodIdentifier:
+    case QualifiedModuleIdentifier:
     case NotAnIdentifier:
         return ownerType.semanticScope;
     }
@@ -2383,16 +2360,17 @@ Location Location::from(const QString &fileName, const QQmlJS::SourceLocation &s
     return Location{ fileName, sourceLocation, textRowAndColumnFrom(code, sourceLocation.end()) };
 }
 
-Location Location::from(const QString &fileName, const QString &code, quint32 startLine,
-                        quint32 startCharacter, quint32 length)
+Location Location::from(const QString &fileName, const QString &code, qsizetype startLine,
+                        qsizetype startCharacter, qsizetype length)
 {
-    const quint32 offset = QQmlLSUtils::textOffsetFrom(code, startLine - 1, startCharacter - 1);
-    return from(fileName, QQmlJS::SourceLocation{ offset, length, startLine, startCharacter },
+    const auto offset = QQmlLSUtils::textOffsetFrom(code, startLine - 1, startCharacter - 1);
+    return from(fileName,
+                QQmlJS::SourceLocation::fromQSizeType(offset, length, startLine, startCharacter),
                 code);
 }
 
-Edit Edit::from(const QString &fileName, const QString &code, quint32 startLine,
-                quint32 startCharacter, quint32 length, const QString &newName)
+Edit Edit::from(const QString &fileName, const QString &code, qsizetype startLine,
+                qsizetype startCharacter, qsizetype length, const QString &newName)
 {
     Edit rename;
     rename.location = Location::from(fileName, code, startLine, startCharacter, length);
@@ -2426,9 +2404,9 @@ https://doc.qt.io/qt-6/windows-building.html#step-2-install-build-requirements c
 to have CMake in your path to build Qt. So a developer machine running qmlls has a high chance of
 having CMake in their path, if CMake is installed and used.
 */
-QPair<QString, QStringList> cmakeBuildCommand(const QString &path)
+std::pair<QString, QStringList> cmakeBuildCommand(const QString &path)
 {
-    const QPair<QString, QStringList> result{
+    const std::pair<QString, QStringList> result{
         u"cmake"_s, { u"--build"_s, path, u"-t"_s, u"all_qmltyperegistrations"_s }
     };
     return result;
