@@ -38,14 +38,14 @@ typedef QQmlListCompositor Compositor;
 class QQmlDelegateModelAttachedMetaObject;
 class QQmlAbstractDelegateComponent;
 
-class Q_QMLMODELS_PRIVATE_EXPORT QQmlDelegateModelItemMetaType final
+class Q_QMLMODELS_EXPORT QQmlDelegateModelItemMetaType final
     : public QQmlRefCounted<QQmlDelegateModelItemMetaType>
 {
 public:
     QQmlDelegateModelItemMetaType(QV4::ExecutionEngine *engine, QQmlDelegateModel *model, const QStringList &groupNames);
     ~QQmlDelegateModelItemMetaType();
 
-    void initializeMetaObject();
+    void initializeAttachedMetaObject();
     void initializePrototype();
 
     int parseGroups(const QStringList &groupNames) const;
@@ -54,7 +54,7 @@ public:
     QPointer<QQmlDelegateModel> model;
     const int groupCount;
     QV4::ExecutionEngine * const v4Engine;
-    QQmlDelegateModelAttachedMetaObject *metaObject;
+    QQmlRefPointer<QQmlDelegateModelAttachedMetaObject> attachedMetaObject;
     const QStringList groupNames;
     QV4::PersistentValue modelItemProto;
 };
@@ -90,7 +90,7 @@ public:
                 || ((groups & Compositor::UnresolvedFlag) && (groups & Compositor::GroupMask));
     }
 
-    void Dispose();
+    void dispose();
 
     QObject *modelObject() { return this; }
 
@@ -105,10 +105,11 @@ public:
     int modelIndex() const { return index; }
     virtual void setModelIndex(int idx, int newRow, int newColumn, bool alwaysEmit = false);
 
-    virtual QV4::ReturnedValue get() { return QV4::QObjectWrapper::wrap(v4, this); }
+    virtual QV4::ReturnedValue get() { return QV4::QObjectWrapper::wrap(metaType->v4Engine, this); }
 
     virtual void setValue(const QString &role, const QVariant &value) { Q_UNUSED(role); Q_UNUSED(value); }
     virtual bool resolveIndex(const QQmlAdaptorModel &, int) { return false; }
+    virtual QQmlRefPointer<QQmlContextData> initProxy() { return contextData; }
 
     static QV4::ReturnedValue get_model(const QV4::FunctionObject *, const QV4::Value *thisObject, const QV4::Value *argv, int argc);
     static QV4::ReturnedValue get_groups(const QV4::FunctionObject *, const QV4::Value *thisObject, const QV4::Value *argv, int argc);
@@ -117,18 +118,30 @@ public:
     static QV4::ReturnedValue set_member(QQmlDelegateModelItem *thisItem, uint flag, const QV4::Value &arg);
     static QV4::ReturnedValue get_index(QQmlDelegateModelItem *thisItem, uint flag, const QV4::Value &arg);
 
-    QV4::ExecutionEngine *v4;
-    QQmlRefPointer<QQmlDelegateModelItemMetaType> const metaType;
+    QQmlRefPointer<QQmlDelegateModelItemMetaType> metaType;
     QQmlRefPointer<QQmlContextData> contextData;
     QPointer<QObject> object;
-    QPointer<QQmlDelegateModelAttached> attached;
-    QQDMIncubationTask *incubationTask;
-    QQmlComponent *delegate;
-    int poolTime;
-    int objectRef;
-    int scriptRef;
-    int groups;
-    int index;
+    QQDMIncubationTask *incubationTask = nullptr;
+    QQmlComponent *delegate = nullptr;
+    int objectRef = 0;
+    int scriptRef = 0;
+    int groups = 0;
+
+    QQmlDelegateModelAttached *attached() const
+    {
+        if (!object)
+            return nullptr;
+
+        QQmlData *ddata = QQmlData::get(object);
+        if (!ddata || !ddata->hasExtendedData())
+            return nullptr;
+
+        return static_cast<QQmlDelegateModelAttached *>(
+                ddata->attachedProperties()->value(
+                        QQmlPrivate::attachedPropertiesFunc<QQmlDelegateModel>()));
+    }
+
+    void disableStructuredModelData() { useStructuredModelData = false; }
 
 Q_SIGNALS:
     void modelIndexChanged();
@@ -137,8 +150,11 @@ Q_SIGNALS:
 
 protected:
     void objectDestroyed(QObject *);
-    int row;
-    int column;
+
+    int index = -1;
+    int row = -1;
+    int column = -1;
+    bool useStructuredModelData = true;
 };
 
 namespace QV4 {
@@ -167,14 +183,26 @@ void QV4::Heap::QQmlDelegateModelItemObject::init(QQmlDelegateModelItem *item)
 class QQmlReusableDelegateModelItemsPool
 {
 public:
-    void insertItem(QQmlDelegateModelItem *modelItem);
+    bool insertItem(QQmlDelegateModelItem *modelItem);
     QQmlDelegateModelItem *takeItem(const QQmlComponent *delegate, int newIndexHint);
     void reuseItem(QQmlDelegateModelItem *item, int newModelIndex);
     void drain(int maxPoolTime, std::function<void(QQmlDelegateModelItem *cacheItem)> releaseItem);
-    int size() { return m_reusableItemsPool.size(); }
+    int size()
+    {
+        Q_ASSERT(m_reusableItemsPool.size() <= MaxSize);
+        return int(m_reusableItemsPool.size());
+    }
 
 private:
-    QList<QQmlDelegateModelItem *> m_reusableItemsPool;
+    struct PoolItem
+    {
+        QQmlDelegateModelItem *item = nullptr;
+        int poolTime = 0;
+    };
+
+    static constexpr size_t MaxSize = size_t(std::numeric_limits<int>::max());
+
+    std::vector<PoolItem> m_reusableItemsPool;
 };
 
 class QQmlDelegateModelPrivate;
@@ -186,7 +214,9 @@ public:
         , incubating(nullptr)
         , vdm(l) {}
 
-    void initializeRequiredProperties(QQmlDelegateModelItem *modelItemToIncubate, QObject* object);
+    void initializeRequiredProperties(
+            QQmlDelegateModelItem *modelItemToIncubate, QObject* object,
+            QQmlDelegateModel::DelegateModelAccess access);
     void statusChanged(Status) override;
     void setInitialState(QObject *) override;
 
@@ -201,11 +231,11 @@ public:
 class QQmlDelegateModelGroupEmitter
 {
 public:
-    virtual ~QQmlDelegateModelGroupEmitter() {}
+    virtual ~QQmlDelegateModelGroupEmitter();
     virtual void emitModelUpdated(const QQmlChangeSet &changeSet, bool reset) = 0;
-    virtual void createdPackage(int, QQuickPackage *) {}
-    virtual void initPackage(int, QQuickPackage *) {}
-    virtual void destroyingPackage(QQuickPackage *) {}
+    virtual void createdPackage(int, QQuickPackage *);
+    virtual void initPackage(int, QQuickPackage *);
+    virtual void destroyingPackage(QQuickPackage *);
 
     QIntrusiveListNode emitterNode;
 };
@@ -233,7 +263,7 @@ public:
 
     bool parseIndex(const QV4::Value &value, int *index, Compositor::Group *group) const;
     bool parseGroupArgs(
-            QQmlV4Function *args, Compositor::Group *group, int *index, int *count, int *groups) const;
+            QQmlV4FunctionPtr args, Compositor::Group *group, int *index, int *count, int *groups) const;
 
     Compositor::Group group;
     QPointer<QQmlDelegateModel> model;
@@ -325,7 +355,7 @@ public:
     QQmlStrongJSQObjectReference<QQmlComponent> m_delegate;
     QQmlAbstractDelegateComponent *m_delegateChooser;
     QMetaObject::Connection m_delegateChooserChanged;
-    QQmlDelegateModelItemMetaType *m_cacheMetaType;
+    QQmlRefPointer<QQmlDelegateModelItemMetaType> m_cacheMetaType;
     QPointer<QQmlContext> m_context;
     QQmlDelegateModelParts *m_parts;
     QQmlDelegateModelGroupEmitterList m_pendingParts;
@@ -334,7 +364,6 @@ public:
     QQmlReusableDelegateModelItemsPool m_reusableItemsPool;
     QList<QQDMIncubationTask *> m_finishedIncubating;
     QList<QByteArray> m_watchedRoles;
-    QHash<int, QByteArray> m_roleNamesBeforeReset;
 
     QString m_filterGroup;
 
@@ -348,7 +377,6 @@ public:
     bool m_transaction : 1;
     bool m_incubatorCleanupScheduled : 1;
     bool m_waitingToFetchMore : 1;
-    bool m_maybeResetRoleNames : 1;
 
     union {
         struct {
