@@ -169,9 +169,10 @@ public:
 
         const QQmlAdaptorModel *const model
                 = static_cast<QQmlDMAbstractItemModelData *>(o->d()->item)->type()->model;
-        if (o->d()->item->index >= 0) {
+        if (o->d()->item->modelIndex() >= 0) {
             if (const QAbstractItemModel *const aim = model->aim())
-                RETURN_RESULT(QV4::Encode(aim->hasChildren(aim->index(o->d()->item->index, 0, model->rootIndex))));
+                RETURN_RESULT(QV4::Encode(aim->hasChildren(
+                        aim->index(o->d()->item->modelIndex(), 0, model->rootIndex))));
         }
         RETURN_RESULT(QV4::Encode(false));
     }
@@ -195,9 +196,14 @@ public:
             const QByteArray &propertyName = it.key();
 
             QV4::ScopedString name(scope, v4->newString(QString::fromUtf8(propertyName)));
-            QV4::ExecutionContext *global = v4->rootContext();
-            QV4::ScopedFunctionObject g(scope, v4->memoryManager->allocate<QV4::IndexedBuiltinFunction>(global, propertyId, QQmlDMAbstractItemModelData::get_property));
-            QV4::ScopedFunctionObject s(scope, v4->memoryManager->allocate<QV4::IndexedBuiltinFunction>(global, propertyId, QQmlDMAbstractItemModelData::set_property));
+            QV4::ScopedFunctionObject g(
+                    scope,
+                    v4->memoryManager->allocate<QV4::IndexedBuiltinFunction>(
+                            v4, propertyId, QQmlDMAbstractItemModelData::get_property));
+            QV4::ScopedFunctionObject s(
+                    scope,
+                    v4->memoryManager->allocate<QV4::IndexedBuiltinFunction>(
+                            v4, propertyId, QQmlDMAbstractItemModelData::set_property));
             p->setGetter(g);
             p->setSetter(s);
             proto->insertMember(name, p, QV4::Attr_Accessor|QV4::Attr_NotEnumerable|QV4::Attr_NotConfigurable);
@@ -240,7 +246,7 @@ public:
     {
         if (!metaObject) {
             VDMAbstractItemModelDataType *dataType = const_cast<VDMAbstractItemModelDataType *>(this);
-            dataType->initializeMetaType(model);
+            dataType->initializeMetaObject(model);
         }
 
         if (const QAbstractItemModel *aim = model.aim()) {
@@ -301,11 +307,11 @@ public:
             int index, int row, int column) override
     {
         if (!metaObject)
-            initializeMetaType(model);
+            initializeMetaObject(model);
         return new QQmlDMAbstractItemModelData(metaType, this, index, row, column);
     }
 
-    void initializeMetaType(const QQmlAdaptorModel &model)
+    void initializeMetaObject(const QQmlAdaptorModel &model)
     {
         QMetaObjectBuilder builder;
         QQmlAdaptorModelEngineData::setModelDataType<QQmlDMAbstractItemModelData>(&builder, this);
@@ -317,7 +323,9 @@ public:
             const int propertyId = propertyRoles.size();
             propertyRoles.append(it.key());
             roleNames.insert(it.value(), it.key());
-            QQmlAdaptorModelEngineData::addProperty(&builder, propertyId, it.value(), propertyType);
+            QQmlAdaptorModelEngineData::addProperty(
+                    &builder, propertyId, it.value(), propertyType,
+                    model.delegateModelAccess != QQmlDelegateModel::ReadOnly);
         }
 
         metaObject.reset(builder.toMetaObject());
