@@ -26,44 +26,52 @@ QAbstractItemModel *QQmlTreeModelToTableModel::model() const
     return m_model;
 }
 
+void QQmlTreeModelToTableModel::connectToModel()
+{
+    m_connections = {
+        QObject::connect(m_model, &QAbstractItemModel::destroyed,
+                         this, &QQmlTreeModelToTableModel::modelHasBeenDestroyed),
+        QObject::connect(m_model, &QAbstractItemModel::modelReset,
+                         this, &QQmlTreeModelToTableModel::modelHasBeenReset),
+        QObject::connect(m_model, &QAbstractItemModel::dataChanged,
+                         this, &QQmlTreeModelToTableModel::modelDataChanged),
+
+        QObject::connect(m_model, &QAbstractItemModel::layoutAboutToBeChanged,
+                         this, &QQmlTreeModelToTableModel::modelLayoutAboutToBeChanged),
+        QObject::connect(m_model, &QAbstractItemModel::layoutChanged,
+                         this, &QQmlTreeModelToTableModel::modelLayoutChanged),
+
+        QObject::connect(m_model, &QAbstractItemModel::rowsAboutToBeInserted,
+                         this, &QQmlTreeModelToTableModel::modelRowsAboutToBeInserted),
+        QObject::connect(m_model, &QAbstractItemModel::rowsInserted,
+                         this, &QQmlTreeModelToTableModel::modelRowsInserted),
+        QObject::connect(m_model, &QAbstractItemModel::rowsAboutToBeRemoved,
+                         this, &QQmlTreeModelToTableModel::modelRowsAboutToBeRemoved),
+        QObject::connect(m_model, &QAbstractItemModel::rowsRemoved,
+                         this, &QQmlTreeModelToTableModel::modelRowsRemoved),
+        QObject::connect(m_model, &QAbstractItemModel::rowsAboutToBeMoved,
+                         this, &QQmlTreeModelToTableModel::modelRowsAboutToBeMoved),
+        QObject::connect(m_model, &QAbstractItemModel::rowsMoved,
+                         this, &QQmlTreeModelToTableModel::modelRowsMoved),
+
+        QObject::connect(m_model, &QAbstractItemModel::columnsAboutToBeInserted,
+                         this, &QQmlTreeModelToTableModel::modelColumnsAboutToBeInserted),
+        QObject::connect(m_model, &QAbstractItemModel::columnsAboutToBeRemoved,
+                         this, &QQmlTreeModelToTableModel::modelColumnsAboutToBeRemoved),
+        QObject::connect(m_model, &QAbstractItemModel::columnsInserted,
+                         this, &QQmlTreeModelToTableModel::modelColumnsInserted),
+        QObject::connect(m_model, &QAbstractItemModel::columnsRemoved,
+                         this, &QQmlTreeModelToTableModel::modelColumnsRemoved)
+    };
+}
+
 void QQmlTreeModelToTableModel::setModel(QAbstractItemModel *arg)
 {
-    struct Cx {
-        const char *signal;
-        const char *slot;
-    };
-    const Cx connections[] = {
-        { SIGNAL(destroyed(QObject*)),
-          SLOT(modelHasBeenDestroyed()) },
-        { SIGNAL(modelReset()),
-          SLOT(modelHasBeenReset()) },
-        { SIGNAL(dataChanged(QModelIndex,QModelIndex,QList<int>)),
-          SLOT(modelDataChanged(QModelIndex,QModelIndex,QList<int>)) },
-
-        { SIGNAL(layoutAboutToBeChanged(QList<QPersistentModelIndex>,QAbstractItemModel::LayoutChangeHint)),
-          SLOT(modelLayoutAboutToBeChanged(QList<QPersistentModelIndex>,QAbstractItemModel::LayoutChangeHint)) },
-        { SIGNAL(layoutChanged(QList<QPersistentModelIndex>,QAbstractItemModel::LayoutChangeHint)),
-          SLOT(modelLayoutChanged(QList<QPersistentModelIndex>,QAbstractItemModel::LayoutChangeHint)) },
-
-        { SIGNAL(rowsAboutToBeInserted(QModelIndex,int,int)),
-          SLOT(modelRowsAboutToBeInserted(QModelIndex,int,int)) },
-        { SIGNAL(rowsInserted(QModelIndex,int,int)),
-          SLOT(modelRowsInserted(QModelIndex,int,int)) },
-        { SIGNAL(rowsAboutToBeRemoved(QModelIndex,int,int)),
-          SLOT(modelRowsAboutToBeRemoved(QModelIndex,int,int)) },
-        { SIGNAL(rowsRemoved(QModelIndex,int,int)),
-          SLOT(modelRowsRemoved(QModelIndex,int,int)) },
-        { SIGNAL(rowsAboutToBeMoved(QModelIndex,int,int,QModelIndex,int)),
-          SLOT(modelRowsAboutToBeMoved(QModelIndex,int,int,QModelIndex,int)) },
-        { SIGNAL(rowsMoved(QModelIndex,int,int,QModelIndex,int)),
-          SLOT(modelRowsMoved(QModelIndex,int,int,QModelIndex,int)) },
-        { nullptr, nullptr }
-    };
-
     if (m_model != arg) {
         if (m_model) {
-            for (const Cx *c = &connections[0]; c->signal; c++)
-                disconnect(m_model, c->signal, this, c->slot);
+            for (const auto &c : m_connections)
+                QObject::disconnect(c);
+            m_connections.fill({});
         }
 
         clearModelData();
@@ -73,9 +81,7 @@ void QQmlTreeModelToTableModel::setModel(QAbstractItemModel *arg)
             m_rootIndex = QModelIndex();
 
         if (m_model) {
-            for (const Cx *c = &connections[0]; c->signal; c++)
-                connect(m_model, c->signal, this, c->slot);
-
+            connectToModel();
             showModelTopLevelItems();
         }
 
@@ -288,7 +294,7 @@ QItemSelection  QQmlTreeModelToTableModel::selectionForRowRange(const QModelInde
     if (from > to)
         qSwap(from, to);
 
-    typedef QPair<QModelIndex, QModelIndex> MIPair;
+    typedef std::pair<QModelIndex, QModelIndex> MIPair;
     typedef QHash<QModelIndex, MIPair> MI2MIPairHash;
     MI2MIPairHash ranges;
     QModelIndex firstIndex = m_items.at(from).index;
@@ -482,8 +488,7 @@ void QQmlTreeModelToTableModel::expandRow(int n)
         return;
     item.expanded = true;
     m_expandedItems.insert(item.index);
-    QVector<int> changedRole(1, ExpandedRole);
-    emit dataChanged(index(n, m_column), index(n, m_column), changedRole);
+    emit dataChanged(index(n, 0), index(n, 0), {ExpandedRole});
 
     m_itemsToExpand.append(item);
     expandPendingRows();
@@ -569,8 +574,7 @@ void QQmlTreeModelToTableModel::collapseRow(int n)
     TreeItem &item = m_items[n];
     item.expanded = false;
     m_expandedItems.remove(item.index);
-    QVector<int> changedRole(1, ExpandedRole);
-    queueDataChanged(index(n, m_column), index(n, m_column), changedRole);
+    queueDataChanged(n, n, {ExpandedRole});
     int childrenCount = m_model->rowCount(item.index);
     if ((item.index.flags() & Qt::ItemNeverHasChildren) || !m_model->hasChildren(item.index) || childrenCount == 0)
         return;
@@ -616,12 +620,8 @@ void QQmlTreeModelToTableModel::removeVisibleRows(int startIndex, int endIndex,
 
         /* We need to update the model index for all the items below the removed ones */
         int lastIndex = m_items.size() - 1;
-        if (startIndex <= lastIndex) {
-            const QModelIndex &topLeft = index(startIndex, 0, QModelIndex());
-            const QModelIndex &bottomRight = index(lastIndex, 0, QModelIndex());
-            const QVector<int> changedRole(1, ModelIndexRole);
-            queueDataChanged(topLeft, bottomRight, changedRole);
-        }
+        if (startIndex <= lastIndex)
+            queueDataChanged(startIndex, lastIndex, {ModelIndexRole});
     }
 }
 
@@ -640,7 +640,7 @@ void QQmlTreeModelToTableModel::modelHasBeenReset()
     ASSERT_CONSISTENCY();
 }
 
-void QQmlTreeModelToTableModel::modelDataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight, const QVector<int> &roles)
+void QQmlTreeModelToTableModel::modelDataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight, const QList<int> &roles)
 {
     Q_ASSERT(topLeft.parent() == bottomRight.parent());
     const QModelIndex &parent = topLeft.parent();
@@ -778,9 +778,7 @@ void QQmlTreeModelToTableModel::modelRowsInserted(const QModelIndex & parent, in
     TreeItem item;
     int parentRow = itemIndex(parent);
     if (parentRow >= 0) {
-        const QModelIndex& parentIndex = index(parentRow, m_column);
-        QVector<int> changedRole(1, HasChildrenRole);
-        queueDataChanged(parentIndex, parentIndex, changedRole);
+        queueDataChanged(parentRow, parentRow, {HasChildrenRole});
         item = m_items.at(parentRow);
         if (!item.expanded) {
             ASSERT_CONSISTENCY();
@@ -829,11 +827,8 @@ void QQmlTreeModelToTableModel::modelRowsRemoved(const QModelIndex & parent, int
     Q_UNUSED(start)
     Q_UNUSED(end)
     int parentRow = itemIndex(parent);
-    if (parentRow >= 0) {
-        const QModelIndex& parentIndex = index(parentRow, m_column);
-        QVector<int> changedRole(1, HasChildrenRole);
-        queueDataChanged(parentIndex, parentIndex, changedRole);
-    }
+    if (parentRow >= 0)
+        queueDataChanged(parentRow, parentRow, {HasChildrenRole});
     disableSignalAggregation();
     ASSERT_CONSISTENCY();
 }
@@ -851,10 +846,8 @@ void QQmlTreeModelToTableModel::modelRowsAboutToBeMoved(const QModelIndex & sour
         /* If the destination parent has no children, we'll need to
          * report a change on the HasChildrenRole */
         if (isVisible(destinationParent) && m_model->rowCount(destinationParent) == 0) {
-            const QModelIndex &topLeft = index(itemIndex(destinationParent), 0, QModelIndex());
-            const QModelIndex &bottomRight = topLeft;
-            const QVector<int> changedRole(1, HasChildrenRole);
-            queueDataChanged(topLeft, bottomRight, changedRole);
+            const int parentRow = itemIndex(destinationParent);
+            queueDataChanged(parentRow, parentRow, {HasChildrenRole});
         }
     } else {
         int depthDifference = -1;
@@ -935,17 +928,10 @@ void QQmlTreeModelToTableModel::modelRowsAboutToBeMoved(const QModelIndex & sour
             if (rowCount > 0)
                 bottom = qMax(bottom, lastChildIndex(m_model->index(rowCount - 1, 0, bottomParent)));
         }
-        const QModelIndex &topLeft = index(top, 0, QModelIndex());
-        const QModelIndex &bottomRight = index(bottom, 0, QModelIndex());
-        const QVector<int> changedRole(1, ModelIndexRole);
-        queueDataChanged(topLeft, bottomRight, changedRole);
+        queueDataChanged(top, bottom, {ModelIndexRole});
 
-        if (depthDifference != 0) {
-            const QModelIndex &topLeft = index(bufferCopyOffset, 0, QModelIndex());
-            const QModelIndex &bottomRight = index(bufferCopyOffset + totalMovedCount - 1, 0, QModelIndex());
-            const QVector<int> changedRole(1, DepthRole);
-            queueDataChanged(topLeft, bottomRight, changedRole);
-        }
+        if (depthDifference != 0)
+            queueDataChanged(bufferCopyOffset, bufferCopyOffset + totalMovedCount - 1, {DepthRole});
     }
 }
 
@@ -963,10 +949,7 @@ void QQmlTreeModelToTableModel::modelRowsMoved(const QModelIndex & sourceParent,
     if (isVisible(sourceParent) && m_model->rowCount(sourceParent) == 0) {
         int parentRow = itemIndex(sourceParent);
         collapseRow(parentRow);
-        const QModelIndex &topLeft = index(parentRow, 0, QModelIndex());
-        const QModelIndex &bottomRight = topLeft;
-        const QVector<int> changedRole { ExpandedRole, HasChildrenRole };
-        queueDataChanged(topLeft, bottomRight, changedRole);
+        queueDataChanged(parentRow, parentRow, {ExpandedRole, HasChildrenRole});
     }
 
     disableSignalAggregation();
@@ -974,6 +957,40 @@ void QQmlTreeModelToTableModel::modelRowsMoved(const QModelIndex & sourceParent,
     ASSERT_CONSISTENCY();
 }
 
+void QQmlTreeModelToTableModel::modelColumnsAboutToBeInserted(const QModelIndex & parent, int start, int end)
+{
+    Q_UNUSED(parent);
+    beginInsertColumns({}, start, end);
+}
+
+void QQmlTreeModelToTableModel::modelColumnsAboutToBeRemoved(const QModelIndex & parent, int start, int end)
+{
+    Q_UNUSED(parent);
+    beginRemoveColumns({}, start, end);
+}
+
+void QQmlTreeModelToTableModel::modelColumnsInserted(const QModelIndex & parent, int start, int end)
+{
+    Q_UNUSED(parent);
+    Q_UNUSED(start);
+    Q_UNUSED(end);
+    endInsertColumns();
+    m_items.clear();
+    showModelTopLevelItems();
+    ASSERT_CONSISTENCY();
+}
+
+void QQmlTreeModelToTableModel::modelColumnsRemoved(const QModelIndex & parent, int start, int end)
+{
+    Q_UNUSED(parent);
+    Q_UNUSED(start);
+    Q_UNUSED(end);
+    endRemoveColumns();
+    m_items.clear();
+    showModelTopLevelItems();
+    ASSERT_CONSISTENCY();
+}
+
 void QQmlTreeModelToTableModel::dump() const
 {
     if (!m_model)
@@ -1071,39 +1088,37 @@ void QQmlTreeModelToTableModel::disableSignalAggregation() {
     }
 }
 
-void QQmlTreeModelToTableModel::queueDataChanged(const QModelIndex &topLeft,
-                                               const QModelIndex &bottomRight,
-                                               const QVector<int> &roles)
+void QQmlTreeModelToTableModel::queueDataChanged(int top, int bottom,
+                                                 std::initializer_list<int> roles)
 {
-    if (isAggregatingSignals()) {
-        m_queuedDataChanged.append(DataChangedParams { topLeft, bottomRight, roles });
-    } else {
-        emit dataChanged(topLeft, bottomRight, roles);
-    }
+    if (isAggregatingSignals())
+        m_queuedDataChanged.append(DataChangedParams { top, bottom, roles });
+    else
+        emit dataChanged(index(top, 0), index(bottom, 0), roles);
 }
 
 void QQmlTreeModelToTableModel::emitQueuedSignals()
 {
-    QVector<DataChangedParams> combinedUpdates;
+    QVarLengthArray<DataChangedParams> combinedUpdates;
     /* First, iterate through the queued updates and merge the overlapping ones
      * to reduce the number of updates.
      * We don't merge adjacent updates, because they are typically filed with a
      * different role (a parent row is next to its children).
      */
     for (const DataChangedParams &dataChange : std::as_const(m_queuedDataChanged)) {
-        int startRow = dataChange.topLeft.row();
-        int endRow = dataChange.bottomRight.row();
+        const int startRow = dataChange.top;
+        const int endRow = dataChange.bottom;
         bool merged = false;
         for (DataChangedParams &combined : combinedUpdates) {
-            int combinedStartRow = combined.topLeft.row();
-            int combinedEndRow = combined.bottomRight.row();
+            int combinedStartRow = combined.top;
+            int combinedEndRow = combined.bottom;
             if ((startRow <= combinedStartRow && endRow >= combinedStartRow) ||
                 (startRow <= combinedEndRow && endRow >= combinedEndRow)) {
                 if (startRow < combinedStartRow) {
-                    combined.topLeft = dataChange.topLeft;
+                    combined.top = dataChange.top;
                 }
                 if (endRow > combinedEndRow) {
-                    combined.bottomRight = dataChange.bottomRight;
+                    combined.bottom = dataChange.bottom;
                 }
                 for (int role : dataChange.roles) {
                     if (!combined.roles.contains(role))
@@ -1120,7 +1135,9 @@ void QQmlTreeModelToTableModel::emitQueuedSignals()
 
     /* Finally, emit the dataChanged signals */
     for (const DataChangedParams &dataChange : combinedUpdates) {
-        emit dataChanged(dataChange.topLeft, dataChange.bottomRight, dataChange.roles);
+        const QModelIndex topLeft = index(dataChange.top, 0);
+        const QModelIndex bottomRight = index(dataChange.bottom, 0);
+        emit dataChanged(topLeft, bottomRight, {dataChange.roles.begin(), dataChange.roles.end()});
     }
     m_queuedDataChanged.clear();
 }
