@@ -112,9 +112,12 @@ public:
         WorkerDestroyEvent = QEvent::User + 100
     };
 
-    QQuickWorkerScriptEnginePrivate(QQmlEngine *eng);
+    QQuickWorkerScriptEnginePrivate(QQmlTypeLoader *typeLoader)
+        : m_typeLoader(typeLoader), m_nextId(0)
+    {
+    }
 
-    QQmlEngine *qmlengine;
+    QQmlTypeLoader *m_typeLoader = nullptr;
 
     QMutex m_lock;
     QWaitCondition m_wait;
@@ -125,7 +128,7 @@ public:
     // the worker script.
     QHash<int, QBiPointer<QV4::ExecutionEngine, QQuickWorkerScript>> workers;
 
-    int m_nextId;
+    int m_nextId = 0;
 
     static QV4::ReturnedValue method_sendMessage(const QV4::FunctionObject *, const QV4::Value *thisObject, const QV4::Value *argv, int argc);
     QV4::ExecutionEngine *workerEngine(int id);
@@ -142,11 +145,6 @@ private:
     void reportScriptException(WorkerScript *, const QQmlError &error);
 };
 
-QQuickWorkerScriptEnginePrivate::QQuickWorkerScriptEnginePrivate(QQmlEngine *engine)
-: qmlengine(engine), m_nextId(0)
-{
-}
-
 QV4::ReturnedValue QQuickWorkerScriptEnginePrivate::method_sendMessage(const QV4::FunctionObject *b,
                                                                        const QV4::Value *, const QV4::Value *argv, int argc)
 {
@@ -194,6 +192,8 @@ bool QQuickWorkerScriptEnginePrivate::event(QEvent *event)
 
 QV4::ExecutionEngine *QQuickWorkerScriptEnginePrivate::workerEngine(int id)
 {
+    QMutexLocker locker(&m_lock);
+
     const auto it = workers.find(id);
     if (it == workers.end())
         return nullptr;
@@ -253,12 +253,9 @@ void QQuickWorkerScriptEnginePrivate::processLoad(int id, const QUrl &url)
     script->source = url;
 
     if (fileName.endsWith(QLatin1String(".mjs"))) {
-        auto module = engine->loadModule(url);
-        if (module.compiled) {
-            if (module.compiled->instantiate())
-                module.compiled->evaluate();
-        } else if (module.native) {
-            // Nothing to do. There is no global code in a native module.
+        if (auto module = engine->loadModule(url)) {
+            if (module->instantiate())
+                module->evaluate();
         } else {
             engine->throwError(QStringLiteral("Could not load module file"));
         }
@@ -345,21 +342,19 @@ QQmlError WorkerErrorEvent::error() const
 }
 
 QQuickWorkerScriptEngine::QQuickWorkerScriptEngine(QQmlEngine *parent)
-: QThread(parent), d(new QQuickWorkerScriptEnginePrivate(parent))
+    : QThread(parent)
+    , d(new QQuickWorkerScriptEnginePrivate(&QQmlEnginePrivate::get(parent)->typeLoader))
 {
-    d->m_lock.lock();
     connect(d, SIGNAL(stopThread()), this, SLOT(quit()), Qt::DirectConnection);
+    QMutexLocker locker(&d->m_lock);
     start(QThread::LowestPriority);
     d->m_wait.wait(&d->m_lock);
     d->moveToThread(this);
-    d->m_lock.unlock();
 }
 
 QQuickWorkerScriptEngine::~QQuickWorkerScriptEngine()
 {
-    d->m_lock.lock();
     QCoreApplication::postEvent(d, new QEvent((QEvent::Type)QQuickWorkerScriptEnginePrivate::WorkerDestroyEvent));
-    d->m_lock.unlock();
 
     //We have to force to cleanup the main thread's event queue here
     //to make sure the main GUI release all pending locks/wait conditions which
@@ -393,12 +388,10 @@ WorkerScript::WorkerScript(QV4::ExecutionEngine *engine)
 #if QT_CONFIG(qml_network)
     engine->networkAccessManager = [](QV4::ExecutionEngine *engine) {
         WorkerScript *workerScript = workerScriptExtension(engine);
-        if (workerScript->scriptLocalNAM)
-            return workerScript->scriptLocalNAM.get();
-        if (auto *namFactory = workerScript->p->qmlengine->networkAccessManagerFactory())
-            workerScript->scriptLocalNAM.reset(namFactory->create(workerScript->p));
-        else
-            workerScript->scriptLocalNAM.reset(new QNetworkAccessManager(workerScript->p));
+        if (!workerScript->scriptLocalNAM) {
+            workerScript->scriptLocalNAM.reset(
+                    workerScript->p->m_typeLoader->createNetworkAccessManager(workerScript->p));
+        }
         return workerScript->scriptLocalNAM.get();
     };
 #endif // qml_network
@@ -408,23 +401,26 @@ int QQuickWorkerScriptEngine::registerWorkerScript(QQuickWorkerScript *owner)
 {
     const int id = d->m_nextId++;
 
-    d->m_lock.lock();
+    QMutexLocker locker(&d->m_lock);
     d->workers.insert(id, owner);
-    d->m_lock.unlock();
 
     return id;
 }
 
 void QQuickWorkerScriptEngine::removeWorkerScript(int id)
 {
-    const auto it = d->workers.constFind(id);
-    if (it == d->workers.cend())
-        return;
+    {
+        QMutexLocker locker(&d->m_lock);
+        const auto it = d->workers.find(id);
+        if (it == d->workers.end())
+            return;
 
-    if (it->isT1()) {
-        QV4::ExecutionEngine *engine = it->asT1();
-        workerScriptExtension(engine)->owner = nullptr;
+        if (it->isT1())
+            workerScriptExtension(it->asT1())->owner = nullptr;
+        else
+            *it = static_cast<QQuickWorkerScript *>(nullptr);
     }
+
     QCoreApplication::postEvent(d, new WorkerRemoveEvent(id));
 }
 
@@ -440,12 +436,14 @@ void QQuickWorkerScriptEngine::sendMessage(int id, const QByteArray &data)
 
 void QQuickWorkerScriptEngine::run()
 {
-    d->m_lock.lock();
-    d->m_wait.wakeAll();
-    d->m_lock.unlock();
+    {
+        QMutexLocker locker(&d->m_lock);
+        d->m_wait.wakeAll();
+    }
 
     exec();
 
+    QMutexLocker locker(&d->m_lock);
     for (auto it = d->workers.begin(), end = d->workers.end(); it != end; ++it) {
         if (it->isT1())
             delete it->asT1();
