@@ -243,6 +243,24 @@ QT_BEGIN_NAMESPACE
 
     By default this property is \c false.
 */
+/*! \qmlproperty Item QtQuick::Accessible::labelledBy
+    \brief This property holds the item that is used as a label for this item.
+
+    Setting this property automatically sets up the labelFor relation of the other object.
+
+    By default this property is \c undefined.
+
+    \since 6.10
+ */
+/*! \qmlproperty Item QtQuick::Accessible::labelFor
+    \brief This property holds the item that this item is a label for.
+
+    Setting this property automatically sets up the labelledBy relation of the other object.
+
+    By default this property is \c undefined.
+
+    \since 6.10
+ */
 
 /*!
     \qmlsignal QtQuick::Accessible::pressAction()
@@ -346,12 +364,19 @@ QQuickAccessibleAttached::QQuickAccessibleAttached(QObject *parent)
             return;
         };
         const QMetaObject &smo = staticMetaObject;
-        static const int valueChangedIndex = smo.indexOfSlot("valueChanged()");
-        connectPropertyChangeSignal("value", "valueChanged", valueChangedIndex);
 
-        static const int cursorPositionChangedIndex = smo.indexOfSlot("cursorPositionChanged()");
-        connectPropertyChangeSignal("cursorPosition", "cursorPositionChanged",
-                                    cursorPositionChangedIndex);
+        QAccessibleInterface *iface = ev.accessibleInterface();
+        if (iface && iface->valueInterface()) {
+            static const int valueChangedIndex = smo.indexOfSlot("valueChanged()");
+            connectPropertyChangeSignal("value", "valueChanged", valueChangedIndex);
+        }
+
+        if (iface && iface->textInterface()) {
+            static const int cursorPositionChangedIndex =
+                    smo.indexOfSlot("cursorPositionChanged()");
+            connectPropertyChangeSignal("cursorPosition", "cursorPositionChanged",
+                                        cursorPositionChangedIndex);
+        }
     }
 
     if (!sigPress.isValid()) {
@@ -429,6 +454,14 @@ void QQuickAccessibleAttached::setNameImplicitly(const QString &name)
     m_nameExplicitlySet = false;
 }
 
+void QQuickAccessibleAttached::setDescriptionImplicitly(const QString &desc)
+{
+    if (m_descriptionExplicitlySet)
+        return;
+    setDescription(desc);
+    m_descriptionExplicitlySet = false;
+}
+
 QQuickAccessibleAttached *QQuickAccessibleAttached::qmlAttachedProperties(QObject *obj)
 {
     return new QQuickAccessibleAttached(obj);
@@ -445,6 +478,9 @@ void QQuickAccessibleAttached::setIgnored(bool ignored)
     auto item = qobject_cast<QQuickItem *>(parent());
     if (item && this->ignored() != ignored) {
         item->d_func()->isAccessible = !ignored;
+        QAccessibleEvent event(item,
+                               ignored ? QAccessible::ObjectDestroyed : QAccessible::ObjectCreated);
+        QAccessible::updateAccessibility(&event);
         emit ignoredChanged();
     }
 }
@@ -583,6 +619,16 @@ void QQuickAccessibleAttached::announce(const QString &message, QAccessible::Ann
     QAccessible::updateAccessibility(&event);
 }
 
+QQuickItem *QQuickAccessibleAttached::findRelation(QAccessible::Relation relation) const
+{
+    const auto it = std::find_if(m_relations.cbegin(), m_relations.cend(),
+                                 [relation](const auto &rel) {
+        return rel.second == relation;
+    });
+
+    return it != m_relations.cend() ? it->first : nullptr;
+}
+
 QT_END_NAMESPACE
 
 #include "moc_qquickaccessibleattached_p.cpp"
