@@ -4,6 +4,7 @@
 #include "qquickloader_p_p.h"
 
 #include <QtQml/qqmlinfo.h>
+#include <QtQml/qqmlcomponent.h>
 
 #include <private/qqmlengine_p.h>
 #include <private/qqmlglobal_p.h>
@@ -13,8 +14,6 @@
 
 QT_BEGIN_NAMESPACE
 
-Q_DECLARE_LOGGING_CATEGORY(lcTransient)
-
 static const QQuickItemPrivate::ChangeTypes watchedChanges
     = QQuickItemPrivate::Geometry | QQuickItemPrivate::ImplicitWidth | QQuickItemPrivate::ImplicitHeight;
 
@@ -649,8 +648,8 @@ void QQuickLoaderPrivate::setInitialState(QObject *obj)
         return;
 
     QQmlComponentPrivate *d = QQmlComponentPrivate::get(component);
-    Q_ASSERT(d && d->engine);
-    QV4::ExecutionEngine *v4 = d->engine->handle();
+    Q_ASSERT(d && d->engine());
+    QV4::ExecutionEngine *v4 = d->engine()->handle();
     Q_ASSERT(v4);
     QV4::Scope scope(v4);
     QV4::ScopedValue ipv(scope, initialPropertyValues.value());
