@@ -17,7 +17,7 @@
 
 QT_BEGIN_NAMESPACE
 
-Q_LOGGING_CATEGORY(lcPA, "qt.quick.pincharea")
+Q_STATIC_LOGGING_CATEGORY(lcPA, "qt.quick.pincharea")
 
 /*!
     \qmltype PinchEvent
@@ -26,8 +26,6 @@ Q_LOGGING_CATEGORY(lcPA, "qt.quick.pincharea")
     \ingroup qtquick-input-events
     \brief For specifying information about a pinch event.
 
-    \b {The PinchEvent type was added in QtQuick 1.1}
-
     The \c center, \c startCenter, \c previousCenter properties provide the center position between the two touch points.
 
     The \c scale and \c previousScale properties provide the scale factor.
@@ -43,9 +41,9 @@ Q_LOGGING_CATEGORY(lcPA, "qt.quick.pincharea")
 */
 
 /*!
-    \qmlproperty QPointF QtQuick::PinchEvent::center
-    \qmlproperty QPointF QtQuick::PinchEvent::startCenter
-    \qmlproperty QPointF QtQuick::PinchEvent::previousCenter
+    \qmlproperty point QtQuick::PinchEvent::center
+    \qmlproperty point QtQuick::PinchEvent::startCenter
+    \qmlproperty point QtQuick::PinchEvent::previousCenter
 
     These properties hold the position of the center point between the two touch points.
 
@@ -87,10 +85,10 @@ Q_LOGGING_CATEGORY(lcPA, "qt.quick.pincharea")
 */
 
 /*!
-    \qmlproperty QPointF QtQuick::PinchEvent::point1
-    \qmlproperty QPointF QtQuick::PinchEvent::startPoint1
-    \qmlproperty QPointF QtQuick::PinchEvent::point2
-    \qmlproperty QPointF QtQuick::PinchEvent::startPoint2
+    \qmlproperty point QtQuick::PinchEvent::point1
+    \qmlproperty point QtQuick::PinchEvent::startPoint1
+    \qmlproperty point QtQuick::PinchEvent::point2
+    \qmlproperty point QtQuick::PinchEvent::startPoint2
 
     These properties provide the actual touch points generating the pinch.
 
@@ -137,8 +135,6 @@ QQuickPinchAreaPrivate::~QQuickPinchAreaPrivate()
     \inherits Item
     \brief Enables simple pinch gesture handling.
 
-    \b {The PinchArea type was added in QtQuick 1.1}
-
     A PinchArea is an invisible item that is typically used in conjunction with
     a visible item in order to provide pinch gesture handling for that item.
 
