@@ -2709,7 +2709,7 @@ void QQuickAnimationPropertyUpdater::setValue(qreal v)
     for (int ii = 0; ii < actions.size(); ++ii) {
         QQuickStateAction &action = actions[ii];
 
-        if (v == 1.) {
+        if (v == qreal(1.0) && extendedInterpolator == nullptr) {
             QQmlPropertyPrivate::write(action.property, action.toValue, QQmlPropertyData::BypassInterceptor | QQmlPropertyData::DontRemoveBinding);
         } else {
             if (!fromIsSourced && !fromIsDefined) {
@@ -2725,8 +2725,21 @@ void QQuickAnimationPropertyUpdater::setValue(qreal v)
                     interpolator = QVariantAnimationPrivate::getInterpolator(prevInterpolatorType);
                 }
             }
-            if (interpolator)
-                QQmlPropertyPrivate::write(action.property, interpolator(action.fromValue.constData(), action.toValue.constData(), v), QQmlPropertyData::BypassInterceptor | QQmlPropertyData::DontRemoveBinding);
+
+            QVariant interpolated;
+            if (extendedInterpolator) {
+                QVariant current = action.property.read();
+                interpolated = extendedInterpolator(action.fromValue.constData(),
+                                                    action.toValue.constData(),
+                                                    current,
+                                                    v);
+
+            } else if (interpolator) {
+                interpolated = interpolator(action.fromValue.constData(), action.toValue.constData(), v);
+            }
+            QQmlPropertyPrivate::write(action.property,
+                                       interpolated,
+                                       QQmlPropertyData::BypassInterceptor | QQmlPropertyData::DontRemoveBinding);
         }
         if (deleted)
             return;
@@ -2827,36 +2840,37 @@ QQuickStateActions QQuickPropertyAnimation::createTransitionActions(QQuickStateA
         }
     }
 
-    if (!hasExplicit)
-    for (int ii = 0; ii < actions.size(); ++ii) {
-        QQuickStateAction &action = actions[ii];
+    if (!hasExplicit) {
+        for (int ii = 0; ii < actions.size(); ++ii) {
+            QQuickStateAction &action = actions[ii];
 
-        QObject *obj = action.property.object();
-        QString propertyName = action.property.name();
-        QObject *sObj = action.specifiedObject;
-        QString sPropertyName = action.specifiedProperty;
-        bool same = (obj == sObj);
+            QObject *obj = action.property.object();
+            QString propertyName = action.property.name();
+            QObject *sObj = action.specifiedObject;
+            QString sPropertyName = action.specifiedProperty;
+            bool same = (obj == sObj);
 
-        if ((targets.isEmpty() || targets.contains(obj) || (!same && targets.contains(sObj))) &&
-           (!d->exclude.contains(obj)) && (same || (!d->exclude.contains(sObj))) &&
-           (props.contains(propertyName) || (!same && props.contains(sPropertyName))
-               || (useType && action.property.propertyType() == d->interpolatorType))) {
-            QQuickStateAction myAction = action;
+            if ((targets.isEmpty() || targets.contains(obj) || (!same && targets.contains(sObj))) &&
+               (!d->exclude.contains(obj)) && (same || (!d->exclude.contains(sObj))) &&
+               (props.contains(propertyName) || (!same && props.contains(sPropertyName))
+                   || (useType && action.property.propertyType() == d->interpolatorType))) {
+                QQuickStateAction myAction = action;
 
-            if (d->fromIsDefined)
-                myAction.fromValue = d->from;
-            else
-                myAction.fromValue = QVariant();
-            if (d->toIsDefined)
-                myAction.toValue = d->to;
+                if (d->fromIsDefined)
+                    myAction.fromValue = d->from;
+                else
+                    myAction.fromValue = QVariant();
+                if (d->toIsDefined)
+                    myAction.toValue = d->to;
 
-            d->convertVariant(myAction.fromValue, d->interpolatorType ? QMetaType(d->interpolatorType) : myAction.property.propertyMetaType());
-            d->convertVariant(myAction.toValue, d->interpolatorType ? QMetaType(d->interpolatorType) : myAction.property.propertyMetaType());
+                d->convertVariant(myAction.fromValue, d->interpolatorType ? QMetaType(d->interpolatorType) : myAction.property.propertyMetaType());
+                d->convertVariant(myAction.toValue, d->interpolatorType ? QMetaType(d->interpolatorType) : myAction.property.propertyMetaType());
 
-            modified << action.property;
+                modified << action.property;
 
-            newActions << myAction;
-            action.fromValue = myAction.toValue;
+                newActions << myAction;
+                action.fromValue = myAction.toValue;
+            }
         }
     }
     return newActions;
@@ -2879,6 +2893,7 @@ QAbstractAnimationJob* QQuickPropertyAnimation::transition(QQuickStateActions &a
         QQuickAnimationPropertyUpdater *data = new QQuickAnimationPropertyUpdater;
         data->interpolatorType = d->interpolatorType;
         data->interpolator = d->interpolator;
+        data->extendedInterpolator = d->extendedInterpolator;
         data->reverse = direction == Backward ? true : false;
         data->fromIsSourced = false;
         data->fromIsDefined = d->fromIsDefined;
