@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtQuick module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qquickbehavior_p.h"
 
@@ -50,6 +14,7 @@
 #include <private/qquickanimatorjob_p.h>
 
 #include <private/qobject_p.h>
+#include <QtCore/qpointer.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -187,14 +152,14 @@ public:
 UntypedProxyProperty::UntypedProxyProperty(QUntypedBindable bindable, QQuickBehaviorPrivate *behavior) :
     m_sourcePropertyData(QUntypedBindablePrivate::getPropertyData(bindable)),
     m_sourceInterface(QUntypedBindablePrivate::getInterface(bindable)),
-    m_storage(QVariant(QUntypedBindablePrivate::getInterface(bindable)->metaType()))
+    m_storage(QVariant(bindable.metaType()))
 {
     behavior->setSource(m_bindingData);
 }
 
 /*!
     \qmltype Behavior
-    \instantiates QQuickBehavior
+    \nativetype QQuickBehavior
     \inqmlmodule QtQuick
     \ingroup qtquick-transitions-animations
     \ingroup qtquick-interceptors
@@ -218,7 +183,7 @@ UntypedProxyProperty::UntypedProxyProperty(QUntypedBindable bindable, QQuickBeha
     state change. For general advice on using Behaviors to animate state changes, see
     \l{Using Qt Quick Behaviors with States}.
 
-    \sa {Animation and Transitions in Qt Quick}, {Qt Quick Examples - Animation#Behaviors}{Behavior example}, {Qt QML}
+    \sa {Animation and Transitions in Qt Quick}, {Qt Quick Examples - Animation#Behaviors}{Behavior example}, {Qt Qml}
 */
 
 
@@ -442,7 +407,7 @@ void QQuickBehavior::write(const QVariant &value)
 
     QList<QQmlProperty> after;
     auto *newInstance = d->animation->transition(actions, after, QQuickAbstractAnimation::Forward);
-    Q_ASSERT(newInstance != d->animationInstance);
+    Q_ASSERT(!newInstance || newInstance != d->animationInstance);
     delete d->animationInstance;
     d->animationInstance = newInstance;
 
