@@ -14,9 +14,60 @@
 
 QT_BEGIN_NAMESPACE
 
+Q_STATIC_LOGGING_CATEGORY(lcPath, "qt.quick.shapes.path")
+
+void QQuickPathPrivate::enablePathElement(QQuickPathElement *pathElement)
+{
+    Q_Q(QQuickPath);
+
+    if (QQuickCurve *curve = qobject_cast<QQuickCurve *>(pathElement)) {
+        _pathCurves.append(curve);
+    } else if (QQuickPathText *text = qobject_cast<QQuickPathText *>(pathElement)) {
+        _pathTexts.append(text);
+    } else {
+        QQuickPathAttribute *attribute = qobject_cast<QQuickPathAttribute *>(pathElement);
+        if (attribute && !_attributes.contains(attribute->name()))
+            _attributes.append(attribute->name());
+    }
+
+    // There may be multiple entries of the same value
+    if (!_pathElements.contains(pathElement))
+        q->connect(pathElement, SIGNAL(changed()), q, SLOT(processPath()));
+}
+
+void QQuickPathPrivate::disablePathElement(QQuickPathElement *pathElement)
+{
+    Q_Q(QQuickPath);
+
+    if (QQuickCurve *curve = qobject_cast<QQuickCurve *>(pathElement)) {
+        _pathCurves.removeOne(curve);
+    } else if (QQuickPathText *text = qobject_cast<QQuickPathText *>(pathElement)) {
+        _pathTexts.removeOne(text);
+    } else if (QQuickPathAttribute *attribute = qobject_cast<QQuickPathAttribute *>(pathElement)) {
+        const QString name = attribute->name();
+        bool found = false;
+
+        // TODO: This is rather expensive. Why do the attributes have to be unique?
+        for (QQuickPathElement *other : std::as_const(_pathElements)) {
+            QQuickPathAttribute *otherAttribute = qobject_cast<QQuickPathAttribute *>(other);
+            if (otherAttribute && otherAttribute->name() == name) {
+                found = true;
+                break;
+            }
+        }
+
+        if (!found)
+            _attributes.removeOne(name);
+    }
+
+    // There may be multiple entries of the same value
+    if (!_pathElements.contains(pathElement))
+        q->disconnect(pathElement, SIGNAL(changed()), q, SLOT(processPath()));
+}
+
 /*!
     \qmltype PathElement
-    \instantiates QQuickPathElement
+    \nativetype QQuickPathElement
     \inqmlmodule QtQuick
     \ingroup qtquick-animation-paths
     \brief PathElement is the base path type.
@@ -25,12 +76,12 @@ QT_BEGIN_NAMESPACE
     be instantiated.
 
     \sa Path, PathAttribute, PathPercent, PathLine, PathPolyline, PathQuad, PathCubic, PathArc,
-        PathAngleArc, PathCurve, PathSvg
+        PathAngleArc, PathCurve, PathSvg, PathRectangle
 */
 
 /*!
     \qmltype Path
-    \instantiates QQuickPath
+    \nativetype QQuickPath
     \inqmlmodule QtQuick
     \ingroup qtquick-animation-paths
     \brief Defines a path for use by \l PathView and \l Shape.
@@ -70,7 +121,7 @@ QT_BEGIN_NAMESPACE
         \li Yes
         \li Yes
     \row
-    \li PathMultiLine
+    \li PathMultiline
         \li Yes
         \li Yes
         \li Yes
@@ -99,6 +150,11 @@ QT_BEGIN_NAMESPACE
         \li Yes
         \li Yes
         \li Yes
+    \row
+        \li PathRectangle
+        \li Yes
+        \li Yes
+        \li Yes
     \row
         \li PathAttribute
         \li Yes
@@ -119,7 +175,7 @@ QT_BEGIN_NAMESPACE
     \note Path is a non-visual type; it does not display anything on its own.
     To draw a path, use \l Shape.
 
-    \sa PathView, Shape, PathAttribute, PathPercent, PathLine, PathPolyline, PathMove, PathQuad, PathCubic, PathArc, PathAngleArc, PathCurve, PathSvg
+    \sa PathView, Shape, PathAttribute, PathPercent, PathLine, PathPolyline, PathMove, PathQuad, PathCubic, PathArc, PathAngleArc, PathCurve, PathSvg, PathRectangle
 */
 QQuickPath::QQuickPath(QObject *parent)
  : QObject(*(new QQuickPathPrivate), parent)
@@ -143,7 +199,7 @@ QQuickPath::~QQuickPath()
 qreal QQuickPath::startX() const
 {
     Q_D(const QQuickPath);
-    return d->startX.isNull ? 0 : d->startX.value;
+    return d->startX.isValid() ? d->startX.value() : 0;
 }
 
 void QQuickPath::setStartX(qreal x)
@@ -165,7 +221,7 @@ bool QQuickPath::hasStartX() const
 qreal QQuickPath::startY() const
 {
     Q_D(const QQuickPath);
-    return d->startY.isNull ? 0 : d->startY.value;
+    return d->startY.isValid() ? d->startY.value() : 0;
 }
 
 void QQuickPath::setStartY(qreal y)
@@ -210,6 +266,7 @@ bool QQuickPath::isClosed() const
         \li \l PathArc - an arc to a given position with a radius.
         \li \l PathAngleArc - an arc specified by center point, radii, and angles.
         \li \l PathSvg - a path specified as an SVG path data string.
+        \li \l PathRectangle - a rectangle with a given position and size
         \li \l PathCurve - a point on a Catmull-Rom curve.
         \li \l PathAttribute - an attribute at a given position in the path.
         \li \l PathPercent - a way to spread out items along various segments of the path.
@@ -225,7 +282,9 @@ QQmlListProperty<QQuickPathElement> QQuickPath::pathElements()
                                                pathElements_append,
                                                pathElements_count,
                                                pathElements_at,
-                                               pathElements_clear);
+                                               pathElements_clear,
+                                               pathElements_replace,
+                                               pathElements_removeLast);
 }
 
 static QQuickPathPrivate *privatePath(QObject *object)
@@ -245,26 +304,7 @@ QQuickPathElement *QQuickPath::pathElements_at(QQmlListProperty<QQuickPathElemen
 void QQuickPath::pathElements_append(QQmlListProperty<QQuickPathElement> *property, QQuickPathElement *pathElement)
 {
     QQuickPathPrivate *d = privatePath(property->object);
-    QQuickPath *path = static_cast<QQuickPath*>(property->object);
-
-    d->_pathElements.append(pathElement);
-
-    if (d->componentComplete) {
-        QQuickCurve *curve = qobject_cast<QQuickCurve *>(pathElement);
-        if (curve)
-            d->_pathCurves.append(curve);
-        else if (QQuickPathText *text = qobject_cast<QQuickPathText *>(pathElement))
-            d->_pathTexts.append(text);
-        else {
-            QQuickPathAttribute *attribute = qobject_cast<QQuickPathAttribute *>(pathElement);
-            if (attribute && !d->_attributes.contains(attribute->name()))
-                d->_attributes.append(attribute->name());
-        }
-
-        path->processPath();
-
-        connect(pathElement, SIGNAL(changed()), path, SLOT(processPath()));
-    }
+    d->appendPathElement(pathElement);
 }
 
 qsizetype QQuickPath::pathElements_count(QQmlListProperty<QQuickPathElement> *property)
@@ -284,6 +324,20 @@ void QQuickPath::pathElements_clear(QQmlListProperty<QQuickPathElement> *propert
     d->_pathCurves.clear();
     d->_pointCache.clear();
     d->_pathTexts.clear();
+    d->_path.clear();
+    emit path->changed();
+}
+
+void QQuickPath::pathElements_replace(
+        QQmlListProperty<QQuickPathElement> *property, qsizetype position,
+        QQuickPathElement *pathElement)
+{
+    privatePath(property->object)->replacePathElement(position, pathElement);
+}
+
+void QQuickPath::pathElements_removeLast(QQmlListProperty<QQuickPathElement> *property)
+{
+    privatePath(property->object)->removeLastPathElement();
 }
 
 void QQuickPath::interpolate(int idx, const QString &name, qreal value)
@@ -362,6 +416,26 @@ void QQuickPath::processPath()
     if (!d->componentComplete)
         return;
 
+    if (!d->asynchronous) {
+        doProcessPath();
+    } else if (!d->processPending) {
+        d->processPending = true;
+        QMetaObject::invokeMethod(this, &QQuickPath::doProcessPath, Qt::QueuedConnection);
+    }
+}
+
+void QQuickPath::doProcessPath()
+{
+    Q_D(QQuickPath);
+
+    d->processPending = false;
+
+    if (!d->componentComplete)
+        return;
+
+    if (d->useCustomPath)
+        return;
+
     d->_pointCache.clear();
     d->prevBez.isValid = false;
 
@@ -408,14 +482,16 @@ QPainterPath QQuickPath::createPath(const QPointF &startPoint, const QPointF &en
         first.values[attributes.at(ii)] = 0;
     attributePoints << first;
 
-    qreal startX = d->startX.isValid() ? d->startX.value : startPoint.x();
-    qreal startY = d->startY.isValid() ? d->startY.value : startPoint.y();
+    qreal startX = d->startX.isValid() ? d->startX.value() : startPoint.x();
+    qreal startY = d->startY.isValid() ? d->startY.value() : startPoint.y();
     path.moveTo(startX, startY);
 
     const QString percentString = QStringLiteral("_qfx_percent");
 
     bool usesPercent = false;
     int index = 0;
+    qCDebug(lcPath).nospace() << this << " is creating path starting at " << startPoint
+        << " and ending at " << endPoint << " with " << pathLength << " element(s):";
     for (QQuickPathElement *pathElement : std::as_const(d->_pathElements)) {
         if (QQuickCurve *curve = qobject_cast<QQuickCurve *>(pathElement)) {
             QQuickPathData data;
@@ -427,17 +503,21 @@ QPainterPath QQuickPath::createPath(const QPointF &startPoint, const QPointF &en
             p.origpercent = path.length();
             attributePoints << p;
             ++index;
+            qCDebug(lcPath) << "- index" << index << "curve:" << data.curves.at(data.index);
         } else if (QQuickPathAttribute *attribute = qobject_cast<QQuickPathAttribute *>(pathElement)) {
             AttributePoint &point = attributePoints.last();
             point.values[attribute->name()] = attribute->value();
             interpolate(attributePoints, attributePoints.size() - 1, attribute->name(), attribute->value());
+            qCDebug(lcPath) << "- index" << index << "attribute:" << attribute->value();
         } else if (QQuickPathPercent *percent = qobject_cast<QQuickPathPercent *>(pathElement)) {
             AttributePoint &point = attributePoints.last();
             point.values[percentString] = percent->value();
             interpolate(attributePoints, attributePoints.size() - 1, percentString, percent->value());
+            qCDebug(lcPath) << "- index" << index << "percent:" << percent->value();
             usesPercent = true;
         } else if (QQuickPathText *text = qobject_cast<QQuickPathText *>(pathElement)) {
             text->addToPath(path);
+            qCDebug(lcPath) << "- index" << index << "text:" << text->text();
         }
     }
 
@@ -493,17 +573,20 @@ QPainterPath QQuickPath::createShapePath(const QPointF &startPoint, const QPoint
 
     QPainterPath path;
 
-    qreal startX = d->startX.isValid() ? d->startX.value : startPoint.x();
-    qreal startY = d->startY.isValid() ? d->startY.value : startPoint.y();
+    qreal startX = d->startX.isValid() ? d->startX.value() : startPoint.x();
+    qreal startY = d->startY.isValid() ? d->startY.value() : startPoint.y();
     path.moveTo(startX, startY);
 
     int index = 0;
+    qCDebug(lcPath).nospace() << this << " is creating shape path from " << d->_pathCurves.size()
+        << " curve(s) with endPoint " << endPoint << ":";
     for (QQuickCurve *curve : std::as_const(d->_pathCurves)) {
         QQuickPathData data;
         data.index = index;
         data.endPoint = endPoint;
         data.curves = d->_pathCurves;
         curve->addToPath(path, data);
+        qCDebug(lcPath) << "- index" << data.index << data.curves.at(data.index);
         ++index;
     }
 
@@ -516,10 +599,10 @@ QPainterPath QQuickPath::createShapePath(const QPointF &startPoint, const QPoint
     }
     scalePath(path, d->scale);
 
-    // Note: Length of paths inside ShapePath is not used, so currently
-    // length is always 0. This avoids potentially heavy path.length()
-    //pathLength = path.length();
+    // Note: ShapePaths do not employ the QQuickPathPrivate caching (pathLength and _pointCache),
+    // but may utilize the QPainterPath caching in case of trimming
     pathLength = 0;
+    path.setCachingEnabled(true);
 
     return path;
 }
@@ -534,16 +617,20 @@ void QQuickPath::disconnectPathElements()
 {
     Q_D(const QQuickPath);
 
-    for (QQuickPathElement *pathElement : d->_pathElements)
-        disconnect(pathElement, SIGNAL(changed()), this, SLOT(processPath()));
+    for (QQuickPathElement *pathElement : d->_pathElements) {
+        if (pathElement)
+            disconnect(pathElement, SIGNAL(changed()), this, SLOT(processPath()));
+    }
 }
 
 void QQuickPath::connectPathElements()
 {
     Q_D(const QQuickPath);
 
-    for (QQuickPathElement *pathElement : d->_pathElements)
-        connect(pathElement, SIGNAL(changed()), this, SLOT(processPath()));
+    for (QQuickPathElement *pathElement : d->_pathElements) {
+        if (pathElement)
+            connect(pathElement, SIGNAL(changed()), this, SLOT(processPath()));
+    }
 }
 
 void QQuickPath::gatherAttributes()
@@ -552,6 +639,8 @@ void QQuickPath::gatherAttributes()
 
     QSet<QString> attributes;
 
+    Q_ASSERT(d->_pathCurves.isEmpty());
+
     // First gather up all the attributes
     for (QQuickPathElement *pathElement : std::as_const(d->_pathElements)) {
         if (QQuickCurve *curve = qobject_cast<QQuickCurve *>(pathElement))
@@ -570,9 +659,10 @@ void QQuickPath::componentComplete()
     Q_D(QQuickPath);
     d->componentComplete = true;
 
+    // These functions do what pathElements_append does, except for all elements at once.
     gatherAttributes();
 
-    processPath();
+    doProcessPath();
 
     connectPathElements();
 }
@@ -583,6 +673,16 @@ QPainterPath QQuickPath::path() const
     return d->_path;
 }
 
+void QQuickPath::setPath(const QPainterPath &path)
+{
+    Q_D(QQuickPath);
+    d->useCustomPath = !path.isEmpty();
+    d->_pointCache.clear();
+    d->prevBez.isValid = false;
+    d->_path = path;
+    emit changed();
+}
+
 QStringList QQuickPath::attributes() const
 {
     Q_D(const QQuickPath);
@@ -741,6 +841,31 @@ bool QQuickPath::simplify() const
     return d->simplify;
 }
 
+/*! \qmlproperty bool QtQuick::Path::asynchronous
+    \since 6.9
+
+    When set to true, the path will be processed asynchronously. This is an optimization
+    to process the path only once, after all the methods that possibly affect the path.
+    This means that when set to \c true, the updated path is not available immediately
+    after e.g. adjusting \l startX, \l scale or appending an element, only after the Qt
+    event loop has been processed. The default value is \c false.
+*/
+bool QQuickPath::isAsynchronous() const
+{
+    Q_D(const QQuickPath);
+    return d->asynchronous;
+}
+
+void QQuickPath::setAsynchronous(bool a)
+{
+    Q_D(QQuickPath);
+    if (d->asynchronous == a)
+        return;
+
+    d->asynchronous = a;
+    emit asynchronousChanged();
+}
+
 /*!
     \qmlproperty size QtQuick::Path::scale
 
@@ -916,8 +1041,8 @@ QPointF QQuickPath::backwardsPointAt(const QPainterPath &path, const qreal &path
 QPointF QQuickPath::pointAtPercent(qreal t) const
 {
     Q_D(const QQuickPath);
-    if (d->isShapePath)                     // this since ShapePath does not calculate the length at all,
-        return d->_path.pointAtPercent(t);  // in order to be faster.
+    if (d->isShapePath)
+        return d->_path.pointAtPercent(t); // ShapePath has QPainterPath computation caching
 
     if (d->_pointCache.isEmpty()) {
         createPointCache();
@@ -981,12 +1106,12 @@ qreal QQuickPath::attributeAt(const QString &name, qreal percent) const
 
 qreal QQuickCurve::x() const
 {
-    return _x.isNull ? 0 : _x.value;
+    return _x.isValid() ? _x.value() : 0;
 }
 
 void QQuickCurve::setX(qreal x)
 {
-    if (_x.isNull || _x != x) {
+    if (!_x.isValid() || _x != x) {
         _x = x;
         emit xChanged();
         emit changed();
@@ -1000,12 +1125,12 @@ bool QQuickCurve::hasX()
 
 qreal QQuickCurve::y() const
 {
-    return _y.isNull ? 0 : _y.value;
+    return _y.isValid() ? _y.value() : 0;
 }
 
 void QQuickCurve::setY(qreal y)
 {
-    if (_y.isNull || _y != y) {
+    if (!_y.isValid() || _y != y) {
         _y = y;
         emit yChanged();
         emit changed();
@@ -1024,7 +1149,7 @@ qreal QQuickCurve::relativeX() const
 
 void QQuickCurve::setRelativeX(qreal x)
 {
-    if (_relativeX.isNull || _relativeX != x) {
+    if (!_relativeX.isValid() || _relativeX != x) {
         _relativeX = x;
         emit relativeXChanged();
         emit changed();
@@ -1043,7 +1168,7 @@ qreal QQuickCurve::relativeY() const
 
 void QQuickCurve::setRelativeY(qreal y)
 {
-    if (_relativeY.isNull || _relativeY != y) {
+    if (!_relativeY.isValid() || _relativeY != y) {
         _relativeY = y;
         emit relativeYChanged();
         emit changed();
@@ -1055,11 +1180,25 @@ bool QQuickCurve::hasRelativeY()
     return _relativeY.isValid();
 }
 
+#ifndef QT_NO_DEBUG_STREAM
+QDebug operator<<(QDebug debug, const QQuickCurve *curve)
+{
+    QDebugStateSaver saver(debug);
+    debug.nospace() << curve->metaObject()->className() << '(' << (const void *)curve;
+    debug << " x=" << curve->x();
+    debug << " y=" << curve->y();
+    debug << " relativeX=" << curve->relativeX();
+    debug << " relativeY=" << curve->relativeY();
+    debug << ')';
+    return debug;
+}
+#endif
+
 /****************************************************************************/
 
 /*!
     \qmltype PathAttribute
-    \instantiates QQuickPathAttribute
+    \nativetype QQuickPathAttribute
     \inqmlmodule QtQuick
     \ingroup qtquick-animation-paths
     \brief Specifies how to set an attribute at a given position in a Path.
@@ -1176,7 +1315,7 @@ void QQuickPathAttribute::setValue(qreal value)
 
 /*!
     \qmltype PathLine
-    \instantiates QQuickPathLine
+    \nativetype QQuickPathLine
     \inqmlmodule QtQuick
     \ingroup qtquick-animation-paths
     \brief Defines a straight line.
@@ -1191,7 +1330,7 @@ void QQuickPathAttribute::setValue(qreal value)
     }
     \endqml
 
-    \sa Path, PathQuad, PathCubic, PathArc, PathAngleArc, PathCurve, PathSvg, PathMove, PathPolyline
+    \sa Path, PathQuad, PathCubic, PathArc, PathAngleArc, PathCurve, PathSvg, PathMove, PathPolyline, PathRectangle
 */
 
 /*!
@@ -1235,7 +1374,7 @@ void QQuickPathLine::addToPath(QPainterPath &path, const QQuickPathData &data)
 
 /*!
     \qmltype PathMove
-    \instantiates QQuickPathMove
+    \nativetype QQuickPathMove
     \inqmlmodule QtQuick
     \ingroup qtquick-animation-paths
     \brief Moves the Path's position.
@@ -1293,7 +1432,7 @@ void QQuickPathMove::addToPath(QPainterPath &path, const QQuickPathData &data)
 
 /*!
     \qmltype PathQuad
-    \instantiates QQuickPathQuad
+    \nativetype QQuickPathQuad
     \inqmlmodule QtQuick
     \ingroup qtquick-animation-paths
     \brief Defines a quadratic Bezier curve with a control point.
@@ -1402,7 +1541,7 @@ qreal QQuickPathQuad::relativeControlX() const
 
 void QQuickPathQuad::setRelativeControlX(qreal x)
 {
-    if (_relativeControlX.isNull || _relativeControlX != x) {
+    if (!_relativeControlX.isValid() || _relativeControlX != x) {
         _relativeControlX = x;
         emit relativeControlXChanged();
         emit changed();
@@ -1421,7 +1560,7 @@ qreal QQuickPathQuad::relativeControlY() const
 
 void QQuickPathQuad::setRelativeControlY(qreal y)
 {
-    if (_relativeControlY.isNull || _relativeControlY != y) {
+    if (!_relativeControlY.isValid() || _relativeControlY != y) {
         _relativeControlY = y;
         emit relativeControlYChanged();
         emit changed();
@@ -1445,7 +1584,7 @@ void QQuickPathQuad::addToPath(QPainterPath &path, const QQuickPathData &data)
 
 /*!
     \qmltype PathCubic
-    \instantiates QQuickPathCubic
+    \nativetype QQuickPathCubic
     \inqmlmodule QtQuick
     \ingroup qtquick-animation-paths
     \brief Defines a cubic Bezier curve with two control points.
@@ -1467,7 +1606,7 @@ void QQuickPathQuad::addToPath(QPainterPath &path, const QQuickPathData &data)
     \endqml
     \endtable
 
-    \sa Path, PathQuad, PathLine, PathArc, PathAngleArc, PathCurve, PathSvg
+    \sa Path, PathQuad, PathLine, PathArc, PathAngleArc, PathCurve, PathSvg, PathRectangle
 */
 
 /*!
@@ -1586,7 +1725,7 @@ qreal QQuickPathCubic::relativeControl1X() const
 
 void QQuickPathCubic::setRelativeControl1X(qreal x)
 {
-    if (_relativeControl1X.isNull || _relativeControl1X != x) {
+    if (!_relativeControl1X.isValid() || _relativeControl1X != x) {
         _relativeControl1X = x;
         emit relativeControl1XChanged();
         emit changed();
@@ -1605,7 +1744,7 @@ qreal QQuickPathCubic::relativeControl1Y() const
 
 void QQuickPathCubic::setRelativeControl1Y(qreal y)
 {
-    if (_relativeControl1Y.isNull || _relativeControl1Y != y) {
+    if (!_relativeControl1Y.isValid() || _relativeControl1Y != y) {
         _relativeControl1Y = y;
         emit relativeControl1YChanged();
         emit changed();
@@ -1624,7 +1763,7 @@ qreal QQuickPathCubic::relativeControl2X() const
 
 void QQuickPathCubic::setRelativeControl2X(qreal x)
 {
-    if (_relativeControl2X.isNull || _relativeControl2X != x) {
+    if (!_relativeControl2X.isValid() || _relativeControl2X != x) {
         _relativeControl2X = x;
         emit relativeControl2XChanged();
         emit changed();
@@ -1643,7 +1782,7 @@ qreal QQuickPathCubic::relativeControl2Y() const
 
 void QQuickPathCubic::setRelativeControl2Y(qreal y)
 {
-    if (_relativeControl2Y.isNull || _relativeControl2Y != y) {
+    if (!_relativeControl2Y.isValid() || _relativeControl2Y != y) {
         _relativeControl2Y = y;
         emit relativeControl2YChanged();
         emit changed();
@@ -1669,7 +1808,7 @@ void QQuickPathCubic::addToPath(QPainterPath &path, const QQuickPathData &data)
 
 /*!
     \qmltype PathCurve
-    \instantiates QQuickPathCatmullRomCurve
+    \nativetype QQuickPathCatmullRomCurve
     \inqmlmodule QtQuick
     \ingroup qtquick-animation-paths
     \brief Defines a point on a Catmull-Rom curve.
@@ -1815,7 +1954,7 @@ void QQuickPathCatmullRomCurve::addToPath(QPainterPath &path, const QQuickPathDa
 
 /*!
     \qmltype PathArc
-    \instantiates QQuickPathArc
+    \nativetype QQuickPathArc
     \inqmlmodule QtQuick
     \ingroup qtquick-animation-paths
     \brief Defines an arc with the given radius.
@@ -2026,7 +2165,7 @@ void QQuickPathArc::addToPath(QPainterPath &path, const QQuickPathData &data)
 
 /*!
     \qmltype PathAngleArc
-    \instantiates QQuickPathAngleArc
+    \nativetype QQuickPathAngleArc
     \inqmlmodule QtQuick
     \ingroup qtquick-animation-paths
     \brief Defines an arc with the given radii and center.
@@ -2035,7 +2174,7 @@ void QQuickPathArc::addToPath(QPainterPath &path, const QQuickPathData &data)
     to work as part of a larger path (specifying start and end), PathAngleArc is designed
     to make a path where the arc is primary (such as a circular progress indicator) more intuitive.
 
-    \sa Path, PathLine, PathQuad, PathCubic, PathCurve, PathSvg, PathArc
+    \sa Path, PathLine, PathQuad, PathCubic, PathCurve, PathSvg, PathArc, PathRectangle
 */
 
 /*!
@@ -2199,7 +2338,7 @@ void QQuickPathAngleArc::addToPath(QPainterPath &path, const QQuickPathData &)
 
 /*!
     \qmltype PathSvg
-    \instantiates QQuickPathSvg
+    \nativetype QQuickPathSvg
     \inqmlmodule QtQuick
     \ingroup qtquick-animation-paths
     \brief Defines a path using an SVG path data string.
@@ -2251,9 +2390,372 @@ void QQuickPathSvg::addToPath(QPainterPath &path, const QQuickPathData &)
 
 /****************************************************************************/
 
+/*!
+    \qmltype PathRectangle
+    \nativetype QQuickPathRectangle
+    \inqmlmodule QtQuick
+    \ingroup qtquick-animation-paths
+    \brief Defines a rectangle with optionally rounded corners.
+    \since QtQuick 6.8
+
+    PathRectangle provides an easy way to specify a rectangle, optionally with
+    rounded or beveled corners. The API corresponds to that of the \l Rectangle
+    item.
+
+    \image pathrectangle-bevel.png
+
+    \snippet qml/pathrectangle/pathrectangle-bevel.qml shape
+
+    \sa Path, PathLine, PathQuad, PathCubic, PathArc, PathAngleArc, PathCurve, PathSvg
+*/
+
+/*!
+    \qmlproperty real QtQuick::PathRectangle::x
+    \qmlproperty real QtQuick::PathRectangle::y
+
+    Defines the top left corner of the rectangle.
+
+    Unless that corner is rounded, this will also be the start and end point of the path.
+
+    \sa relativeX, relativeY
+*/
+
+/*!
+    \qmlproperty real QtQuick::PathRectangle::relativeX
+    \qmlproperty real QtQuick::PathRectangle::relativeY
+
+    Defines the top left corner of the rectangle relative to the path's start point.
+
+    If both a relative and absolute end position are specified for a single axis, the relative
+    position will be used.
+
+    Relative and absolute positions can be mixed, for example it is valid to set a relative x
+    and an absolute y.
+
+    \sa x, y
+*/
+
+/*!
+    \qmlproperty real QtQuick::PathRectangle::width
+    \qmlproperty real QtQuick::PathRectangle::height
+
+    Defines the width and height of the rectangle.
+
+    \sa x, y
+*/
+
+qreal QQuickPathRectangle::width() const
+{
+    return _width;
+}
+
+void QQuickPathRectangle::setWidth(qreal width)
+{
+    if (_width == width)
+        return;
+
+    _width = width;
+    emit widthChanged();
+    emit changed();
+}
+
+qreal QQuickPathRectangle::height() const
+{
+    return _height;
+}
+
+void QQuickPathRectangle::setHeight(qreal height)
+{
+    if (_height == height)
+        return;
+
+    _height = height;
+    emit heightChanged();
+    emit changed();
+}
+
+/*!
+    \qmlproperty real QtQuick::PathRectangle::strokeAdjustment
+
+    This property defines the stroke width adjustment to the rectangle coordinates.
+
+    When used in a \l ShapePath with stroking enabled, the actual stroked rectangle will by default
+    extend beyond the defined rectangle by half the stroke width on all sides. This is the expected
+    behavior since the path defines the midpoint line of the stroking, and corresponds to QPainter
+    and SVG rendering.
+
+    If one instead wants the defined rectangle to be the outer edge of the stroked rectangle, like
+    a \l Rectangle item with a border, one can set strokeAdjustment to the stroke width. This will
+    effectively shift all edges inwards by half the stroke width. Like in the following example:
+
+    \qml
+    ShapePath {
+        id: myRec
+        fillColor: "white"
+        strokeColor: "black"
+        strokeWidth: 16
+        joinStyle: ShapePath.MiterJoin
+
+        PathRectangle { x: 10; y: 10; width: 200; height: 100; strokeAdjustment: myRec.strokeWidth }
+    }
+    \endqml
+*/
+
+qreal QQuickPathRectangle::strokeAdjustment() const
+{
+    return _strokeAdjustment;
+}
+
+void QQuickPathRectangle::setStrokeAdjustment(qreal newStrokeAdjustment)
+{
+    if (_strokeAdjustment == newStrokeAdjustment)
+        return;
+    _strokeAdjustment = newStrokeAdjustment;
+    emit strokeAdjustmentChanged();
+    emit changed();
+}
+
+/*!
+    \include pathrectangle.qdocinc {radius-property} {QtQuick::PathRectangle}
+
+    The default value is \c 0.
+*/
+
+qreal QQuickPathRectangle::radius() const
+{
+    return _extra.isAllocated() ? _extra->radius : 0;
+}
+
+void QQuickPathRectangle::setRadius(qreal newRadius)
+{
+    if (_extra.value().radius == newRadius)
+        return;
+    _extra->radius = newRadius;
+    emit radiusChanged();
+    if (!(_extra->isRadiusSet(Qt::TopLeftCorner)))
+        emit topLeftRadiusChanged();
+    if (!(_extra->isRadiusSet(Qt::TopRightCorner)))
+        emit topRightRadiusChanged();
+    if (!(_extra->isRadiusSet(Qt::BottomLeftCorner)))
+        emit bottomLeftRadiusChanged();
+    if (!(_extra->isRadiusSet(Qt::BottomRightCorner)))
+        emit bottomRightRadiusChanged();
+    emit changed();
+}
+
+/*!
+    \include pathrectangle.qdocinc {radius-properties} {PathRectangle} {qml/pathrectangle/pathrectangle.qml} {shape}
+*/
+
+qreal QQuickPathRectangle::cornerRadius(Qt::Corner corner) const
+{
+    if (_extra.isAllocated())
+        return (_extra->isRadiusSet(corner)) ? _extra->cornerRadii[corner] : _extra->radius;
+    else
+        return 0;
+}
+
+void QQuickPathRectangle::setCornerRadius(Qt::Corner corner, qreal newCornerRadius)
+{
+    if (_extra.value().cornerRadii[corner] == newCornerRadius
+        && (_extra->isRadiusSet(corner)))
+        return;
+    _extra->cornerRadii[corner] = newCornerRadius;
+    _extra->cornerProperties |= (1 << corner);
+
+    emitCornerRadiusChanged(corner);
+}
+
+void QQuickPathRectangle::resetCornerRadius(Qt::Corner corner)
+{
+    if (!_extra.isAllocated() || !(_extra->isRadiusSet(corner)))
+        return;
+    _extra->cornerProperties &= ~(1 << corner);
+    emitCornerRadiusChanged(corner);
+}
+
+void QQuickPathRectangle::emitCornerRadiusChanged(Qt::Corner corner)
+{
+    switch (corner) {
+    case Qt::TopLeftCorner:
+        emit topLeftRadiusChanged();
+        break;
+    case Qt::TopRightCorner:
+        emit topRightRadiusChanged();
+        break;
+    case Qt::BottomLeftCorner:
+        emit bottomLeftRadiusChanged();
+        break;
+    case Qt::BottomRightCorner:
+        emit bottomRightRadiusChanged();
+        break;
+    }
+    emit changed();
+}
+
+/*!
+    \include pathrectangle.qdocinc {bevel-property}
+        {QtQuick::PathRectangle}{qml/pathrectangle/pathrectangle-bevel.qml}
+        {shape}
+    \since 6.10
+*/
+
+bool QQuickPathRectangle::hasBevel() const
+{
+    return _extra.isAllocated() ? (_extra->cornerProperties & (1 << 8)) != 0 : false;
+}
+
+void QQuickPathRectangle::setBevel(bool bevel)
+{
+    if (((_extra.value().cornerProperties & (1 << 8)) != 0) == bevel)
+        return;
+    if (bevel)
+        _extra->cornerProperties |= (1 << 8);
+    else
+        _extra->cornerProperties &= ~(1 << 8);
+
+    emit bevelChanged();
+    if (!(_extra->isBevelSet(Qt::TopLeftCorner)))
+        emit topLeftBevelChanged();
+    if (!(_extra->isBevelSet(Qt::TopRightCorner)))
+        emit topRightBevelChanged();
+    if (!(_extra->isBevelSet(Qt::BottomLeftCorner)))
+        emit bottomLeftBevelChanged();
+    if (!(_extra->isBevelSet(Qt::BottomRightCorner)))
+        emit bottomRightBevelChanged();
+    emit changed();
+}
+/*!
+    \include pathrectangle.qdocinc {bevel-properties}
+        {QtQuick::PathRectangle} {qml/pathrectangle/pathrectangle.qml} {shape}
+*/
+
+bool QQuickPathRectangle::cornerBevel(Qt::Corner corner) const
+{
+    if (_extra.isAllocated())
+        return _extra->isBevelSet(corner);
+    else
+        return false;
+}
+
+void QQuickPathRectangle::setCornerBevel(Qt::Corner corner, bool newCornerBevel)
+{
+    if ((_extra.value().isBevelSet(corner)) == newCornerBevel)
+        return;
+    if (!newCornerBevel) {
+        resetCornerBevel(corner);
+        return;
+    }
+    _extra->cornerProperties |= (1 << (corner + 4));
+    emitCornerBevelChanged(corner);
+}
+
+void QQuickPathRectangle::resetCornerBevel(Qt::Corner corner)
+{
+    if (!_extra.isAllocated() || !(_extra->isBevelSet(corner)))
+        return;
+    _extra->cornerProperties &= ~(1 << (corner + 4));
+    emitCornerBevelChanged(corner);
+}
+
+void QQuickPathRectangle::emitCornerBevelChanged(Qt::Corner corner)
+{
+    switch (corner) {
+    case Qt::TopLeftCorner:
+        emit topLeftBevelChanged();
+        break;
+    case Qt::TopRightCorner:
+        emit topRightBevelChanged();
+        break;
+    case Qt::BottomLeftCorner:
+        emit bottomLeftBevelChanged();
+        break;
+    case Qt::BottomRightCorner:
+        emit bottomRightBevelChanged();
+        break;
+    }
+    emit changed();
+}
+
+void QQuickPathRectangle::addToPath(QPainterPath &path, const QQuickPathData &data)
+{
+    QRectF rect(positionForCurve(data, path.currentPosition()), QSizeF(_width, _height));
+
+    qreal halfStroke = _strokeAdjustment * 0.5;
+    rect.adjust(halfStroke, halfStroke, -halfStroke, -halfStroke);
+    if (rect.isEmpty())
+        return;
+
+    if (!_extra.isAllocated()) {
+        // No rounded corners
+        path.addRect(rect);
+    } else {
+        // Radii must not exceed half of the width or half of the height
+        const qreal maxDiameter = qMin(rect.width(), rect.height());
+        const qreal generalDiameter = qMax(qreal(0), qMin(maxDiameter, 2 * _extra->radius));
+        auto effectiveDiameter = [&](Qt::Corner corner) {
+            qreal radius = _extra->cornerRadii[corner];
+            return (_extra->isRadiusSet(corner)) ? qMin(maxDiameter, 2 * radius) : generalDiameter;
+        };
+        const qreal diamTL = effectiveDiameter(Qt::TopLeftCorner);
+        const qreal diamTR = effectiveDiameter(Qt::TopRightCorner);
+        const qreal diamBL = effectiveDiameter(Qt::BottomLeftCorner);
+        const qreal diamBR = effectiveDiameter(Qt::BottomRightCorner);
+
+        path.moveTo(rect.left() + diamTL * 0.5, rect.top());
+        if (diamTR) {
+            if (!cornerBevel(Qt::TopRightCorner)) {
+                // Rounded corners.
+                path.arcTo(QRectF(QPointF(rect.right() - diamTR, rect.top()), QSizeF(diamTR, diamTR)), 90, -90);
+            } else {
+                // Beveled corners.
+                path.lineTo(QPointF(rect.right() - diamTR * 0.5, rect.top()));
+                path.lineTo(QPointF(rect.right(), rect.top() + diamTR * 0.5));
+            }
+        } else {
+            // Regular corners.
+            path.lineTo(rect.topRight());
+        }
+
+        if (diamBR) {
+            if (!cornerBevel(Qt::BottomRightCorner)) {
+                path.arcTo(QRectF(QPointF(rect.right() - diamBR, rect.bottom() - diamBR), QSizeF(diamBR, diamBR)), 0, -90);
+            } else {
+                path.lineTo(QPointF(rect.right(), rect.bottom() - diamBR * 0.5));
+                path.lineTo(QPointF(rect.right() - diamBR * 0.5, rect.bottom()));
+            }
+        } else {
+            path.lineTo(rect.bottomRight());
+        }
+
+        if (diamBL) {
+            if (!cornerBevel(Qt::BottomLeftCorner)) {
+                path.arcTo(QRectF(QPointF(rect.left(), rect.bottom() - diamBL), QSizeF(diamBL, diamBL)), 270, -90);
+            } else {
+                path.lineTo(QPointF(rect.left() + diamBL * 0.5, rect.bottom()));
+                path.lineTo(QPointF(rect.left(), rect.bottom() - diamBL * 0.5));
+            }
+        } else {
+            path.lineTo(rect.bottomLeft());
+        }
+
+        if (diamTL) {
+            if (!cornerBevel(Qt::TopLeftCorner))
+                path.arcTo(QRectF(rect.topLeft(), QSizeF(diamTL, diamTL)), 180, -90);
+            else
+                path.lineTo(QPointF(rect.left(), rect.top() + diamTL * 0.5));
+        } else {
+            path.lineTo(rect.topLeft());
+        }
+        path.closeSubpath();
+    }
+}
+
+/****************************************************************************/
+
 /*!
     \qmltype PathPercent
-    \instantiates QQuickPathPercent
+    \nativetype QQuickPathPercent
     \inqmlmodule QtQuick
     \ingroup qtquick-animation-paths
     \brief Manipulates the way a path is interpreted.
@@ -2348,7 +2850,7 @@ void QQuickPathPercent::setValue(qreal value)
 
 /*!
     \qmltype PathPolyline
-    \instantiates QQuickPathPolyline
+    \nativetype QQuickPathPolyline
     \inqmlmodule QtQuick
     \ingroup qtquick-animation-paths
     \brief Defines a polyline through a list of coordinates.
@@ -2370,7 +2872,7 @@ void QQuickPathPercent::setValue(qreal value)
     }
     \endqml
 
-    \sa Path, PathQuad, PathCubic, PathArc, PathAngleArc, PathCurve, PathSvg, PathMove, PathPolyline
+    \sa Path, PathQuad, PathCubic, PathArc, PathAngleArc, PathCurve, PathSvg, PathMove
 */
 
 /*!
@@ -2456,7 +2958,7 @@ void QQuickPathPolyline::addToPath(QPainterPath &path, const QQuickPathData &/*d
 
 /*!
     \qmltype PathMultiline
-    \instantiates QQuickPathMultiline
+    \nativetype QQuickPathMultiline
     \inqmlmodule QtQuick
     \ingroup qtquick-animation-paths
     \brief Defines a set of polylines through a list of lists of coordinates.
@@ -2607,7 +3109,7 @@ void QQuickPathMultiline::addToPath(QPainterPath &path, const QQuickPathData &)
 
 /*!
     \qmltype PathText
-    \instantiates QQuickPathText
+    \nativetype QQuickPathText
     \inqmlmodule QtQuick
     \ingroup qtquick-animation-paths
     \brief Defines a string in a specified font.
@@ -2617,6 +3119,13 @@ void QQuickPathMultiline::addToPath(QPainterPath &path, const QQuickPathData &)
     baseline will be translated to the x and y coordinates, and the outlines from the font
     will be added to the path accordingly.
 
+    When used to render texts in a Shape item, note the following:
+    \list
+        \li For correct fill, the ShapePath's fillRule should be set to ShapePath.WindingFill.
+        \li Not all fonts provide a nice outline suitable for stroking. If you want a stroked
+            outline and are getting unsatisfactory results, try a different font.
+    \endlist
+
     \qml
     PathText {
         x: 0
@@ -2796,12 +3305,33 @@ void QQuickPathMultiline::addToPath(QPainterPath &path, const QQuickPathData &)
     \endqml
 */
 
+/*!
+    \qmlproperty object QtQuick::PathText::font.variableAxes
+    \since 6.7
+
+    \include qquicktext.cpp qml-font-variable-axes
+*/
+
 /*!
     \qmlproperty object QtQuick::PathText::font.features
     \since 6.6
 
     \include qquicktext.cpp qml-font-features
 */
+
+/*!
+    \qmlproperty bool QtQuick::PathText::font.contextFontMerging
+    \since 6.8
+
+    \include qquicktext.cpp qml-font-context-font-merging
+*/
+
+/*!
+    \qmlproperty bool QtQuick::PathText::font.preferTypoLineMetrics
+    \since 6.8
+
+    \include qquicktext.cpp qml-font-prefer-typo-line-metrics
+*/
 void QQuickPathText::updatePath() const
 {
     if (!_path.isEmpty())
