@@ -11,7 +11,7 @@
 
 QT_BEGIN_NAMESPACE
 
-Q_LOGGING_CATEGORY(lcAttachedProperty, "qt.quick.dialogs.quickfontdialogimpl.attachedOrWarn")
+Q_STATIC_LOGGING_CATEGORY(lcAttachedProperty, "qt.quick.dialogs.quickfontdialogimpl.attachedOrWarn")
 
 QQuickFontDialogImplPrivate::QQuickFontDialogImplPrivate()
 {
@@ -43,7 +43,6 @@ void QQuickFontDialogImplPrivate::handleClick(QQuickAbstractButton *button)
 QQuickFontDialogImpl::QQuickFontDialogImpl(QObject *parent)
     : QQuickDialog(*(new QQuickFontDialogImplPrivate), parent)
 {
-    setPopupType(QQuickPopup::Window);
 }
 
 QQuickFontDialogImplAttached *QQuickFontDialogImpl::qmlAttachedProperties(QObject *object)
