@@ -15,6 +15,9 @@
 #if QT_CONFIG(shortcut)
 #  include <QtGui/private/qshortcutmap_p.h>
 #endif
+#if QT_CONFIG(accessibility)
+#include <QtGui/private/qaccessiblehelper_p.h>
+#endif
 #include <QtGui/private/qguiapplication_p.h>
 #include <QtGui/qpa/qplatformtheme.h>
 #include <QtQuick/private/qquickevents_p_p.h>
@@ -370,6 +373,11 @@ void QQuickAbstractButtonPrivate::click()
         emit q->clicked();
 }
 
+void QQuickAbstractButtonPrivate::accessiblePressAction()
+{
+    trigger();
+}
+
 void QQuickAbstractButtonPrivate::trigger(bool doubleClick)
 {
     Q_Q(QQuickAbstractButton);
@@ -1286,7 +1294,9 @@ void QQuickAbstractButton::buttonChange(ButtonChange change)
         break;
     case ButtonTextChange: {
         const QString txt = text();
-        maybeSetAccessibleName(txt);
+#if QT_CONFIG(accessibility)
+        maybeSetAccessibleName(qt_accStripAmp(txt));
+#endif
 #if QT_CONFIG(shortcut)
         setShortcut(QKeySequence::mnemonic(txt));
 #endif
@@ -1325,7 +1335,7 @@ void QQuickAbstractButton::accessibilityActiveChanged(bool active)
 
     Q_D(QQuickAbstractButton);
     if (active) {
-        maybeSetAccessibleName(text());
+        maybeSetAccessibleName(qt_accStripAmp(text()));
         setAccessibleProperty("pressed", d->pressed);
         setAccessibleProperty("checked", d->checked);
         setAccessibleProperty("checkable", d->checkable);
@@ -1344,7 +1354,7 @@ QAccessible::Role QQuickAbstractButton::accessibleRole() const
 void QQuickAbstractButton::accessiblePressAction()
 {
     Q_D(QQuickAbstractButton);
-    d->trigger();
+    d->accessiblePressAction();
 }
 #endif
 
