@@ -34,6 +34,7 @@
 #include <private/qqmlobjectmodel_p.h>
 #include <QtQuick/private/qquickitem_p.h>
 #include <QtQuick/private/qquickitemchangelistener_p.h>
+#include <QtQuick/private/qquickitemview_p_p.h>
 #include <QtQuick/private/qquickevents_p_p.h>
 #include <QtQuick/private/qquicklistview_p.h>
 #include <QtQuick/private/qquickrendercontrol_p.h>
@@ -76,84 +77,23 @@ static const int SUBMENU_DELAY = 225;
     the mouse cursor on desktop platforms that have a mouse cursor available, and
     otherwise centered over its parent item.
 
-    \code
-    MouseArea {
-        anchors.fill: parent
-        acceptedButtons: Qt.LeftButton | Qt.RightButton
-        onClicked: (mouse) => {
-            if (mouse.button === Qt.RightButton)
-                contextMenu.popup()
-        }
-        onPressAndHold: (mouse) => {
-            if (mouse.source === Qt.MouseEventNotSynthesized)
-                contextMenu.popup()
-        }
-
-        Menu {
-            id: contextMenu
-            MenuItem { text: "Cut" }
-            MenuItem { text: "Copy" }
-            MenuItem { text: "Paste" }
-        }
-    }
-    \endcode
+    \snippet qtquickcontrols-menu-contextmenu.qml root
 
     When used as a popup menu, it is easiest to specify the position by specifying
     the desired \l {Popup::}{x} and \l {Popup::}{y} coordinates using the respective
     properties, and call \l {Popup::}{open()} to open the menu.
 
-    \code
-    Button {
-        id: fileButton
-        text: "File"
-        onClicked: menu.open()
-
-        Menu {
-            id: menu
-            y: fileButton.height
-
-            MenuItem {
-                text: "New..."
-            }
-            MenuItem {
-                text: "Open..."
-            }
-            MenuItem {
-                text: "Save"
-            }
-        }
-    }
-    \endcode
+    \snippet qtquickcontrols-menu-button-menu.qml root
 
     If the button should also close the menu when clicked, use the
     \c Popup.CloseOnPressOutsideParent flag:
-    \code
-    onClicked: menu.visible = !menu.visible
 
-    Menu {
-        // ...
-        closePolicy: Popup.CloseOnEscape | Popup.CloseOnPressOutsideParent
-    \endcode
+    \snippet qtquickcontrols-menu-closepolicy.qml closePolicy
 
     Since QtQuick.Controls 2.3 (Qt 5.10), it is also possible to create sub-menus
     and declare Action objects inside Menu:
 
-    \code
-    Menu {
-        Action { text: "Cut" }
-        Action { text: "Copy" }
-        Action { text: "Paste" }
-
-        MenuSeparator { }
-
-        Menu {
-            title: "Find/Replace"
-            Action { text: "Find Next" }
-            Action { text: "Find Previous" }
-            Action { text: "Replace" }
-        }
-    }
-    \endcode
+    \snippet qtquickcontrols-menu-submenus-and-actions.qml root
 
     Sub-menus are \l {cascade}{cascading} by default on desktop platforms
     that have a mouse cursor available. Non-cascading menus are shown one
@@ -524,9 +464,16 @@ void QQuickMenuPrivate::syncWithNativeMenu()
     qCDebug(lcNativeMenus) << "... finished syncing" << q;
 }
 
+/*!
+    \internal
+
+    Removes the native menu, including its native menu items.
+
+    \note this doesn't remove any QQuickMenuItems from the contentModel;
+    it merely removes the associated native menu items.
+*/
 void QQuickMenuPrivate::removeNativeMenu()
 {
-    // Remove the native menu, including it's native menu items
     Q_Q(QQuickMenu);
     const int qtyItemsToRemove = nativeItems.size();
     if (qtyItemsToRemove != 0)
@@ -559,44 +506,6 @@ void QQuickMenuPrivate::syncWithUseNativeMenu()
     }
 }
 
-/*!
-    \internal
-
-    Recursively destroys native sub-menus of \a menu.
-
-    This function checks if each native item in \c menu has a sub-menu,
-    and if so:
-    \list
-    \li Calls itself with that sub-menu
-    \li Resets the item's data (important to avoid accessing a deleted QQuickAction
-        when printing in QQuickNativeMenuItem's destructor)
-    \li Deletes (eventually) the native item
-    \endlist
-
-    Similar (besides the recursion) to removeNativeItem(), except that
-    we can avoid repeated calls to syncWithNativeMenu().
-*/
-void QQuickMenuPrivate::recursivelyDestroyNativeSubMenus(QQuickMenu *menu)
-{
-    auto *menuPrivate = QQuickMenuPrivate::get(menu);
-    Q_ASSERT(menuPrivate->handle);
-    qCDebug(lcNativeMenus) << "recursivelyDestroyNativeSubMenus called with" << menu << "...";
-
-    while (!menuPrivate->nativeItems.isEmpty()) {
-        std::unique_ptr<QQuickNativeMenuItem> item(menuPrivate->nativeItems.takeFirst());
-        qCDebug(lcNativeMenus) << "- taking and destroying" << item->debugText();
-        if (QQuickMenu *subMenu = item->subMenu())
-            recursivelyDestroyNativeSubMenus(subMenu);
-
-        if (item->handle())
-            menuPrivate->handle->removeMenuItem(item->handle());
-    }
-
-    menuPrivate->resetNativeData();
-
-    qCDebug(lcNativeMenus) << "... finished destroying native sub-menus of" << menu;
-}
-
 static QWindow *effectiveWindow(QWindow *window, QPoint *offset)
 {
     QQuickWindow *quickWindow = qobject_cast<QQuickWindow *>(window);
@@ -657,12 +566,6 @@ void QQuickMenuPrivate::setNativeMenuVisible(bool visible)
     }
 }
 
-// Used by QQuickContextMenu when it's opened on a text-editing control.
-void QQuickMenuPrivate::makeEditMenu()
-{
-    handle->setMenuType(QPlatformMenu::EditMenu);
-}
-
 QQuickItem *QQuickMenuPrivate::itemAt(int index) const
 {
     return qobject_cast<QQuickItem *>(contentModel->get(index));
@@ -794,7 +697,17 @@ void QQuickMenuPrivate::removeItem(int index, QQuickItem *item, DestructionPolic
         printContentModelItems();
 }
 
-void QQuickMenuPrivate::removeNativeItem(int index)
+/*!
+    \internal
+
+    Removes the native menu item at \a index from this menu.
+
+    \note this doesn't remove the QQuickMenuItem from the contentModel;
+    it merely removes the associated native menu item. It's for this reason
+    that this is a separate function to removeItem, which \e does remove
+    the QQuickMenuItem.
+*/
+void QQuickMenuPrivate::removeNativeItem(int index, SyncPolicy syncPolicy)
 {
     // Either we're still using native menus and are removing item(s), or we've switched
     // to a non-native menu; either way, we should actually have items to remove before we're called.
@@ -807,12 +720,28 @@ void QQuickMenuPrivate::removeNativeItem(int index)
     std::unique_ptr<QQuickNativeMenuItem> nativeItem(nativeItems.takeAt(index));
     qCDebug(lcNativeMenus) << "removing native item" << nativeItem->debugText() << "at index" << index
         << "from" << q_func() << "...";
-    if (QQuickMenu *subMenu = nativeItem->subMenu())
-        recursivelyDestroyNativeSubMenus(subMenu);
+    QQuickMenu *subMenu = nativeItem->subMenu();
+    if (subMenu) {
+        Q_ASSERT(nativeItem->handle());
+        auto *subMenuPrivate = QQuickMenuPrivate::get(subMenu);
+        while (!subMenuPrivate->nativeItems.isEmpty()) {
+            subMenuPrivate->removeNativeItem(0, SyncPolicy::DoNotSync);
+        }
+    }
 
-    if (nativeItem->handle()) {
-        handle->removeMenuItem(nativeItem->handle());
+    Q_ASSERT(nativeItem->handle());
+    handle->removeMenuItem(nativeItem->handle());
+    if (syncPolicy == SyncPolicy::Sync)
         syncWithNativeMenu();
+
+    if (subMenu) {
+        auto *subMenuPrivate = QQuickMenuPrivate::get(subMenu);
+        // Reset the item's data. This is important as it avoids accessing a deleted
+        // QQuickAction when printing in QQuickNativeMenuItem's destructor.
+        // It's also important that we do this _after_ the removeMenuItem call above,
+        // because otherwise we sever the connection between the sub and parent menu,
+        // which causes warnings in QCocoaMenu::removeMenuItem.
+        subMenuPrivate->resetNativeData();
     }
 
     qCDebug(lcNativeMenus).nospace() << "... after removing item at index " << index
@@ -955,10 +884,16 @@ void QQuickMenuPrivate::itemSiblingOrderChanged(QQuickItem *)
 
 void QQuickMenuPrivate::itemDestroyed(QQuickItem *item)
 {
-    QQuickPopupPrivate::itemDestroyed(item);
-    int index = contentModel->indexOf(item, nullptr);
-    if (index != -1)
-        removeItem(index, item);
+    if (item == contentItem) {
+        resetContentItem();
+    } else {
+        QQuickPopupPrivate::itemDestroyed(item);
+        if (contentModel) {
+            int index = contentModel->indexOf(item, nullptr);
+            if (index != -1)
+                removeItem(index, item);
+        }
+    }
 }
 
 void QQuickMenuPrivate::itemGeometryChanged(QQuickItem *item, QQuickGeometryChange, const QRectF &)
@@ -1023,6 +958,22 @@ bool QQuickMenuPrivate::prepareEnterTransition()
     // the right, it flips on the other side of the parent menu.
     allowHorizontalFlip = cascade && parentMenu;
 
+    // Enter transitions may want to animate the Menu's height based on its implicitHeight.
+    // The Menu's implicitHeight is typically based on the ListView's contentHeight,
+    // among other things. The docs for ListView's forceLayout function say:
+    // "Responding to changes in the model is usually batched to happen only once per frame."
+    // As e.g. NumberAnimation's from and to values are set before any polishes happen,
+    // any re-evaluation of their bindings happen too late, and the starting height can be
+    // out-dated when menu items are added after component completion
+    // (QQuickItemView::componentComplete does a layout, so items declared as children aren't
+    // affected by this). To account for this, we force a layout before the transition starts.
+    // We try to avoid unnecessary re-layouting if we can avoid it.
+    auto *contentItemAsListView = qobject_cast<QQuickListView *>(contentItem);
+    if (contentItemAsListView) {
+        if (QQuickItemViewPrivate::get(contentItemAsListView)->currentChanges.hasPendingChanges())
+            contentItemAsListView->forceLayout();
+    }
+
     if (!QQuickPopupPrivate::prepareEnterTransition())
         return false;
 
@@ -1375,6 +1326,20 @@ void QQuickMenuPrivate::contentData_clear(QQmlListProperty<QObject> *prop)
     QQuickMenuPrivate::get(q)->contentData.clear();
 }
 
+void QQuickMenuPrivate::resetContentItem()
+{
+    if (contentItem) {
+        QQuickItemPrivate::get(contentItem)->removeItemChangeListener(this, QQuickItemPrivate::Children);
+        QQuickItemPrivate::get(contentItem)->removeItemChangeListener(this, QQuickItemPrivate::Destroyed);
+        QQuickItemPrivate::get(contentItem)->removeItemChangeListener(this, QQuickItemPrivate::Geometry);
+
+        const auto children = contentItem->childItems();
+        for (QQuickItem *child : std::as_const(children))
+            QQuickItemPrivate::get(child)->removeItemChangeListener(this, QQuickItemPrivate::SiblingOrder);
+        contentItem = nullptr;
+    }
+}
+
 QQuickMenu::QQuickMenu(QObject *parent)
     : QQuickPopup(*(new QQuickMenuPrivate), parent)
 {
@@ -1391,6 +1356,15 @@ QQuickMenu::~QQuickMenu()
                           << "item count:"
                           << d->contentModel->count()
                           << "native item count:" << d->nativeItems.count();
+    // It would be better to reset the sub-menu within the menu-item during its destruction
+    // as there can be a chance that the parent menu use invalid reference leading to
+    // application crash (as mentioned in the bug report QTBUG-137160)
+    if (auto *menuItem = qobject_cast<QQuickMenuItem *>(d->parentItem)) {
+        if (menuItem->subMenu() == this) {
+            auto *menuItemPriv = QQuickMenuItemPrivate::get(menuItem);
+            menuItemPriv->setSubMenu(nullptr);
+        }
+    }
     // We have to remove items to ensure that our change listeners on the item
     // are removed. It's too late to do this in ~QQuickMenuPrivate, as
     // contentModel has already been destroyed before that is called.
@@ -1399,14 +1373,7 @@ QQuickMenu::~QQuickMenu()
     while (d->contentModel->count() > 0)
         d->removeItem(0, d->itemAt(0), QQuickMenuPrivate::DestructionPolicy::Destroy);
 
-    if (d->contentItem) {
-        QQuickItemPrivate::get(d->contentItem)->removeItemChangeListener(d, QQuickItemPrivate::Children);
-        QQuickItemPrivate::get(d->contentItem)->removeItemChangeListener(d, QQuickItemPrivate::Geometry);
-
-        const auto children = d->contentItem->childItems();
-        for (QQuickItem *child : std::as_const(children))
-            QQuickItemPrivate::get(child)->removeItemChangeListener(d, QQuickItemPrivate::SiblingOrder);
-    }
+    d->resetContentItem();
 }
 
 /*!
@@ -1423,7 +1390,10 @@ QQuickItem *QQuickMenu::itemAt(int index) const
 /*!
     \qmlmethod void QtQuick.Controls::Menu::addItem(Item item)
 
-    Adds \a item to the end of the list of items.
+    Adds \a item to the end of the list of items. The menu does not take
+    ownership of the newly added \a item.
+
+    \sa {Dynamically Generating Menu Items}
 */
 void QQuickMenu::addItem(QQuickItem *item)
 {
@@ -1434,7 +1404,10 @@ void QQuickMenu::addItem(QQuickItem *item)
 /*!
     \qmlmethod void QtQuick.Controls::Menu::insertItem(int index, Item item)
 
-    Inserts \a item at \a index.
+    Inserts \a item at \a index. The menu does not take ownership of the newly
+    inserted \a item.
+
+    \sa {Dynamically Generating Menu Items}
 */
 void QQuickMenu::insertItem(int index, QQuickItem *item)
 {
@@ -1536,7 +1509,8 @@ QQuickMenu *QQuickMenu::menuAt(int index) const
     \since QtQuick.Controls 2.3 (Qt 5.10)
     \qmlmethod void QtQuick.Controls::Menu::addMenu(Menu menu)
 
-    Adds \a menu as a sub-menu to the end of this menu.
+    Adds \a menu as a sub-menu to the end of this menu. The menu does not take
+    ownership of the newly added \a menu.
 */
 void QQuickMenu::addMenu(QQuickMenu *menu)
 {
@@ -1548,7 +1522,8 @@ void QQuickMenu::addMenu(QQuickMenu *menu)
     \since QtQuick.Controls 2.3 (Qt 5.10)
     \qmlmethod void QtQuick.Controls::Menu::insertMenu(int index, Menu menu)
 
-    Inserts \a menu as a sub-menu at \a index. The index is within all items in the menu.
+    Inserts \a menu as a sub-menu at \a index. The index is within all items in
+    the menu. The menu does not take ownership of the newly inserted \a menu.
 */
 void QQuickMenu::insertMenu(int index, QQuickMenu *menu)
 {
@@ -1588,7 +1563,8 @@ void QQuickMenu::removeMenu(QQuickMenu *menu)
     \since QtQuick.Controls 2.3 (Qt 5.10)
     \qmlmethod Menu QtQuick.Controls::Menu::takeMenu(int index)
 
-    Removes and returns the menu at \a index. The index is within all items in the menu.
+    Removes and returns the menu at \a index. The index is within all items in
+    the menu.
 
     \note The ownership of the menu is transferred to the caller.
 */
@@ -1637,7 +1613,8 @@ QQuickAction *QQuickMenu::actionAt(int index) const
     \since QtQuick.Controls 2.3 (Qt 5.10)
     \qmlmethod void QtQuick.Controls::Menu::addAction(Action action)
 
-    Adds \a action to the end of this menu.
+    Adds \a action to the end of this menu. The menu does not take ownership of
+    the newly added \a action.
 */
 void QQuickMenu::addAction(QQuickAction *action)
 {
@@ -1650,6 +1627,7 @@ void QQuickMenu::addAction(QQuickAction *action)
     \qmlmethod void QtQuick.Controls::Menu::insertAction(int index, Action action)
 
     Inserts \a action at \a index. The index is within all items in the menu.
+    The menu does not take ownership of the newly inserted \a action.
 */
 void QQuickMenu::insertAction(int index, QQuickAction *action)
 {
@@ -1689,7 +1667,8 @@ void QQuickMenu::removeAction(QQuickAction *action)
     \since QtQuick.Controls 2.3 (Qt 5.10)
     \qmlmethod Action QtQuick.Controls::Menu::takeAction(int index)
 
-    Removes and returns the action at \a index. The index is within all items in the menu.
+    Removes and returns the action at \a index. The index is within all items in
+    the menu.
 
     \note The ownership of the action is transferred to the caller.
 */
@@ -2176,10 +2155,12 @@ void QQuickMenu::contentItemChange(QQuickItem *newItem, QQuickItem *oldItem)
 
     if (oldItem) {
         QQuickItemPrivate::get(oldItem)->removeItemChangeListener(d, QQuickItemPrivate::Children);
+        QQuickItemPrivate::get(newItem)->removeItemChangeListener(d, QQuickItemPrivate::Destroyed);
         QQuickItemPrivate::get(oldItem)->removeItemChangeListener(d, QQuickItemPrivate::Geometry);
     }
     if (newItem) {
         QQuickItemPrivate::get(newItem)->addItemChangeListener(d, QQuickItemPrivate::Children);
+        QQuickItemPrivate::get(newItem)->addItemChangeListener(d, QQuickItemPrivate::Destroyed);
         QQuickItemPrivate::get(newItem)->updateOrAddGeometryChangeListener(d, QQuickGeometryChange::Width);
     }
 
