@@ -3,11 +3,11 @@
 
 /*---
 esid: sec-array.prototype.filter
-es5id: 15.4.4.20-4-3
-description: Array.prototype.filter throws TypeError if callbackfn is null
+es5id: 15.4.4.20-1-1
+description: Array.prototype.filter applied to undefined throws a TypeError
 ---*/
 
-var arr = new Array(10);
+
 assert.throws(TypeError, function() {
-  arr.filter(null);
+  Array.prototype.filter.call(undefined); // TypeError is thrown if value is undefined
 });
