@@ -3,23 +3,22 @@
 
 /*---
 esid: sec-array.prototype.foreach
-es5id: 15.4.4.18-3-5
-description: >
-    Array.prototype.forEach - value of 'length' is a number (value is
-    -0)
+es5id: 15.4.4.18-1-9
+description: Array.prototype.forEach applied to Function object
 ---*/
 
-var accessed = false;
+var result = false;
 
 function callbackfn(val, idx, obj) {
-  accessed = true;
+  result = obj instanceof Function;
 }
 
-var obj = {
-  0: 11,
-  length: -0
+var obj = function(a, b) {
+  return a + b;
 };
+obj[0] = 11;
+obj[1] = 9;
 
 Array.prototype.forEach.call(obj, callbackfn);
 
-assert.sameValue(accessed, false, 'accessed');
+assert(result, 'result !== true');
