@@ -2,24 +2,25 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.3.5.4_2-85gs
+es5id: 15.3.5.4_2-69gs
 description: >
     Strict mode - checking access to strict function caller from
-    non-strict function (non-strict function declaration called by
-    strict Function.prototype.call(null))
+    non-strict function (strict function declaration called by
+    Function.prototype.call(globalObject))
 flags: [noStrict]
-features: [caller]
 ---*/
 
-function f() {
-  return gNonStrict();
-};
-(function() {
-  "use strict";
-  f.call(null);
-})();
+var global = this;
 
+function f() {
+  "use strict";
+  gNonStrict();
+};
+
+assert.throws(TypeError, function() {
+  f.call(this);
+});
 
 function gNonStrict() {
-  return gNonStrict.caller;
+  return gNonStrict.caller || gNonStrict.caller.throwTypeError;
 }
