@@ -2,12 +2,10 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.12.1.1-0-3
-description: >
-    <FF> is not valid JSON whitespace as specified by the production
-    JSONWhitespace.
+es5id: 15.12.1.1-0-1
+description: The JSON lexical grammar treats whitespace as a token seperator
 ---*/
 
 assert.throws(SyntaxError, function() {
-  JSON.parse('\u000c1234'); // should produce a syntax error 
+  JSON.parse('12\t\r\n 34'); // should produce a syntax error as whitespace results in two tokens
 });
