@@ -2,10 +2,10 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.12.3-6-b-1
+es5id: 15.12.3-8-a-5
 description: >
-    JSON.stringify treats numeric space arguments less than 1
-    (0.999999)the same as empty string space argument.
+    JSON.stringify treats non-Number or String object space arguments
+    the same as a missing space argument.
 ---*/
 
 var obj = {
@@ -19,5 +19,4 @@ var obj = {
   a2: 'a2'
 };
 
-/* empty string should be same as no space arg */
-assert.sameValue(JSON.stringify(obj, null, 0.999999), JSON.stringify(obj));
+assert.sameValue(JSON.stringify(obj), JSON.stringify(obj, null, obj), 'JSON.stringify(obj)');
