@@ -2,10 +2,10 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.2-2-13
+es5id: 15.2.3.2-2-14
 description: >
     Object.getPrototypeOf returns the [[Prototype]] of its parameter
-    (RangeError)
+    (ReferenceError)
 ---*/
 
-assert.sameValue(Object.getPrototypeOf(RangeError), Error, 'Object.getPrototypeOf(RangeError)');
+assert.sameValue(Object.getPrototypeOf(ReferenceError), Error, 'Object.getPrototypeOf(ReferenceError)');
