@@ -2,12 +2,14 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 7.8.4-17-s
-description: An OctalEscapeSequence is not allowed in a String under Strict Mode
+es5id: 7.8.4-33-s
+description: >
+    Three OctalEscapeSequences in a String are not allowed in a String
+    under Strict Mode
 flags: [onlyStrict]
 ---*/
 
 
 assert.throws(SyntaxError, function() {
-    eval('var x = "\\411";');
+    eval('var x = "\\1\\2\\7";');
 });
