@@ -5,14 +5,12 @@
 #include <QtQml/qqml.h>
 #include <QDebug>
 
-class MyPluginType : public QObject
+class MyPluginTypeNested : public QObject
 {
     Q_OBJECT
     Q_PROPERTY(QString value READ value)
 
 public:
-    MyPluginType(QObject *parent=nullptr) : QObject(parent) {}
-
     QString value() const { return "Hello"; }
 };
 
@@ -22,29 +20,26 @@ class MyNestedPluginType : public QObject
     Q_PROPERTY(QString value READ value)
 
 public:
-    MyNestedPluginType(QObject *parent=nullptr) : QObject(parent) {}
-
     QString value() const { return "Goodbye"; }
 };
 
-
-class MyPlugin : public QQmlExtensionPlugin
+class MyPluginNested : public QQmlExtensionPlugin
 {
     Q_OBJECT
     Q_PLUGIN_METADATA(IID QQmlExtensionInterface_iid)
 
 public:
-    MyPlugin() {}
-
     void registerTypes(const char *uri) override
     {
         Q_ASSERT(QLatin1String(uri) == "org.qtproject.AutoTestQmlNestedPluginType");
-        qmlRegisterType<MyPluginType>(uri, 1, 0, "MyPluginType");
+        qmlRegisterType<MyPluginTypeNested>(uri, 1, 0, "MyPluginType");
+        qmlRegisterType<MyPluginTypeNested>(uri, 1, 0, "Conflict");
 
         QString nestedUri(uri);
         nestedUri += QLatin1String(".Nested");
 
         qmlRegisterType<MyNestedPluginType>(nestedUri.toLatin1().constData(), 1, 0, "MyNestedPluginType");
+        qmlRegisterType<MyNestedPluginType>(nestedUri.toLatin1().constData(), 1, 0, "Conflict");
     }
 };
 
