@@ -1,9 +1,11 @@
 // Copyright (C) 2021 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
+#include <QtCore/qpointer.h>
 #include <qtest.h>
 #include <QSignalSpy>
 #include <QDebug>
+#include <QtGui/private/qeventpoint_p.h>
 #include <QtQml/QQmlEngine>
 #include <QtQml/QQmlComponent>
 #include <QtQuick/QQuickItem>
@@ -12,18 +14,15 @@
 #include <QtQuick/private/qquickrectangle_p.h>
 #include <QtQuick/private/qquickflickable_p.h>
 #include <QtQuick/private/qquicklistview_p.h>
+#include <QtQuick/private/qquickmousearea_p.h>
 #include <QtQuick/private/qquickpointhandler_p.h>
 #include <QtQuick/private/qquickshadereffectsource_p.h>
 #include <QtQuick/private/qquicktaphandler_p.h>
 #include <QtQuick/private/qquickwindow_p.h>
+#include <QtQuickTest/quicktest.h>
 #include <QtQuickTestUtils/private/qmlutils_p.h>
 #include <QtQuickTestUtils/private/visualtestutils_p.h>
 #include <QtQuickTestUtils/private/viewtestutils_p.h>
-#include <QtQuick/private/qquickmousearea_p.h>
-
-#include <QtGui/private/qeventpoint_p.h>
-
-#include <QtCore/qpointer.h>
 
 Q_LOGGING_CATEGORY(lcTests, "qt.quick.tests")
 
@@ -49,6 +48,7 @@ struct HoverItem : public QQuickItem
     {
         hoverEnter = true;
         e->setAccepted(block);
+        globalHoverPosition = e->globalPosition();
     }
 
     void hoverLeaveEvent(QHoverEvent *e) override
@@ -57,9 +57,16 @@ struct HoverItem : public QQuickItem
         e->setAccepted(block);
     }
 
+    void mousePressEvent(QMouseEvent *e) override
+    {
+        globalMousePosition = e->globalPosition();
+    }
+
     bool hoverEnter = false;
     bool hoverLeave = false;
     bool block = false;
+    QPointF globalHoverPosition;
+    QPointF globalMousePosition;
 };
 
 // A QQuick3DViewport simulator
@@ -146,6 +153,7 @@ private slots:
     void clearItemsOnHoverLeave();
     void deleteTargetOnPress();
     void compoundControlsFocusInSubscene();
+    void hoverEventGlobalPosition();
 
 private:
     std::unique_ptr<QPointingDevice> touchscreen{QTest::createTouchDevice()};
@@ -660,22 +668,47 @@ void tst_qquickdeliveryagent::compoundControlsFocusInSubscene()
     auto clickPos = spinboxFocusScope->boundingRect().translated(0, spinboxFocusScope->height() + 20).center().toPoint();
     QTest::mouseClick(&window, Qt::LeftButton, Qt::NoModifier, clickPos);
 
-    QVERIFY(textField->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(textField);
     QVERIFY(textField->hasFocus());
-    QTRY_VERIFY(spinbox->hasActiveFocus());
+    QTRY_VERIFY_ACTIVE_FOCUS(spinbox);
     QVERIFY(spinbox->hasFocus());
     QCOMPARE(spinbox->scopedFocusItem(), textField);
-    QVERIFY(spinboxFocusScope->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(spinboxFocusScope);
     QVERIFY(spinboxFocusScope->hasFocus());
     QCOMPARE(spinboxFocusScope->scopedFocusItem(), spinbox);
 
     QQuickDeliveryAgentPrivate *daPriv = static_cast<QQuickDeliveryAgentPrivate *>(QQuickDeliveryAgentPrivate::get(subscene.deliveryAgent));
-    QVERIFY(daPriv->rootItem->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(daPriv->rootItem);
     QCOMPARE(daPriv->activeFocusItem, textField);
     QCOMPARE(QQuickWindowPrivate::get(&window)->deliveryAgentPrivate()->activeFocusItem, textField);
     QCOMPARE(QQuickWindowPrivate::get(&window)->deliveryAgentPrivate()->rootItem->scopedFocusItem(), spinboxFocusScope);
 }
 
+void tst_qquickdeliveryagent::hoverEventGlobalPosition()
+{
+    QQuickWindow window;
+    window.resize(200, 200);
+    window.show();
+    QVERIFY(QTest::qWaitForWindowActive(&window));
+
+    HoverItem child(window.contentItem());
+    child.setAcceptHoverEvents(true);
+    child.setAcceptedMouseButtons(Qt::LeftButton);
+    child.setX(50);
+    child.setY(50);
+    child.setWidth(100);
+    child.setHeight(100);
+
+    QTest::mouseMove(&window, QPoint(25, 25));
+    QCOMPARE(child.hoverEnter, false);
+
+    QPoint point(100, 100);
+    QTest::mouseMove(&window, point);
+    QCOMPARE(child.hoverEnter, true);
+    QTest::mousePress(&window, Qt::LeftButton, {}, point);
+    QCOMPARE(child.globalHoverPosition, child.globalMousePosition);
+}
+
 QTEST_MAIN(tst_qquickdeliveryagent)
 
 #include "tst_qquickdeliveryagent.moc"
