@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <qtest.h>
 
@@ -41,6 +16,8 @@
 #include <private/qopenglcontext_p.h>
 #endif
 
+#include <QtGui/qrawfont.h>
+
 #include <private/qsgcontext_p.h>
 #include <private/qsgrenderloop_p.h>
 #include <private/qsgrhisupport_p.h>
@@ -52,6 +29,7 @@
 #include <QtGui/private/qguiapplication_p.h>
 #include <QtGui/qpa/qplatformintegration.h>
 
+using namespace Qt::StringLiterals;
 using namespace QQuickVisualTestUtils;
 
 class PerPixelRect : public QQuickItem
@@ -121,6 +99,9 @@ private slots:
     void createTextureFromImage();
     void withAdoptedRhi();
     void resizeTextureFromImage();
+    void textureNativeInterface();
+    void distanceFieldCacheInvalidation();
+    void unexposeDuringPolish();
 
 private:
     QQuickView *createView(const QString &file, QWindow *parent = nullptr, int x = -1, int y = -1, int w = -1, int h = -1);
@@ -213,9 +194,11 @@ public:
 
 void tst_SceneGraph::manyWindows()
 {
-    if ((QGuiApplication::platformName() == QLatin1String("offscreen"))
-        || (QGuiApplication::platformName() == QLatin1String("minimal")))
-        QSKIP("Skipping due to grabWindow not functional on offscreen/minimal platforms");
+    SKIP_IF_NO_WINDOW_GRAB;
+    if (QGuiApplication::platformName() == QLatin1String("offscreen")) {
+        QSKIP("Skipping due to createPlatformOpenGLContext returning nullptr with "
+            "offscreen platform plugin on non-x11 platforms like QNX");
+    }
 
     QFETCH(QString, file);
     QFETCH(bool, toplevel);
@@ -314,7 +297,9 @@ struct Sample {
     }
 
     bool check(const QImage &image, qreal scale) {
-        QColor color(image.pixel(x * scale, y * scale));
+        const int scaledX = qRound(x * scale);
+        const int scaledY = qRound(y * scale);
+        const QColor color(image.pixel(scaledX, scaledY));
         return qAbs(color.redF() - r) <= tolerance
                 && qAbs(color.greenF() - g) <= tolerance
                 && qAbs(color.blueF() - b) <= tolerance;
@@ -377,34 +362,35 @@ void tst_SceneGraph::render_data()
     QTest::addColumn<QList<Sample> >("baseStage");
     QTest::addColumn<QList<Sample> >("finalStage");
 
-    QList<QString> files;
-    files << "render_DrawSets.qml"
-          << "render_Overlap.qml"
-          << "render_MovingOverlap.qml"
-          << "render_BreakOpacityBatch.qml"
-          << "render_OutOfFloatRange.qml"
-          << "render_StackingOrder.qml"
-          << "render_ImageFiltering.qml"
-          << "render_bug37422.qml"
-          << "render_OpacityThroughBatchRoot.qml"
-          << "render_Mipmap.qml"
-          << "render_AlphaOverlapRebuild.qml";
+    const QString files[] = {
+        u"render_DrawSets.qml"_s,
+        u"render_Overlap.qml"_s,
+        u"render_MovingOverlap.qml"_s,
+        u"render_BreakOpacityBatch.qml"_s,
+        u"render_OutOfFloatRange.qml"_s,
+        u"render_StackingOrder.qml"_s,
+        u"render_ImageFiltering.qml"_s,
+        u"render_bug37422.qml"_s,
+        u"render_OpacityThroughBatchRoot.qml"_s,
+        u"render_Mipmap.qml"_s,
+        u"render_AlphaOverlapRebuild.qml"_s,
+    };
 
     QRegularExpression sampleCount("#samples: *(\\d+)");
     //                          X:int   Y:int   R:float       G:float       B:float       Error:float
     QRegularExpression baseSamples("#base: *(\\d+) *(\\d+) *(\\d\\.\\d+) *(\\d\\.\\d+) *(\\d\\.\\d+) *(\\d\\.\\d+)");
     QRegularExpression finalSamples("#final: *(\\d+) *(\\d+) *(\\d\\.\\d+) *(\\d\\.\\d+) *(\\d\\.\\d+) *(\\d\\.\\d+)");
 
-    foreach (QString fileName, files) {
+    for (const QString &fileName : files) {
         QFile file(testFile(fileName));
         if (!file.open(QFile::ReadOnly)) {
             qFatal("render_data: QFile::open failed! file=%s, error=%s",
                    qPrintable(fileName), qPrintable(file.errorString()));
         }
-        QStringList contents = QString::fromLatin1(file.readAll()).split(QLatin1Char('\n'));
+        const QStringList contents = QString::fromLatin1(file.readAll()).split(QLatin1Char('\n'));
 
         int samples = -1;
-        foreach (QString line, contents) {
+        for (const QString &line : contents) {
             auto match = sampleCount.match(line);
             if (match.hasMatch()) {
                 samples = match.captured(1).toInt();
@@ -415,7 +401,7 @@ void tst_SceneGraph::render_data()
             qFatal("render_data: failed to find string '#samples: [count], file=%s", qPrintable(fileName));
 
         QList<Sample> baseStage, finalStage;
-        foreach (QString line, contents) {
+        for (const QString &line : contents) {
             auto match = baseSamples.match(line);
             if (match.hasMatch())
                 baseStage << sample_from_regexp(&match);
@@ -455,6 +441,10 @@ void tst_SceneGraph::render()
     // ideal world.
     // Just keep this in mind when writing tests.
     qreal scale = view.devicePixelRatio();
+    const bool isIntegerScale = qFuzzyIsNull(qreal(qFloor(scale)) - scale);
+
+    if (file == "render_OutOfFloatRange.qml" && !isIntegerScale)
+        QSKIP("render_OutOfFloatRange doesn't work with non-integer scaling factors");
 
     // Grab the window and check all our base stage samples
     QImage content = view.grabWindow();
@@ -725,7 +715,7 @@ void tst_SceneGraph::withAdoptedRhi()
                                     readResult.pixelSize.width(), readResult.pixelSize.height(),
                                     QImage::Format_RGBA8888_Premultiplied);
                 if (rhi->isYUpInFramebuffer())
-                    result = wrapperImage.mirrored();
+                    result = wrapperImage.flipped();
                 else
                     result = wrapperImage.copy();
             };
@@ -802,6 +792,59 @@ void tst_SceneGraph::resizeTextureFromImage()
     TestOffscreenScene::cleanup();
 }
 
+void tst_SceneGraph::textureNativeInterface()
+{
+    if (!isRunningOnRhi())
+        QSKIP("Skipping texture native interface tests due to not running with QRhi");
+
+    // test it offscreen because we want to do QSG stuff here on the main thread
+    QScopedPointer<TestOffscreenScene> scene(createOffscreenScene(testFileUrl(QLatin1String("renderControl_rect.qml"))));
+    QVERIFY(scene->renderControl && scene->window && scene->rootItem);
+
+    QImage image(512, 512, QImage::Format_RGBA8888);
+
+    QScopedPointer<QSGTexture> texture(scene->window->createTextureFromImage(image, QQuickWindow::TextureHasAlphaChannel));
+    QVERIFY(texture.data());
+
+    commitTexture(scene->window->rhi(), texture.data());
+
+#if QT_CONFIG(metal)
+    if (scene->window->graphicsApi() == QSGRendererInterface::Metal) {
+        auto texNatIf = texture->nativeInterface<QNativeInterface::QSGMetalTexture>();
+        QVERIFY(texNatIf);
+        QVERIFY(texNatIf->nativeTexture());
+    }
+#endif
+
+#if defined(Q_OS_WIN)
+    if (scene->window->graphicsApi() == QSGRendererInterface::Direct3D11) {
+        auto texNatIf = texture->nativeInterface<QNativeInterface::QSGD3D11Texture>();
+        QVERIFY(texNatIf);
+        QVERIFY(texNatIf->nativeTexture());
+    } else if (scene->window->graphicsApi() == QSGRendererInterface::Direct3D12) {
+        auto texNatIf = texture->nativeInterface<QNativeInterface::QSGD3D12Texture>();
+        QVERIFY(texNatIf);
+        QVERIFY(texNatIf->nativeTexture());
+    }
+#endif
+
+#if QT_CONFIG(opengl)
+    if (scene->window->graphicsApi() == QSGRendererInterface::OpenGL) {
+        auto texNatIf = texture->nativeInterface<QNativeInterface::QSGOpenGLTexture>();
+        QVERIFY(texNatIf);
+        QVERIFY(texNatIf->nativeTexture());
+    }
+#endif
+
+#if QT_CONFIG(vulkan)
+    if (scene->window->graphicsApi() == QSGRendererInterface::Vulkan) {
+        auto texNatIf = texture->nativeInterface<QNativeInterface::QSGVulkanTexture>();
+        QVERIFY(texNatIf);
+        QVERIFY(texNatIf->nativeImage());
+    }
+#endif
+}
+
 bool tst_SceneGraph::isRunningOnRhi()
 {
     static bool retval = false;
@@ -821,6 +864,99 @@ bool tst_SceneGraph::isRunningOnRhi()
     return retval;
 }
 
+void tst_SceneGraph::distanceFieldCacheInvalidation()
+{
+    if (!isRunningOnRhi())
+        QSKIP("Skipping complex rendering tests due to not running with QRhi");
+
+    QFont font(QStringLiteral("Arial"));
+    QRawFont rawFont = QRawFont::fromFont(font);
+
+    QQuickView view;
+    view.setSource(testFileUrl(QLatin1String("distanceFieldCacheInvalidation.qml")));
+    view.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&view));
+
+    {
+        QSGRenderContext *rc = QQuickItemPrivate::get(view.rootObject())->sceneGraphRenderContext();
+
+        QSGDistanceFieldGlyphCache *cache = rc->distanceFieldGlyphCache(rawFont, 1);
+        QVERIFY(cache != nullptr);
+
+        auto glyphIndexes = rawFont.glyphIndexesForString(QStringLiteral("a b"));
+        cache->populate(glyphIndexes);
+        QVERIFY(cache->isActive());
+
+        cache->release(glyphIndexes);
+        QVERIFY(!cache->isActive());
+    }
+}
+
+class NotificationItem : public QQuickItem
+{
+    Q_OBJECT
+    Q_PROPERTY(
+            QRectF contentsRect READ contentsRect WRITE setContentsRect NOTIFY contentsRectChanged)
+
+public:
+    explicit NotificationItem(QQuickItem *parent = nullptr) : QQuickItem(parent) { }
+
+    QRectF contentsRect() const { return m_contentsRect; }
+
+    void setContentsRect(const QRectF &contentsRect)
+    {
+        if (m_contentsRect != contentsRect) {
+            m_contentsRect = contentsRect;
+            polish();
+            Q_EMIT contentsRectChanged();
+        }
+    }
+
+Q_SIGNALS:
+    void contentsRectChanged();
+
+protected:
+    void updatePolish() override
+    {
+        if (m_contentsRect.isEmpty()) {
+            window()->setVisible(false);
+        }
+    }
+
+private:
+    QRectF m_contentsRect = QRectF(0, 0, 100, 100);
+};
+
+static bool waitForWindowUnexposed(QWindow *window, int timeout = 5000)
+{
+    return QTest::qWaitFor([window]() { return !window->isExposed(); }, timeout);
+}
+
+void tst_SceneGraph::unexposeDuringPolish()
+{
+    qmlRegisterType<NotificationItem>("SceneGraphTest", 1, 0, "Notification");
+
+    // This test verifies that the window will not be painted if it's unexposed while polishing
+    // items. It's relevant with QPAs such as QtWayland.
+
+    QQuickView view;
+    view.setSource(testFileUrl(QLatin1String("unexposeDuringPolish.qml")));
+    view.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&view));
+
+    connect(&view, &QQuickWindow::afterAnimating, this,
+            [this, &view]() { QVERIFY(view.isExposed()); });
+    connect(&view, &QQuickWindow::beforeSynchronizing, this,
+            [this, &view]() { QVERIFY(view.isExposed()); });
+
+    QQuickItem *rootItem = view.rootObject();
+    rootItem->setProperty("contentsRect", QRectF());
+
+    QVERIFY(waitForWindowUnexposed(&view));
+    QVERIFY(!view.isExposed());
+    QVERIFY(!view.isVisible());
+}
+
 #include "tst_scenegraph.moc"
 
 QTEST_MAIN(tst_SceneGraph)
