@@ -4,7 +4,8 @@
 #include <QtGui/qpa/qplatformintegration.h>
 #include <QtGui/qpa/qplatformtheme.h>
 #include <QtGui/private/qguiapplication_p.h>
-#include <QtTest>
+#include <QTest>
+#include <QSignalSpy>
 #include <QtQml>
 #include <QtQuickTestUtils/private/qmlutils_p.h>
 #include <QtQuickTestUtils/private/visualtestutils_p.h>
@@ -17,6 +18,7 @@
 #include <QtQuickTemplates2/private/qquickmenubaritem_p.h>
 #include <QtQuickTemplates2/private/qquickmenuitem_p.h>
 #include <QtQuickTemplates2/private/qquickpopupwindow_p_p.h>
+#include <QtQuickTest/quicktest.h>
 #include <QtQuickControlsTestUtils/private/controlstestutils_p.h>
 #include <QtQuickControlsTestUtils/private/qtest_quickcontrols_p.h>
 
@@ -79,10 +81,10 @@ private slots:
     void invalidDelegate();
     void panMenuBar_data();
     void panMenuBar();
+    void clearMenus();
 
 private:
     bool nativeMenuBarSupported = false;
-    bool popupWindowsSupported = false;
     QScopedPointer<QPointingDevice> touchScreen = QScopedPointer<QPointingDevice>(QTest::createTouchDevice());
 };
 
@@ -97,9 +99,6 @@ tst_qquickmenubar::tst_qquickmenubar()
     qputenv("QML_NO_TOUCH_COMPRESSION", "1");
     QQuickMenuBar mb;
     nativeMenuBarSupported = QQuickMenuBarPrivate::get(&mb)->useNativeMenuBar();
-#if defined(Q_OS_WINDOWS) || defined(Q_OS_MACOS)
-    popupWindowsSupported = QGuiApplicationPrivate::platformIntegration()->hasCapability(QPlatformIntegration::Capability::MultipleWindows);
-#endif
 }
 
 void tst_qquickmenubar::init()
@@ -137,22 +136,19 @@ void tst_qquickmenubar::mouse_data()
     QTest::addColumn<QQuickPopup::PopupType>("popupType");
     QTest::newRow("Popup.Item") << QQuickPopup::Item;
     // Uncomment when popup windows work 100% (QTBUG-128479)
-    // if (popupWindowsSupported)
+    // if (arePopupWindowsSupported())
     //     QTest::newRow("Popup.Window") << QQuickPopup::Window;
 }
 
 void tst_qquickmenubar::mouse()
 {
+    SKIP_IF_NO_WINDOW_ACTIVATION;
+    SKIP_IF_NO_MOUSE_HOVER;
+
     QFETCH(QQuickPopup::PopupType, popupType);
     QCoreApplication::setAttribute(Qt::AA_DontUseNativeMenuBar);
     QCoreApplication::setAttribute(Qt::AA_DontUseNativeMenuWindows);
 
-    SKIP_IF_NO_WINDOW_ACTIVATION
-
-    if ((QGuiApplication::platformName() == QLatin1String("offscreen"))
-        || (QGuiApplication::platformName() == QLatin1String("minimal")))
-        QSKIP("Mouse highlight not functional on offscreen/minimal platforms");
-
     QQmlApplicationEngine engine(testFileUrl("menubaritems.qml"));
 
     QScopedPointer<QQuickApplicationWindow> window(qobject_cast<QQuickApplicationWindow *>(engine.rootObjects().value(0)));
@@ -210,7 +206,7 @@ void tst_qquickmenubar::mouse()
     // re-trigger a menubar item to hide the menu - it should close on press
     QTest::mousePress(window.data(), Qt::LeftButton, Qt::NoModifier, itemSceneCenter(editMenuBarItem));
     QVERIFY(editMenuBarItem->isHighlighted());
-    QVERIFY(editMenuBarItem->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(editMenuBarItem);
     QTRY_VERIFY(!editMenuBarMenu->isVisible());
     QTest::mouseRelease(window.data(), Qt::LeftButton, Qt::NoModifier, itemSceneCenter(editMenuBarItem));
 
@@ -366,7 +362,7 @@ void tst_qquickmenubar::keys_data()
     QTest::addColumn<QQuickPopup::PopupType>("popupType");
     QTest::newRow("Popup.Item") << QQuickPopup::Item;
     // Uncomment when popup windows work 100% (QTBUG-128479)
-    // if (popupWindowsSupported)
+    // if (arePopupWindowsSupported())
     //     QTest::newRow("Popup.Window") << QQuickPopup::Window;
 }
 
@@ -376,7 +372,7 @@ void tst_qquickmenubar::keys()
     QCoreApplication::setAttribute(Qt::AA_DontUseNativeMenuBar);
     QCoreApplication::setAttribute(Qt::AA_DontUseNativeMenuWindows);
 
-    SKIP_IF_NO_WINDOW_ACTIVATION
+    SKIP_IF_NO_WINDOW_ACTIVATION;
 
     QQmlApplicationEngine engine(testFileUrl("menubaritems.qml"));
 
@@ -411,7 +407,7 @@ void tst_qquickmenubar::keys()
     QVERIFY(editMenuBarItem->isHighlighted());
     QVERIFY(editMenuBarMenu->isVisible());
     QTRY_VERIFY(editMenuBarMenu->isOpened());
-    QVERIFY(editMenuBarMenu->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(editMenuBarMenu);
 
     // navigate down to the menu
     QQuickMenuItem *cutMenuItem = qobject_cast<QQuickMenuItem *>(editMenuBarMenu->itemAt(0));
@@ -420,13 +416,13 @@ void tst_qquickmenubar::keys()
     QVERIFY(!cutMenuItem->hasActiveFocus());
     QTest::keyClick(window.data(), Qt::Key_Down);
     QVERIFY(cutMenuItem->isHighlighted());
-    QVERIFY(cutMenuItem->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(cutMenuItem);
 
     // navigate up, back to the menubar
     QTest::keyClick(window.data(), Qt::Key_Up);
-    QVERIFY(editMenuBarItem->isHighlighted());
-    QVERIFY(editMenuBarItem->hasActiveFocus());
     QTRY_VERIFY(!editMenuBarMenu->isVisible());
+    QVERIFY(editMenuBarItem->isHighlighted());
+    QVERIFY_ACTIVE_FOCUS(editMenuBarItem);
 
 // There seem to be problems in focus handling in webOS QPA, see https://bugreports.qt.io/browse/WEBOSCI-45
 #ifdef Q_OS_WEBOS
@@ -441,9 +437,9 @@ void tst_qquickmenubar::keys()
     QVERIFY(!editMenuBarItem->hasActiveFocus());
     QVERIFY(editMenuBarMenu->isVisible());
     QTRY_VERIFY(editMenuBarMenu->isOpened());
-    QVERIFY(editMenuBarMenu->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(editMenuBarMenu);
     QVERIFY(cutMenuItem->isHighlighted());
-    QVERIFY(cutMenuItem->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(cutMenuItem);
 
     // navigate left in popup mode (menu open)
     QTest::keyClick(window.data(), Qt::Key_Left);
@@ -487,7 +483,7 @@ void tst_qquickmenubar::keys()
     QVERIFY(viewMenuBarMenu->isVisible());
     QTRY_VERIFY(viewMenuBarMenu->isOpened());
     QVERIFY(!viewMenuBarItem->hasActiveFocus());
-    QVERIFY(viewMenuBarMenu->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(viewMenuBarMenu);
 
     // open a sub-menu
     QQuickMenuItem *alignmentSubMenuItem = qobject_cast<QQuickMenuItem *>(viewMenuBarMenu->itemAt(0));
@@ -573,7 +569,7 @@ void tst_qquickmenubar::mnemonics_data()
     QTest::addColumn<QQuickPopup::PopupType>("popupType");
     QTest::newRow("Popup.Item") << QQuickPopup::Item;
     // Uncomment when popup windows work 100% (QTBUG-128479)
-    // if (popupWindowsSupported)
+    // if (arePopupWindowsSupported())
     //     QTest::newRow("Popup.Window") << QQuickPopup::Window;
 }
 
@@ -583,7 +579,7 @@ void tst_qquickmenubar::mnemonics()
     QCoreApplication::setAttribute(Qt::AA_DontUseNativeMenuBar);
     QCoreApplication::setAttribute(Qt::AA_DontUseNativeMenuWindows);
 
-    SKIP_IF_NO_WINDOW_ACTIVATION
+    SKIP_IF_NO_WINDOW_ACTIVATION;
 
 #if defined(Q_OS_MACOS) or defined(Q_OS_WEBOS)
     QSKIP("Mnemonics are not used on this platform");
@@ -632,7 +628,7 @@ void tst_qquickmenubar::mnemonics()
     QVERIFY(!editMenuBarItem->hasActiveFocus());
     QVERIFY(editMenuBarMenu->isVisible());
     QTRY_VERIFY(editMenuBarMenu->isOpened());
-    QVERIFY(editMenuBarMenu->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(editMenuBarMenu);
 
     // press Alt to hide the menu
     keySim.click(Qt::Key_Alt);
@@ -648,7 +644,7 @@ void tst_qquickmenubar::mnemonics()
     QVERIFY(editMenuBarItem->isHighlighted());
     QVERIFY(editMenuBarMenu->isVisible());
     QTRY_VERIFY(editMenuBarMenu->isOpened());
-    QVERIFY(editMenuBarMenu->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(editMenuBarMenu);
     QVERIFY(!editMenuBarItem->hasActiveFocus());
 
     // trigger another menubar item to open another menu, leave Alt pressed
@@ -718,7 +714,7 @@ void tst_qquickmenubar::mnemonics()
     QVERIFY(fileMenuBarItem->isHighlighted());
     QVERIFY(fileMenuBarMenu->isVisible());
     QTRY_VERIFY(fileMenuBarMenu->isOpened());
-    QVERIFY(fileMenuBarMenu->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(fileMenuBarMenu);
 
     // trigger a menu item to close the menu, which shouldn't trigger a button
     // action behind the menu (QTBUG-86276)
@@ -764,7 +760,7 @@ void tst_qquickmenubar::altNavigation()
     // releasing Alt with AltModifier removes the AltModifier.
     QTest::keyPress(window.get(), Qt::Key_Alt);
     QTest::keyRelease(window.get(), Qt::Key_Alt, Qt::AltModifier);
-    QVERIFY(menuBar->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(menuBar);
     QVERIFY(fileMenuBarItem->isHighlighted());
 
     // if menu has focus, pressing the mnemonic without Alt should open the menu
@@ -775,7 +771,7 @@ void tst_qquickmenubar::altNavigation()
     QVERIFY(editMenuBarItem->isHighlighted());
     QVERIFY(editMenuBarMenu->isVisible());
     QTRY_VERIFY(editMenuBarMenu->isOpened());
-    QVERIFY(editMenuBarMenu->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(editMenuBarMenu);
 }
 
 void tst_qquickmenubar::addRemove_data()
@@ -1101,7 +1097,7 @@ void tst_qquickmenubar::addRemoveExistingMenus_data()
     if (nativeMenuBarSupported)
         QTest::newRow("native, native") << true << true;
     // Uncomment when popup windows work 100%
-    // if (popupWindowsSupported)
+    // if (arePopupWindowsSupported())
     //     QTest::newRow("non-native, popup window") << false << true;
 }
 
@@ -1138,19 +1134,18 @@ void tst_qquickmenubar::checkHighlightWhenMenuDismissed_data()
     QTest::addColumn<QQuickPopup::PopupType>("popupType");
     QTest::newRow("Popup.Item") << QQuickPopup::Item;
     // Uncomment when popup windows work 100% (QTBUG-128479)
-    // if (popupWindowsSupported)
+    // if (arePopupWindowsSupported())
     //     QTest::newRow("Popup.Window") << QQuickPopup::Window;
 }
 
 void tst_qquickmenubar::checkHighlightWhenMenuDismissed()
 {
+    SKIP_IF_NO_MOUSE_HOVER;
+
     QFETCH(QQuickPopup::PopupType, popupType);
 
     QCoreApplication::setAttribute(Qt::AA_DontUseNativeMenuBar);
     QCoreApplication::setAttribute(Qt::AA_DontUseNativeMenuWindows);
-    if ((QGuiApplication::platformName() == QLatin1String("offscreen"))
-        || (QGuiApplication::platformName() == QLatin1String("minimal")))
-        QSKIP("Mouse highlight not functional on offscreen/minimal platforms");
 
     QQmlApplicationEngine engine(testFileUrl("checkHighlightWhenDismissed.qml"));
     QScopedPointer<QQuickApplicationWindow> window(qobject_cast<QQuickApplicationWindow *>(engine.rootObjects().value(0)));
@@ -1211,17 +1206,16 @@ void tst_qquickmenubar::hoverAfterClosingWithEscape_data()
     QTest::addColumn<bool>("usePopupWindow");
     QTest::newRow("in-scene popup") << false;
     // Uncomment when popup windows work 100%
-    // if (popupWindowsSupported)
+    // if (arePopupWindowsSupported())
     //     QTest::newRow("popup window") << true;
 }
 
 void tst_qquickmenubar::hoverAfterClosingWithEscape()
 {
+    SKIP_IF_NO_MOUSE_HOVER;
+
     QCoreApplication::setAttribute(Qt::AA_DontUseNativeMenuBar);
     QCoreApplication::setAttribute(Qt::AA_DontUseNativeMenuWindows);
-    if ((QGuiApplication::platformName() == QLatin1String("offscreen"))
-        || (QGuiApplication::platformName() == QLatin1String("minimal")))
-        QSKIP("Mouse highlight not functional on offscreen/minimal platforms");
 
     QQuickControlsApplicationHelper helper(this, QLatin1String("hoverAfterClosingWithEscape.qml"));
     QVERIFY2(helper.ready, helper.failureMessage());
@@ -1261,12 +1255,11 @@ void tst_qquickmenubar::closeByClickingOutside_data()
 
 void tst_qquickmenubar::closeByClickingOutside()
 {
+    SKIP_IF_NO_MOUSE_HOVER;
+
     QFETCH(QQuickPopup::PopupType, popupType);
 
     QCoreApplication::setAttribute(Qt::AA_DontUseNativeMenuBar);
-    if ((QGuiApplication::platformName() == QLatin1String("offscreen"))
-        || (QGuiApplication::platformName() == QLatin1String("minimal")))
-        QSKIP("Mouse highlight not functional on offscreen/minimal platforms");
 
     QQuickControlsApplicationHelper helper(this, QLatin1String("hoverAfterClosingWithEscape.qml"));
     QVERIFY2(helper.ready, helper.failureMessage());
@@ -1312,14 +1305,15 @@ void tst_qquickmenubar::AA_DontUseNativeMenuBar()
     QQuickMenuBar *menuBar = window->property("menuBar").value<QQuickMenuBar *>();
     QVERIFY(menuBar);
     auto menuBarPrivate = QQuickMenuBarPrivate::get(menuBar);
-    QQuickItem *contents = window->property("contents").value<QQuickItem *>();
+    QQuickItem *contents = window->contentItem();
     QVERIFY(contents);
 
     QVERIFY(!menuBarPrivate->nativeHandle());
     QVERIFY(menuBar->isVisible());
     QVERIFY(menuBar->count() > 0);
     QVERIFY(menuBar->height() > 0);
-    QCOMPARE(contents->height(), window->height() - menuBar->height());
+    auto bottomSafeMargin = window->safeAreaMargins().bottom();
+    QCOMPARE(contents->height(), window->height() - menuBar->height() - bottomSafeMargin);
 
     // If the menu bar is not native, the menus should not be native either.
     // The main reason for this limitation is that a native menu typically
@@ -1516,9 +1510,14 @@ void tst_qquickmenubar::applicationWindow()
     QQuickMenuBar *menuBar = window->property("menuBar").value<QQuickMenuBar *>();
     QVERIFY(menuBar);
     auto menuBarPrivate = QQuickMenuBarPrivate::get(menuBar);
-    QQuickItem *contents = window->property("contents").value<QQuickItem *>();
+    QQuickItem *contents = window->contentItem();
     QVERIFY(contents);
 
+    // The window may report safe area margins when invisible, but they will not
+    // propagate to the Quick SafeArea until shown.
+    auto topSafeMargin = initiallyVisible ? window->safeAreaMargins().top() : 0;
+    auto bottomSafeMargin = initiallyVisible ? window->safeAreaMargins().bottom() : 0;
+
     for (const bool visible : {initiallyVisible, !initiallyVisible, initiallyVisible}) {
         menuBar->setVisible(visible);
 
@@ -1528,14 +1527,14 @@ void tst_qquickmenubar::applicationWindow()
         if (!visible) {
             QVERIFY(!menuBar->isVisible());
             QVERIFY(!nativeMenuBarVisible);
-            QCOMPARE(contents->height(), window->height());
+            QCOMPARE(contents->height(), window->height() - topSafeMargin - bottomSafeMargin);
         } else if (nativeMenuBarVisible) {
             QVERIFY(menuBar->isVisible());
-            QCOMPARE(contents->height(), window->height());
+            QCOMPARE(contents->height(), window->height() - topSafeMargin - bottomSafeMargin);
         } else {
             QVERIFY(menuBar->isVisible());
             QVERIFY(menuBar->height() > 0);
-            QCOMPARE(contents->height(), window->height() - menuBar->height());
+            QCOMPARE(contents->height(), window->height() - menuBar->height() - bottomSafeMargin);
         }
     }
 }
@@ -1564,17 +1563,18 @@ void tst_qquickmenubar::menubarAsHeader()
     QQuickMenuBar *menuBar = window->property("header").value<QQuickMenuBar *>();
     QVERIFY(menuBar);
     auto menuBarPrivate = QQuickMenuBarPrivate::get(menuBar);
-    QQuickItem *contents = window->property("contents").value<QQuickItem *>();
+    QQuickItem *contents = window->contentItem();
     QVERIFY(contents);
     QVERIFY(menuBar->count() > 0);
     QCOMPARE(menuBarPrivate->nativeHandle() != nullptr, native);
 
     if (menuBarPrivate->nativeHandle()) {
         // Using native menubar
-        QCOMPARE(contents->height(), window->height());
+        QCOMPARE(contents->height(), window->height() - window->safeAreaMargins().top());
     } else {
         // Not using native menubar
-        QCOMPARE(contents->height(), window->height() - menuBar->height());
+        auto bottomSafeMargin = window->safeAreaMargins().bottom();
+        QCOMPARE(contents->height(), window->height() - menuBar->height() - bottomSafeMargin);
     }
 }
 
@@ -1582,7 +1582,7 @@ void tst_qquickmenubar::menuPosition_data()
 {
     QTest::addColumn<QQuickPopup::PopupType>("popupType");
     QTest::newRow("Popup.Item") << QQuickPopup::Item;
-    if (popupWindowsSupported)
+    if (arePopupWindowsSupported())
         QTest::newRow("Popup.Window") << QQuickPopup::Window;
 }
 
@@ -1611,12 +1611,13 @@ void tst_qquickmenubar::menuPosition()
     QVERIFY(window);
     QQuickMenuBar *menuBar = window->property("menuBar").value<QQuickMenuBar *>();
     QVERIFY(menuBar);
-
-    const QPoint requestedPos{50, 50};
-
     QQuickMenu *editMenu = menuBar->menuAt(1);
     QVERIFY(editMenu);
     QQuickMenuPrivate *editMenuPrivate = QQuickMenuPrivate::get(editMenu);
+
+    const QPoint requestedPos {50, 50};
+    const QPointF insetAdjustments {-editMenu->leftInset(), -editMenu->topInset()};
+
     editMenu->setPopupType(popupType);
     editMenu->setX(requestedPos.x());
     editMenu->setY(requestedPos.y());
@@ -1627,17 +1628,17 @@ void tst_qquickmenubar::menuPosition()
         QVERIFY(QTest::qWaitForWindowExposed(editMenuPrivate->popupWindow));
     }
 
-    QVERIFY(pixelsCloseEnough(editMenu->x(), requestedPos.x()));
-    QVERIFY(pixelsCloseEnough(editMenu->y(), requestedPos.y()));
+    static const QString errorString1("Expected %1, was %2");
+    QVERIFY2(pixelsCloseEnough(editMenu->x(), requestedPos.x()), qPrintable(errorString1.arg(requestedPos.x()).arg(editMenu->x())));
+    QVERIFY2(pixelsCloseEnough(editMenu->y(), requestedPos.y()), qPrintable(errorString1.arg(requestedPos.y()).arg(editMenu->y())));
 
     QQuickItem *background = editMenu->background();
     QVERIFY(background);
 
-    const QPoint bgPos = editMenu->parentItem()->mapFromGlobal(background->mapToGlobal({0, 0})).toPoint();
-    QVERIFY2(pixelsCloseEnough(requestedPos.x(), bgPos.x()),
-             "The background's x coordinate changed when mapped to the overlay's coordinate space.");
-    QVERIFY2(pixelsCloseEnough(requestedPos.y(), bgPos.y()),
-             "The background's y coordinate changed when mapped to the overlay's coordinate space.");
+    static const QString errorString2("The background's %1 coordinate changed when mapped to the overlay's coordinate space. %2");
+    const QPoint bgPos = editMenu->parentItem()->mapFromGlobal(background->mapToGlobal(insetAdjustments)).toPoint();
+    QVERIFY2(pixelsCloseEnough(requestedPos.x(), bgPos.x()), qPrintable(errorString2.arg("x").arg(errorString1.arg(requestedPos.x()).arg(bgPos.x()))));
+    QVERIFY2(pixelsCloseEnough(requestedPos.y(), bgPos.y()), qPrintable(errorString2.arg("y").arg(errorString1.arg(requestedPos.y()).arg(bgPos.y()))));
 }
 
 void tst_qquickmenubar::changeDelegate_data()
@@ -1820,7 +1821,7 @@ void tst_qquickmenubar::panMenuBar_data()
     QTest::addColumn<bool>("usePopupWindow");
     QTest::newRow("in-scene popup") << false;
     // Uncomment when popup windows work 100%
-    // if (popupWindowsSupported)
+    // if (arePopupWindowsSupported())
     //     QTest::newRow("popup window") << true;
 }
 
@@ -1871,6 +1872,16 @@ void tst_qquickmenubar::panMenuBar()
     QTRY_VERIFY(!menuBarItem0->menu()->isOpened());
 }
 
+void tst_qquickmenubar::clearMenus()
+{
+    QQmlEngine engine;
+    QQmlComponent c(&engine, testFileUrl("clearMenus.qml"));
+    QVERIFY2(c.isReady(), qPrintable(c.errorString()));
+    QScopedPointer<QObject> o(c.create());
+    QVERIFY(!o.isNull());
+    QTRY_COMPARE(o->property("v").toInt(), 2);
+}
+
 QTEST_QUICKCONTROLS_MAIN(tst_qquickmenubar)
 
 #include "tst_qquickmenubar.moc"
